/* Generated by Cython 0.27.3 */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_27_3"
#define CYTHON_FUTURE_DIVISION 0
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (0 && PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
  #undef SHIFT
  #undef BASE
  #undef MASK
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#if PY_VERSION_HEX < 0x030700A0 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject **args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject **args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : PyInstanceMethod_New(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif


#define __PYX_ERR(f_index, lineno, Ln_error) \
{ \
  __pyx_filename = __pyx_f[f_index]; __pyx_lineno = lineno; __pyx_clineno = __LINE__; goto Ln_error; \
}

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__ssw__sswpy
#define __PYX_HAVE_API__ssw__sswpy
#include <stdint.h>
#include <string.h>
#include <stdio.h>
#include "str_util.h"
#include "ssw.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT 0
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
#define __Pyx_PyBool_FromLong(b) ((b) ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False))
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c));
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "ssw/sswpy.pyx",
  "stringsource",
  "ssw/c_util.pxd",
  "type.pxd",
};

/*--- Type declarations ---*/
struct __pyx_obj_3ssw_5sswpy_SSW;

/* "ssw/sswpy.pyx":66
 * 
 * 
 * cdef class SSW:             # <<<<<<<<<<<<<<
 * 
 *     cdef int8_t* score_matrix
 */
struct __pyx_obj_3ssw_5sswpy_SSW {
  PyObject_HEAD
  struct __pyx_vtabstruct_3ssw_5sswpy_SSW *__pyx_vtab;
  int8_t *score_matrix;
  s_profile *profile;
  PyObject *read;
  int8_t *read_arr;
  Py_ssize_t read_length;
  PyObject *reference;
  int8_t *ref_arr;
  Py_ssize_t ref_length;
};



struct __pyx_vtabstruct_3ssw_5sswpy_SSW {
  int (*printResult_c)(struct __pyx_obj_3ssw_5sswpy_SSW *, s_align *, Py_ssize_t);
  s_align *(*align_c)(struct __pyx_obj_3ssw_5sswpy_SSW *, int, int, Py_ssize_t, int32_t);
  int (*buildDNAScoreMatrix)(struct __pyx_obj_3ssw_5sswpy_SSW *, uint8_t const , uint8_t const , int8_t *);
};
static struct __pyx_vtabstruct_3ssw_5sswpy_SSW *__pyx_vtabptr_3ssw_5sswpy_SSW;

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* None.proto */
static CYTHON_INLINE Py_ssize_t __Pyx_div_Py_ssize_t(Py_ssize_t, Py_ssize_t);

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* UnicodeAsUCS4.proto */
static CYTHON_INLINE Py_UCS4 __Pyx_PyUnicode_AsPy_UCS4(PyObject*);

/* object_ord.proto */
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyObject_Ord(c)\
    (likely(PyUnicode_Check(c)) ? (long)__Pyx_PyUnicode_AsPy_UCS4(c) : __Pyx__PyObject_Ord(c))
#else
#define __Pyx_PyObject_Ord(c) __Pyx__PyObject_Ord(c)
#endif
static long __Pyx__PyObject_Ord(PyObject* c);

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* SwapException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSwap(type, value, tb)  __Pyx__ExceptionSwap(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* None.proto */
static CYTHON_INLINE void __Pyx_RaiseUnboundLocalError(const char *varname);

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#endif

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* GetModuleGlobalName.proto */
static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name);

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely((Py_TYPE(obj) == type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* decode_c_string_utf16.proto */
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_DecodeUTF16(const char *s, Py_ssize_t size, const char *errors) {
    int byteorder = 0;
    return PyUnicode_DecodeUTF16(s, size, errors, &byteorder);
}
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_DecodeUTF16LE(const char *s, Py_ssize_t size, const char *errors) {
    int byteorder = -1;
    return PyUnicode_DecodeUTF16(s, size, errors, &byteorder);
}
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_DecodeUTF16BE(const char *s, Py_ssize_t size, const char *errors) {
    int byteorder = 1;
    return PyUnicode_DecodeUTF16(s, size, errors, &byteorder);
}

/* decode_c_bytes.proto */
static CYTHON_INLINE PyObject* __Pyx_decode_c_bytes(
         const char* cstring, Py_ssize_t length, Py_ssize_t start, Py_ssize_t stop,
         const char* encoding, const char* errors,
         PyObject* (*decode_func)(const char *s, Py_ssize_t size, const char *errors));

/* decode_bytes.proto */
static CYTHON_INLINE PyObject* __Pyx_decode_bytes(
         PyObject* string, Py_ssize_t start, Py_ssize_t stop,
         const char* encoding, const char* errors,
         PyObject* (*decode_func)(const char *s, Py_ssize_t size, const char *errors)) {
    return __Pyx_decode_c_bytes(
        PyBytes_AS_STRING(string), PyBytes_GET_SIZE(string),
        start, stop, encoding, errors, decode_func);
}

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* SetVTable.proto */
static int __Pyx_SetVtable(PyObject *dict, void *vtable);

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* Print.proto */
static int __Pyx_Print(PyObject*, PyObject *, int);
#if CYTHON_COMPILING_IN_PYPY || PY_MAJOR_VERSION >= 3
static PyObject* __pyx_print = 0;
static PyObject* __pyx_print_kwargs = 0;
#endif

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int32_t(int32_t value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint16_t(uint16_t value);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* PrintOne.proto */
static int __Pyx_PrintOne(PyObject* stream, PyObject *o);

/* CIntFromPy.proto */
static CYTHON_INLINE uint32_t __Pyx_PyInt_As_uint32_t(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE uint16_t __Pyx_PyInt_As_uint16_t(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE int32_t __Pyx_PyInt_As_int32_t(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* PyIdentifierFromString.proto */
#if !defined(__Pyx_PyIdentifier_FromString)
#if PY_MAJOR_VERSION < 3
  #define __Pyx_PyIdentifier_FromString(s) PyString_FromString(s)
#else
  #define __Pyx_PyIdentifier_FromString(s) PyUnicode_FromString(s)
#endif
#endif

/* ModuleImport.proto */
static PyObject *__Pyx_ImportModule(const char *name);

/* TypeImport.proto */
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name, size_t size, int strict);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

static int __pyx_f_3ssw_5sswpy_3SSW_printResult_c(struct __pyx_obj_3ssw_5sswpy_SSW *__pyx_v_self, s_align *__pyx_v_result, Py_ssize_t __pyx_v_start_idx); /* proto*/
static s_align *__pyx_f_3ssw_5sswpy_3SSW_align_c(struct __pyx_obj_3ssw_5sswpy_SSW *__pyx_v_self, int __pyx_v_gap_open, int __pyx_v_gap_extension, Py_ssize_t __pyx_v_start_idx, int32_t __pyx_v_mod_ref_length); /* proto*/
static int __pyx_f_3ssw_5sswpy_3SSW_buildDNAScoreMatrix(CYTHON_UNUSED struct __pyx_obj_3ssw_5sswpy_SSW *__pyx_v_self, uint8_t const __pyx_v_match_score, uint8_t const __pyx_v_mismatch_penalty, int8_t *__pyx_v_matrix); /* proto*/

/* Module declarations from 'cpython.mem' */

/* Module declarations from 'libc.stdint' */

/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdio' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.type' */
static PyTypeObject *__pyx_ptype_7cpython_4type_type = 0;

/* Module declarations from 'cpython' */

/* Module declarations from 'cpython.object' */

/* Module declarations from 'cpython.ref' */

/* Module declarations from 'ssw.c_util' */
static CYTHON_INLINE PyObject *__pyx_f_3ssw_6c_util__str(PyObject *); /*proto*/
static CYTHON_INLINE char *__pyx_f_3ssw_6c_util_obj_to_cstr_len(PyObject *, Py_ssize_t *); /*proto*/

/* Module declarations from 'ssw.sswpy' */
static PyTypeObject *__pyx_ptype_3ssw_5sswpy_SSW = 0;
#define __Pyx_MODULE_NAME "ssw.sswpy"
extern int __pyx_module_is_main_ssw__sswpy;
int __pyx_module_is_main_ssw__sswpy = 0;

/* Implementation of 'ssw.sswpy' */
static PyObject *__pyx_builtin_MemoryError;
static PyObject *__pyx_builtin_range;
static PyObject *__pyx_builtin_ValueError;
static PyObject *__pyx_builtin_chr;
static PyObject *__pyx_builtin_TypeError;
static PyObject *__pyx_builtin_OSError;
static const char __pyx_k_a[] = "a";
static const char __pyx_k__7[] = "";
static const char __pyx_k__12[] = " ";
static const char __pyx_k_chr[] = "chr";
static const char __pyx_k_d_s[] = "%d%s";
static const char __pyx_k_end[] = "end";
static const char __pyx_k_res[] = "res";
static const char __pyx_k_file[] = "file";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_name[] = "__name__";
static const char __pyx_k_read[] = "read";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_utf8[] = "utf8";
static const char __pyx_k_CIGAR[] = "CIGAR";
static const char __pyx_k_STR_T[] = "STR_T";
static const char __pyx_k_Union[] = "Union";
static const char __pyx_k_align[] = "align";
static const char __pyx_k_len_x[] = "len_x";
static const char __pyx_k_print[] = "print";
static const char __pyx_k_range[] = "range";
static const char __pyx_k_encode[] = "encode";
static const char __pyx_k_format[] = "format";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_result[] = "result";
static const char __pyx_k_typing[] = "typing";
static const char __pyx_k_OSError[] = "OSError";
static const char __pyx_k_aligner[] = "aligner";
static const char __pyx_k_end_idx[] = "end_idx";
static const char __pyx_k_ref_out[] = "ref_out";
static const char __pyx_k_setRead[] = "setRead";
static const char __pyx_k_do_print[] = "do_print";
static const char __pyx_k_gap_open[] = "gap_open";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_read_end[] = "read_end";
static const char __pyx_k_read_out[] = "read_out";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_Alignment[] = "Alignment";
static const char __pyx_k_TypeError[] = "TypeError";
static const char __pyx_k_alignment[] = "alignment";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_reference[] = "reference";
static const char __pyx_k_ssw_sswpy[] = "ssw.sswpy";
static const char __pyx_k_start_idx[] = "start_idx";
static const char __pyx_k_start_ref[] = "start_ref";
static const char __pyx_k_NamedTuple[] = "NamedTuple";
static const char __pyx_k_ValueError[] = "ValueError";
static const char __pyx_k_buffer_ref[] = "buffer_ref";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_read_start[] = "read_start";
static const char __pyx_k_start_read[] = "start_read";
static const char __pyx_k_MemoryError[] = "MemoryError";
static const char __pyx_k_buffer_read[] = "buffer_read";
static const char __pyx_k_force_align[] = "force_align";
static const char __pyx_k_match_score[] = "match_score";
static const char __pyx_k_obj_to_cstr[] = "obj_to_cstr:";
static const char __pyx_k_setReference[] = "setReference";
static const char __pyx_k_Out_of_Memory[] = "Out of Memory";
static const char __pyx_k_gap_extension[] = "gap_extension";
static const char __pyx_k_optimal_score[] = "optimal_score";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_reference_end[] = "reference_end";
static const char __pyx_k_ssw_sswpy_pyx[] = "ssw/sswpy.pyx";
static const char __pyx_k_force_overhang[] = "force_overhang";
static const char __pyx_k_reference_start[] = "reference_start";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_copy_obj_to_cstr[] = "copy_obj_to_cstr:";
static const char __pyx_k_mismatch_penalty[] = "mismatch_penalty";
static const char __pyx_k_No_solution_found[] = "No solution found";
static const char __pyx_k_sub_optimal_score[] = "sub_optimal_score";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_format_force_align[] = "format_force_align";
static const char __pyx_k_Must_set_profile_first[] = "Must set profile first";
static const char __pyx_k_call_setReference_first[] = "call setReference first";
static const char __pyx_k_negative_indexing_not_supported[] = "negative indexing not supported";
static const char __pyx_k_Could_not_allocate_memory_for_se[] = "Could not allocate memory for sequence.";
static const char __pyx_k_Problem_Running_alignment_see_st[] = "Problem Running alignment, see stdout";
static const char __pyx_k_Read_does_not_align_to_one_overh[] = "Read does not align to one overhang";
static const char __pyx_k_no_default___reduce___due_to_non[] = "no default __reduce__ due to non-trivial __cinit__";
static const char __pyx_k_obj_to_cstr_PyBytes_AsStringAndS[] = "obj_to_cstr: PyBytes_AsStringAndSize error";
static const char __pyx_k_obj_to_cstr_PyUnicode_AsUTF8AndS[] = "obj_to_cstr: PyUnicode_AsUTF8AndSize error";
static const char __pyx_k_start_idx_or_end_idx_can_t_be_gr[] = "start_idx: {} or end_idx: {} can't be greater than ref_length: {}";
static PyObject *__pyx_n_s_Alignment;
static PyObject *__pyx_n_s_CIGAR;
static PyObject *__pyx_kp_s_Could_not_allocate_memory_for_se;
static PyObject *__pyx_n_s_MemoryError;
static PyObject *__pyx_kp_s_Must_set_profile_first;
static PyObject *__pyx_n_s_NamedTuple;
static PyObject *__pyx_kp_s_No_solution_found;
static PyObject *__pyx_n_s_OSError;
static PyObject *__pyx_kp_s_Out_of_Memory;
static PyObject *__pyx_kp_s_Problem_Running_alignment_see_st;
static PyObject *__pyx_kp_s_Read_does_not_align_to_one_overh;
static PyObject *__pyx_n_s_STR_T;
static PyObject *__pyx_n_s_TypeError;
static PyObject *__pyx_n_s_Union;
static PyObject *__pyx_n_s_ValueError;
static PyObject *__pyx_kp_s__12;
static PyObject *__pyx_kp_s__7;
static PyObject *__pyx_n_s_a;
static PyObject *__pyx_n_s_align;
static PyObject *__pyx_n_s_aligner;
static PyObject *__pyx_n_s_alignment;
static PyObject *__pyx_n_s_buffer_read;
static PyObject *__pyx_n_s_buffer_ref;
static PyObject *__pyx_kp_s_call_setReference_first;
static PyObject *__pyx_n_s_chr;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_kp_s_copy_obj_to_cstr;
static PyObject *__pyx_kp_s_d_s;
static PyObject *__pyx_n_s_do_print;
static PyObject *__pyx_n_s_encode;
static PyObject *__pyx_n_s_end;
static PyObject *__pyx_n_s_end_idx;
static PyObject *__pyx_n_s_file;
static PyObject *__pyx_n_s_force_align;
static PyObject *__pyx_n_s_force_overhang;
static PyObject *__pyx_n_s_format;
static PyObject *__pyx_n_s_format_force_align;
static PyObject *__pyx_n_s_gap_extension;
static PyObject *__pyx_n_s_gap_open;
static PyObject *__pyx_n_s_getstate;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_len_x;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_match_score;
static PyObject *__pyx_n_s_mismatch_penalty;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_kp_s_negative_indexing_not_supported;
static PyObject *__pyx_kp_s_no_default___reduce___due_to_non;
static PyObject *__pyx_kp_s_obj_to_cstr;
static PyObject *__pyx_kp_s_obj_to_cstr_PyBytes_AsStringAndS;
static PyObject *__pyx_kp_s_obj_to_cstr_PyUnicode_AsUTF8AndS;
static PyObject *__pyx_n_s_optimal_score;
static PyObject *__pyx_n_s_print;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_range;
static PyObject *__pyx_n_s_read;
static PyObject *__pyx_n_s_read_end;
static PyObject *__pyx_n_s_read_out;
static PyObject *__pyx_n_s_read_start;
static PyObject *__pyx_n_s_reduce;
static PyObject *__pyx_n_s_reduce_cython;
static PyObject *__pyx_n_s_reduce_ex;
static PyObject *__pyx_n_s_ref_out;
static PyObject *__pyx_n_s_reference;
static PyObject *__pyx_n_s_reference_end;
static PyObject *__pyx_n_s_reference_start;
static PyObject *__pyx_n_s_res;
static PyObject *__pyx_n_s_result;
static PyObject *__pyx_n_s_setRead;
static PyObject *__pyx_n_s_setReference;
static PyObject *__pyx_n_s_setstate;
static PyObject *__pyx_n_s_setstate_cython;
static PyObject *__pyx_n_s_ssw_sswpy;
static PyObject *__pyx_kp_s_ssw_sswpy_pyx;
static PyObject *__pyx_n_s_start_idx;
static PyObject *__pyx_kp_s_start_idx_or_end_idx_can_t_be_gr;
static PyObject *__pyx_n_s_start_read;
static PyObject *__pyx_n_s_start_ref;
static PyObject *__pyx_n_s_sub_optimal_score;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_typing;
static PyObject *__pyx_n_s_utf8;
static int __pyx_pf_3ssw_5sswpy_3SSW___cinit__(struct __pyx_obj_3ssw_5sswpy_SSW *__pyx_v_self, CYTHON_UNUSED int __pyx_v_match_score, CYTHON_UNUSED int __pyx_v_mismatch_penalty); /* proto */
static int __pyx_pf_3ssw_5sswpy_3SSW_2__init__(struct __pyx_obj_3ssw_5sswpy_SSW *__pyx_v_self, int __pyx_v_match_score, int __pyx_v_mismatch_penalty); /* proto */
static void __pyx_pf_3ssw_5sswpy_3SSW_4__dealloc__(struct __pyx_obj_3ssw_5sswpy_SSW *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_3ssw_5sswpy_3SSW_6printResult(struct __pyx_obj_3ssw_5sswpy_SSW *__pyx_v_self, PyObject *__pyx_v_result, PyObject *__pyx_v_start_idx); /* proto */
static PyObject *__pyx_pf_3ssw_5sswpy_3SSW_8setRead(struct __pyx_obj_3ssw_5sswpy_SSW *__pyx_v_self, PyObject *__pyx_v_read); /* proto */
static PyObject *__pyx_pf_3ssw_5sswpy_3SSW_10setReference(struct __pyx_obj_3ssw_5sswpy_SSW *__pyx_v_self, PyObject *__pyx_v_reference); /* proto */
static PyObject *__pyx_pf_3ssw_5sswpy_3SSW_12align(struct __pyx_obj_3ssw_5sswpy_SSW *__pyx_v_self, int __pyx_v_gap_open, int __pyx_v_gap_extension, Py_ssize_t __pyx_v_start_idx, Py_ssize_t __pyx_v_end_idx); /* proto */
static PyObject *__pyx_pf_3ssw_5sswpy_3SSW_14__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_3ssw_5sswpy_SSW *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_3ssw_5sswpy_3SSW_16__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_3ssw_5sswpy_SSW *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_3ssw_5sswpy_force_align(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_read, PyObject *__pyx_v_reference, PyObject *__pyx_v_force_overhang, struct __pyx_obj_3ssw_5sswpy_SSW *__pyx_v_aligner); /* proto */
static PyObject *__pyx_pf_3ssw_5sswpy_2format_force_align(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_read, PyObject *__pyx_v_reference, PyObject *__pyx_v_alignment, PyObject *__pyx_v_do_print); /* proto */
static PyObject *__pyx_tp_new_3ssw_5sswpy_SSW(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_int_0;
static PyObject *__pyx_int_4;
static PyObject *__pyx_tuple_;
static PyObject *__pyx_tuple__2;
static PyObject *__pyx_tuple__3;
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_tuple__5;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_tuple__8;
static PyObject *__pyx_tuple__9;
static PyObject *__pyx_tuple__10;
static PyObject *__pyx_tuple__11;
static PyObject *__pyx_tuple__13;
static PyObject *__pyx_tuple__14;
static PyObject *__pyx_tuple__15;
static PyObject *__pyx_tuple__16;
static PyObject *__pyx_tuple__17;
static PyObject *__pyx_tuple__18;
static PyObject *__pyx_tuple__19;
static PyObject *__pyx_tuple__20;
static PyObject *__pyx_tuple__21;
static PyObject *__pyx_tuple__23;
static PyObject *__pyx_codeobj__22;
static PyObject *__pyx_codeobj__24;

/* "ssw/sswpy.pyx":79
 *     cdef Py_ssize_t ref_length
 * 
 *     def __cinit__(self, int match_score=2,             # <<<<<<<<<<<<<<
 *                         int mismatch_penalty=2):
 *         self.score_matrix = NULL
 */

/* Python wrapper */
static int __pyx_pw_3ssw_5sswpy_3SSW_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_3ssw_5sswpy_3SSW_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  CYTHON_UNUSED int __pyx_v_match_score;
  CYTHON_UNUSED int __pyx_v_mismatch_penalty;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_match_score,&__pyx_n_s_mismatch_penalty,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_match_score);
          if (value) { values[0] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_mismatch_penalty);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(0, 79, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    if (values[0]) {
      __pyx_v_match_score = __Pyx_PyInt_As_int(values[0]); if (unlikely((__pyx_v_match_score == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 79, __pyx_L3_error)
    } else {
      __pyx_v_match_score = ((int)2);
    }
    if (values[1]) {
      __pyx_v_mismatch_penalty = __Pyx_PyInt_As_int(values[1]); if (unlikely((__pyx_v_mismatch_penalty == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 80, __pyx_L3_error)
    } else {
      __pyx_v_mismatch_penalty = ((int)2);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 0, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 79, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ssw.sswpy.SSW.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_3ssw_5sswpy_3SSW___cinit__(((struct __pyx_obj_3ssw_5sswpy_SSW *)__pyx_v_self), __pyx_v_match_score, __pyx_v_mismatch_penalty);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_3ssw_5sswpy_3SSW___cinit__(struct __pyx_obj_3ssw_5sswpy_SSW *__pyx_v_self, CYTHON_UNUSED int __pyx_v_match_score, CYTHON_UNUSED int __pyx_v_mismatch_penalty) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "ssw/sswpy.pyx":81
 *     def __cinit__(self, int match_score=2,
 *                         int mismatch_penalty=2):
 *         self.score_matrix = NULL             # <<<<<<<<<<<<<<
 *         self.profile = NULL
 *         self.read_arr = NULL
 */
  __pyx_v_self->score_matrix = NULL;

  /* "ssw/sswpy.pyx":82
 *                         int mismatch_penalty=2):
 *         self.score_matrix = NULL
 *         self.profile = NULL             # <<<<<<<<<<<<<<
 *         self.read_arr = NULL
 *         self.ref_arr = NULL
 */
  __pyx_v_self->profile = NULL;

  /* "ssw/sswpy.pyx":83
 *         self.score_matrix = NULL
 *         self.profile = NULL
 *         self.read_arr = NULL             # <<<<<<<<<<<<<<
 *         self.ref_arr = NULL
 *     # end def
 */
  __pyx_v_self->read_arr = NULL;

  /* "ssw/sswpy.pyx":84
 *         self.profile = NULL
 *         self.read_arr = NULL
 *         self.ref_arr = NULL             # <<<<<<<<<<<<<<
 *     # end def
 * 
 */
  __pyx_v_self->ref_arr = NULL;

  /* "ssw/sswpy.pyx":79
 *     cdef Py_ssize_t ref_length
 * 
 *     def __cinit__(self, int match_score=2,             # <<<<<<<<<<<<<<
 *                         int mismatch_penalty=2):
 *         self.score_matrix = NULL
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ssw/sswpy.pyx":87
 *     # end def
 * 
 *     def __init__(self,  int match_score=2,             # <<<<<<<<<<<<<<
 *                         int mismatch_penalty=2):
 *         """ Requires a
 */

/* Python wrapper */
static int __pyx_pw_3ssw_5sswpy_3SSW_3__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_3ssw_5sswpy_3SSW_2__init__[] = " Requires a\n\n        Args:\n            match_score (int): for scoring matches\n            mismatch_penalty (int): for scoring mismatches\n        ";
#if CYTHON_COMPILING_IN_CPYTHON
struct wrapperbase __pyx_wrapperbase_3ssw_5sswpy_3SSW_2__init__;
#endif
static int __pyx_pw_3ssw_5sswpy_3SSW_3__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_v_match_score;
  int __pyx_v_mismatch_penalty;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_match_score,&__pyx_n_s_mismatch_penalty,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_match_score);
          if (value) { values[0] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_mismatch_penalty);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 87, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    if (values[0]) {
      __pyx_v_match_score = __Pyx_PyInt_As_int(values[0]); if (unlikely((__pyx_v_match_score == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 87, __pyx_L3_error)
    } else {
      __pyx_v_match_score = ((int)2);
    }
    if (values[1]) {
      __pyx_v_mismatch_penalty = __Pyx_PyInt_As_int(values[1]); if (unlikely((__pyx_v_mismatch_penalty == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 88, __pyx_L3_error)
    } else {
      __pyx_v_mismatch_penalty = ((int)2);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 0, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 87, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ssw.sswpy.SSW.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_3ssw_5sswpy_3SSW_2__init__(((struct __pyx_obj_3ssw_5sswpy_SSW *)__pyx_v_self), __pyx_v_match_score, __pyx_v_mismatch_penalty);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_3ssw_5sswpy_3SSW_2__init__(struct __pyx_obj_3ssw_5sswpy_SSW *__pyx_v_self, int __pyx_v_match_score, int __pyx_v_mismatch_penalty) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ssw/sswpy.pyx":95
 *             mismatch_penalty (int): for scoring mismatches
 *         """
 *         self.score_matrix = <int8_t*> PyMem_Malloc(25*sizeof(int8_t))             # <<<<<<<<<<<<<<
 *         self.buildDNAScoreMatrix(   <uint8_t>match_score,
 *                                     <uint8_t> mismatch_penalty,
 */
  __pyx_v_self->score_matrix = ((int8_t *)PyMem_Malloc((25 * (sizeof(int8_t)))));

  /* "ssw/sswpy.pyx":96
 *         """
 *         self.score_matrix = <int8_t*> PyMem_Malloc(25*sizeof(int8_t))
 *         self.buildDNAScoreMatrix(   <uint8_t>match_score,             # <<<<<<<<<<<<<<
 *                                     <uint8_t> mismatch_penalty,
 *                                     self.score_matrix)
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_3ssw_5sswpy_SSW *)__pyx_v_self->__pyx_vtab)->buildDNAScoreMatrix(__pyx_v_self, ((uint8_t)__pyx_v_match_score), ((uint8_t)__pyx_v_mismatch_penalty), __pyx_v_self->score_matrix); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(0, 96, __pyx_L1_error)

  /* "ssw/sswpy.pyx":99
 *                                     <uint8_t> mismatch_penalty,
 *                                     self.score_matrix)
 *         self.read = None             # <<<<<<<<<<<<<<
 *         self.reference = None
 *     # end def
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->read);
  __Pyx_DECREF(__pyx_v_self->read);
  __pyx_v_self->read = Py_None;

  /* "ssw/sswpy.pyx":100
 *                                     self.score_matrix)
 *         self.read = None
 *         self.reference = None             # <<<<<<<<<<<<<<
 *     # end def
 * 
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->reference);
  __Pyx_DECREF(__pyx_v_self->reference);
  __pyx_v_self->reference = Py_None;

  /* "ssw/sswpy.pyx":87
 *     # end def
 * 
 *     def __init__(self,  int match_score=2,             # <<<<<<<<<<<<<<
 *                         int mismatch_penalty=2):
 *         """ Requires a
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ssw.sswpy.SSW.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ssw/sswpy.pyx":103
 *     # end def
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         PyMem_Free(self.score_matrix)
 * 
 */

/* Python wrapper */
static void __pyx_pw_3ssw_5sswpy_3SSW_5__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_3ssw_5sswpy_3SSW_5__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_3ssw_5sswpy_3SSW_4__dealloc__(((struct __pyx_obj_3ssw_5sswpy_SSW *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_3ssw_5sswpy_3SSW_4__dealloc__(struct __pyx_obj_3ssw_5sswpy_SSW *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "ssw/sswpy.pyx":104
 * 
 *     def __dealloc__(self):
 *         PyMem_Free(self.score_matrix)             # <<<<<<<<<<<<<<
 * 
 *         if self.profile != NULL:
 */
  PyMem_Free(__pyx_v_self->score_matrix);

  /* "ssw/sswpy.pyx":106
 *         PyMem_Free(self.score_matrix)
 * 
 *         if self.profile != NULL:             # <<<<<<<<<<<<<<
 *             init_destroy(self.profile)
 *             self.profile = NULL
 */
  __pyx_t_1 = ((__pyx_v_self->profile != NULL) != 0);
  if (__pyx_t_1) {

    /* "ssw/sswpy.pyx":107
 * 
 *         if self.profile != NULL:
 *             init_destroy(self.profile)             # <<<<<<<<<<<<<<
 *             self.profile = NULL
 * 
 */
    init_destroy(__pyx_v_self->profile);

    /* "ssw/sswpy.pyx":108
 *         if self.profile != NULL:
 *             init_destroy(self.profile)
 *             self.profile = NULL             # <<<<<<<<<<<<<<
 * 
 *         if self.read_arr != NULL:
 */
    __pyx_v_self->profile = NULL;

    /* "ssw/sswpy.pyx":106
 *         PyMem_Free(self.score_matrix)
 * 
 *         if self.profile != NULL:             # <<<<<<<<<<<<<<
 *             init_destroy(self.profile)
 *             self.profile = NULL
 */
  }

  /* "ssw/sswpy.pyx":110
 *             self.profile = NULL
 * 
 *         if self.read_arr != NULL:             # <<<<<<<<<<<<<<
 *             PyMem_Free(self.read_arr)
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->read_arr != NULL) != 0);
  if (__pyx_t_1) {

    /* "ssw/sswpy.pyx":111
 * 
 *         if self.read_arr != NULL:
 *             PyMem_Free(self.read_arr)             # <<<<<<<<<<<<<<
 * 
 *         if self.ref_arr != NULL:
 */
    PyMem_Free(__pyx_v_self->read_arr);

    /* "ssw/sswpy.pyx":110
 *             self.profile = NULL
 * 
 *         if self.read_arr != NULL:             # <<<<<<<<<<<<<<
 *             PyMem_Free(self.read_arr)
 * 
 */
  }

  /* "ssw/sswpy.pyx":113
 *             PyMem_Free(self.read_arr)
 * 
 *         if self.ref_arr != NULL:             # <<<<<<<<<<<<<<
 *             PyMem_Free(self.ref_arr)
 *     # end def
 */
  __pyx_t_1 = ((__pyx_v_self->ref_arr != NULL) != 0);
  if (__pyx_t_1) {

    /* "ssw/sswpy.pyx":114
 * 
 *         if self.ref_arr != NULL:
 *             PyMem_Free(self.ref_arr)             # <<<<<<<<<<<<<<
 *     # end def
 * 
 */
    PyMem_Free(__pyx_v_self->ref_arr);

    /* "ssw/sswpy.pyx":113
 *             PyMem_Free(self.read_arr)
 * 
 *         if self.ref_arr != NULL:             # <<<<<<<<<<<<<<
 *             PyMem_Free(self.ref_arr)
 *     # end def
 */
  }

  /* "ssw/sswpy.pyx":103
 *     # end def
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         PyMem_Free(self.score_matrix)
 * 
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "ssw/sswpy.pyx":117
 *     # end def
 * 
 *     cdef int printResult_c(self, s_align* result, Py_ssize_t start_idx) except -1:             # <<<<<<<<<<<<<<
 *         cdef const char* read_cstr
 *         cdef Py_ssize_t read_length, ref_length
 */

static int __pyx_f_3ssw_5sswpy_3SSW_printResult_c(struct __pyx_obj_3ssw_5sswpy_SSW *__pyx_v_self, s_align *__pyx_v_result, Py_ssize_t __pyx_v_start_idx) {
  char const *__pyx_v_read_cstr;
  Py_ssize_t __pyx_v_read_length;
  Py_ssize_t __pyx_v_ref_length;
  char const *__pyx_v_ref_cstr;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("printResult_c", 0);

  /* "ssw/sswpy.pyx":122
 *         cdef const char* ref_cstr
 * 
 *         print(self.read)             # <<<<<<<<<<<<<<
 *         if result != NULL:
 *             read_cstr = c_util.obj_to_cstr_len(self.read, &read_length)
 */
  if (__Pyx_PrintOne(0, __pyx_v_self->read) < 0) __PYX_ERR(0, 122, __pyx_L1_error)

  /* "ssw/sswpy.pyx":123
 * 
 *         print(self.read)
 *         if result != NULL:             # <<<<<<<<<<<<<<
 *             read_cstr = c_util.obj_to_cstr_len(self.read, &read_length)
 *             ref_cstr = c_util.obj_to_cstr_len(self.reference, &ref_length)
 */
  __pyx_t_1 = ((__pyx_v_result != NULL) != 0);
  if (__pyx_t_1) {

    /* "ssw/sswpy.pyx":124
 *         print(self.read)
 *         if result != NULL:
 *             read_cstr = c_util.obj_to_cstr_len(self.read, &read_length)             # <<<<<<<<<<<<<<
 *             ref_cstr = c_util.obj_to_cstr_len(self.reference, &ref_length)
 * 
 */
    __pyx_t_2 = __pyx_v_self->read;
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v_read_cstr = __pyx_f_3ssw_6c_util_obj_to_cstr_len(__pyx_t_2, (&__pyx_v_read_length));
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "ssw/sswpy.pyx":125
 *         if result != NULL:
 *             read_cstr = c_util.obj_to_cstr_len(self.read, &read_length)
 *             ref_cstr = c_util.obj_to_cstr_len(self.reference, &ref_length)             # <<<<<<<<<<<<<<
 * 
 *             ssw_write_cigar(result)
 */
    __pyx_t_2 = __pyx_v_self->reference;
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v_ref_cstr = __pyx_f_3ssw_6c_util_obj_to_cstr_len(__pyx_t_2, (&__pyx_v_ref_length));
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "ssw/sswpy.pyx":127
 *             ref_cstr = c_util.obj_to_cstr_len(self.reference, &ref_length)
 * 
 *             ssw_write_cigar(result)             # <<<<<<<<<<<<<<
 *             ssw_writer(result, &ref_cstr[start_idx], read_cstr)
 *         return 0
 */
    ssw_write_cigar(__pyx_v_result);

    /* "ssw/sswpy.pyx":128
 * 
 *             ssw_write_cigar(result)
 *             ssw_writer(result, &ref_cstr[start_idx], read_cstr)             # <<<<<<<<<<<<<<
 *         return 0
 *     # end def
 */
    ssw_writer(__pyx_v_result, (&(__pyx_v_ref_cstr[__pyx_v_start_idx])), __pyx_v_read_cstr);

    /* "ssw/sswpy.pyx":123
 * 
 *         print(self.read)
 *         if result != NULL:             # <<<<<<<<<<<<<<
 *             read_cstr = c_util.obj_to_cstr_len(self.read, &read_length)
 *             ref_cstr = c_util.obj_to_cstr_len(self.reference, &ref_length)
 */
  }

  /* "ssw/sswpy.pyx":129
 *             ssw_write_cigar(result)
 *             ssw_writer(result, &ref_cstr[start_idx], read_cstr)
 *         return 0             # <<<<<<<<<<<<<<
 *     # end def
 * 
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "ssw/sswpy.pyx":117
 *     # end def
 * 
 *     cdef int printResult_c(self, s_align* result, Py_ssize_t start_idx) except -1:             # <<<<<<<<<<<<<<
 *         cdef const char* read_cstr
 *         cdef Py_ssize_t read_length, ref_length
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("ssw.sswpy.SSW.printResult_c", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ssw/sswpy.pyx":133
 * 
 * 
 *     def printResult(self, result: Alignment, start_idx: int = 0):             # <<<<<<<<<<<<<<
 *         """ rebuild a s_align struct from a result dictionary
 *         so as to be able to call ssw terminal print functions
 */

/* Python wrapper */
static PyObject *__pyx_pw_3ssw_5sswpy_3SSW_7printResult(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_3ssw_5sswpy_3SSW_6printResult[] = " rebuild a s_align struct from a result dictionary\n        so as to be able to call ssw terminal print functions\n\n        Args:\n            result: Alignment containing tthe result from a call to SSW.align\n            start_idx: index to start printing from. defaults to 0\n\n        raises:\n            MemoryError\n        ";
static PyObject *__pyx_pw_3ssw_5sswpy_3SSW_7printResult(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_result = 0;
  PyObject *__pyx_v_start_idx = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("printResult (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_result,&__pyx_n_s_start_idx,0};
    PyObject* values[2] = {0,0};
    values[1] = ((PyObject *)__pyx_int_0);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_result)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_start_idx);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "printResult") < 0)) __PYX_ERR(0, 133, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_result = values[0];
    __pyx_v_start_idx = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("printResult", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 133, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ssw.sswpy.SSW.printResult", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_3ssw_5sswpy_3SSW_6printResult(((struct __pyx_obj_3ssw_5sswpy_SSW *)__pyx_v_self), __pyx_v_result, __pyx_v_start_idx);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3ssw_5sswpy_3SSW_6printResult(struct __pyx_obj_3ssw_5sswpy_SSW *__pyx_v_self, PyObject *__pyx_v_result, PyObject *__pyx_v_start_idx) {
  uint32_t *__pyx_v_cigar_array;
  s_align *__pyx_v_res_align;
  char __pyx_v_letter;
  uint32_t __pyx_v_length;
  Py_ssize_t __pyx_v_i;
  Py_ssize_t __pyx_v_j;
  PyObject *__pyx_v_cigar = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  uint32_t __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  long __pyx_t_9;
  uint16_t __pyx_t_10;
  int32_t __pyx_t_11;
  int __pyx_t_12;
  int __pyx_t_13;
  char const *__pyx_t_14;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *__pyx_t_19 = NULL;
  PyObject *__pyx_t_20 = NULL;
  __Pyx_RefNannySetupContext("printResult", 0);

  /* "ssw/sswpy.pyx":144
 *             MemoryError
 *         """
 *         cdef uint32_t* cigar_array = NULL             # <<<<<<<<<<<<<<
 *         cdef s_align *res_align = NULL
 *         cdef char letter
 */
  __pyx_v_cigar_array = NULL;

  /* "ssw/sswpy.pyx":145
 *         """
 *         cdef uint32_t* cigar_array = NULL
 *         cdef s_align *res_align = NULL             # <<<<<<<<<<<<<<
 *         cdef char letter
 *         cdef uint32_t length
 */
  __pyx_v_res_align = NULL;

  /* "ssw/sswpy.pyx":151
 *         cdef Py_ssize_t i, j
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             res_align = <s_align*> PyMem_Malloc(sizeof(s_align))
 *             if res_align == NULL:
 */
  /*try:*/ {

    /* "ssw/sswpy.pyx":152
 * 
 *         try:
 *             res_align = <s_align*> PyMem_Malloc(sizeof(s_align))             # <<<<<<<<<<<<<<
 *             if res_align == NULL:
 *                 raise MemoryError('Out of Memory')
 */
    __pyx_v_res_align = ((s_align *)PyMem_Malloc((sizeof(s_align))));

    /* "ssw/sswpy.pyx":153
 *         try:
 *             res_align = <s_align*> PyMem_Malloc(sizeof(s_align))
 *             if res_align == NULL:             # <<<<<<<<<<<<<<
 *                 raise MemoryError('Out of Memory')
 *             cigar = result.CIGAR
 */
    __pyx_t_1 = ((__pyx_v_res_align == NULL) != 0);
    if (__pyx_t_1) {

      /* "ssw/sswpy.pyx":154
 *             res_align = <s_align*> PyMem_Malloc(sizeof(s_align))
 *             if res_align == NULL:
 *                 raise MemoryError('Out of Memory')             # <<<<<<<<<<<<<<
 *             cigar = result.CIGAR
 *             if cigar is not None:
 */
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_MemoryError, __pyx_tuple_, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 154, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_Raise(__pyx_t_2, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __PYX_ERR(0, 154, __pyx_L4_error)

      /* "ssw/sswpy.pyx":153
 *         try:
 *             res_align = <s_align*> PyMem_Malloc(sizeof(s_align))
 *             if res_align == NULL:             # <<<<<<<<<<<<<<
 *                 raise MemoryError('Out of Memory')
 *             cigar = result.CIGAR
 */
    }

    /* "ssw/sswpy.pyx":155
 *             if res_align == NULL:
 *                 raise MemoryError('Out of Memory')
 *             cigar = result.CIGAR             # <<<<<<<<<<<<<<
 *             if cigar is not None:
 *                 cigar_array = <uint32_t*> PyMem_Malloc(len(cigar)//2*sizeof(uint32_t))
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_result, __pyx_n_s_CIGAR); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 155, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_v_cigar = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "ssw/sswpy.pyx":156
 *                 raise MemoryError('Out of Memory')
 *             cigar = result.CIGAR
 *             if cigar is not None:             # <<<<<<<<<<<<<<
 *                 cigar_array = <uint32_t*> PyMem_Malloc(len(cigar)//2*sizeof(uint32_t))
 *                 if cigar_array == NULL:
 */
    __pyx_t_1 = (__pyx_v_cigar != Py_None);
    __pyx_t_3 = (__pyx_t_1 != 0);
    if (__pyx_t_3) {

      /* "ssw/sswpy.pyx":157
 *             cigar = result.CIGAR
 *             if cigar is not None:
 *                 cigar_array = <uint32_t*> PyMem_Malloc(len(cigar)//2*sizeof(uint32_t))             # <<<<<<<<<<<<<<
 *                 if cigar_array == NULL:
 *                     raise MemoryError('Out of Memory')
 */
      __pyx_t_4 = PyObject_Length(__pyx_v_cigar); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(0, 157, __pyx_L4_error)
      __pyx_v_cigar_array = ((uint32_t *)PyMem_Malloc((__Pyx_div_Py_ssize_t(__pyx_t_4, 2) * (sizeof(uint32_t)))));

      /* "ssw/sswpy.pyx":158
 *             if cigar is not None:
 *                 cigar_array = <uint32_t*> PyMem_Malloc(len(cigar)//2*sizeof(uint32_t))
 *                 if cigar_array == NULL:             # <<<<<<<<<<<<<<
 *                     raise MemoryError('Out of Memory')
 *                 j = 0
 */
      __pyx_t_3 = ((__pyx_v_cigar_array == NULL) != 0);
      if (__pyx_t_3) {

        /* "ssw/sswpy.pyx":159
 *                 cigar_array = <uint32_t*> PyMem_Malloc(len(cigar)//2*sizeof(uint32_t))
 *                 if cigar_array == NULL:
 *                     raise MemoryError('Out of Memory')             # <<<<<<<<<<<<<<
 *                 j = 0
 *                 for i in range(0, len(cigar), 2):
 */
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_MemoryError, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 159, __pyx_L4_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_Raise(__pyx_t_2, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __PYX_ERR(0, 159, __pyx_L4_error)

        /* "ssw/sswpy.pyx":158
 *             if cigar is not None:
 *                 cigar_array = <uint32_t*> PyMem_Malloc(len(cigar)//2*sizeof(uint32_t))
 *                 if cigar_array == NULL:             # <<<<<<<<<<<<<<
 *                     raise MemoryError('Out of Memory')
 *                 j = 0
 */
      }

      /* "ssw/sswpy.pyx":160
 *                 if cigar_array == NULL:
 *                     raise MemoryError('Out of Memory')
 *                 j = 0             # <<<<<<<<<<<<<<
 *                 for i in range(0, len(cigar), 2):
 *                     length = int(cigar[i])
 */
      __pyx_v_j = 0;

      /* "ssw/sswpy.pyx":161
 *                     raise MemoryError('Out of Memory')
 *                 j = 0
 *                 for i in range(0, len(cigar), 2):             # <<<<<<<<<<<<<<
 *                     length = int(cigar[i])
 *                     letter = ord(cigar[i+1])
 */
      __pyx_t_4 = PyObject_Length(__pyx_v_cigar); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(0, 161, __pyx_L4_error)
      for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=2) {
        __pyx_v_i = __pyx_t_5;

        /* "ssw/sswpy.pyx":162
 *                 j = 0
 *                 for i in range(0, len(cigar), 2):
 *                     length = int(cigar[i])             # <<<<<<<<<<<<<<
 *                     letter = ord(cigar[i+1])
 *                     cigar_array[j] = to_cigar_int(length, letter)
 */
        __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_cigar, __pyx_v_i, Py_ssize_t, 1, PyInt_FromSsize_t, 0, 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 162, __pyx_L4_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_6 = __Pyx_PyNumber_Int(__pyx_t_2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 162, __pyx_L4_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_t_7 = __Pyx_PyInt_As_uint32_t(__pyx_t_6); if (unlikely((__pyx_t_7 == ((uint32_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 162, __pyx_L4_error)
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __pyx_v_length = __pyx_t_7;

        /* "ssw/sswpy.pyx":163
 *                 for i in range(0, len(cigar), 2):
 *                     length = int(cigar[i])
 *                     letter = ord(cigar[i+1])             # <<<<<<<<<<<<<<
 *                     cigar_array[j] = to_cigar_int(length, letter)
 *                     j += 1
 */
        __pyx_t_8 = (__pyx_v_i + 1);
        __pyx_t_6 = __Pyx_GetItemInt(__pyx_v_cigar, __pyx_t_8, Py_ssize_t, 1, PyInt_FromSsize_t, 0, 0, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 163, __pyx_L4_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_9 = __Pyx_PyObject_Ord(__pyx_t_6); if (unlikely(__pyx_t_9 == ((long)(long)(Py_UCS4)-1))) __PYX_ERR(0, 163, __pyx_L4_error)
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __pyx_v_letter = __pyx_t_9;

        /* "ssw/sswpy.pyx":164
 *                     length = int(cigar[i])
 *                     letter = ord(cigar[i+1])
 *                     cigar_array[j] = to_cigar_int(length, letter)             # <<<<<<<<<<<<<<
 *                     j += 1
 *                 res_align.cigar = cigar_array
 */
        (__pyx_v_cigar_array[__pyx_v_j]) = to_cigar_int(__pyx_v_length, __pyx_v_letter);

        /* "ssw/sswpy.pyx":165
 *                     letter = ord(cigar[i+1])
 *                     cigar_array[j] = to_cigar_int(length, letter)
 *                     j += 1             # <<<<<<<<<<<<<<
 *                 res_align.cigar = cigar_array
 *                 res_align.cigarLen = len(cigar)//2
 */
        __pyx_v_j = (__pyx_v_j + 1);
      }

      /* "ssw/sswpy.pyx":166
 *                     cigar_array[j] = to_cigar_int(length, letter)
 *                     j += 1
 *                 res_align.cigar = cigar_array             # <<<<<<<<<<<<<<
 *                 res_align.cigarLen = len(cigar)//2
 *             else:
 */
      __pyx_v_res_align->cigar = __pyx_v_cigar_array;

      /* "ssw/sswpy.pyx":167
 *                     j += 1
 *                 res_align.cigar = cigar_array
 *                 res_align.cigarLen = len(cigar)//2             # <<<<<<<<<<<<<<
 *             else:
 *                 res_align.cigar = NULL
 */
      __pyx_t_4 = PyObject_Length(__pyx_v_cigar); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(0, 167, __pyx_L4_error)
      __pyx_v_res_align->cigarLen = __Pyx_div_Py_ssize_t(__pyx_t_4, 2);

      /* "ssw/sswpy.pyx":156
 *                 raise MemoryError('Out of Memory')
 *             cigar = result.CIGAR
 *             if cigar is not None:             # <<<<<<<<<<<<<<
 *                 cigar_array = <uint32_t*> PyMem_Malloc(len(cigar)//2*sizeof(uint32_t))
 *                 if cigar_array == NULL:
 */
      goto __pyx_L7;
    }

    /* "ssw/sswpy.pyx":169
 *                 res_align.cigarLen = len(cigar)//2
 *             else:
 *                 res_align.cigar = NULL             # <<<<<<<<<<<<<<
 *             res_align.score1 = result.optimal_score
 *             res_align.score2 = result.sub_optimal_score
 */
    /*else*/ {
      __pyx_v_res_align->cigar = NULL;
    }
    __pyx_L7:;

    /* "ssw/sswpy.pyx":170
 *             else:
 *                 res_align.cigar = NULL
 *             res_align.score1 = result.optimal_score             # <<<<<<<<<<<<<<
 *             res_align.score2 = result.sub_optimal_score
 *             res_align.ref_begin1 = result.reference_start
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_result, __pyx_n_s_optimal_score); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 170, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_10 = __Pyx_PyInt_As_uint16_t(__pyx_t_6); if (unlikely((__pyx_t_10 == ((uint16_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 170, __pyx_L4_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_v_res_align->score1 = __pyx_t_10;

    /* "ssw/sswpy.pyx":171
 *                 res_align.cigar = NULL
 *             res_align.score1 = result.optimal_score
 *             res_align.score2 = result.sub_optimal_score             # <<<<<<<<<<<<<<
 *             res_align.ref_begin1 = result.reference_start
 *             res_align.ref_end1 = result.reference_end
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_result, __pyx_n_s_sub_optimal_score); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 171, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_10 = __Pyx_PyInt_As_uint16_t(__pyx_t_6); if (unlikely((__pyx_t_10 == ((uint16_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 171, __pyx_L4_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_v_res_align->score2 = __pyx_t_10;

    /* "ssw/sswpy.pyx":172
 *             res_align.score1 = result.optimal_score
 *             res_align.score2 = result.sub_optimal_score
 *             res_align.ref_begin1 = result.reference_start             # <<<<<<<<<<<<<<
 *             res_align.ref_end1 = result.reference_end
 *             res_align.read_begin1 = result.read_start
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_result, __pyx_n_s_reference_start); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 172, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_11 = __Pyx_PyInt_As_int32_t(__pyx_t_6); if (unlikely((__pyx_t_11 == ((int32_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 172, __pyx_L4_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_v_res_align->ref_begin1 = __pyx_t_11;

    /* "ssw/sswpy.pyx":173
 *             res_align.score2 = result.sub_optimal_score
 *             res_align.ref_begin1 = result.reference_start
 *             res_align.ref_end1 = result.reference_end             # <<<<<<<<<<<<<<
 *             res_align.read_begin1 = result.read_start
 *             res_align.read_end1 = result.read_end
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_result, __pyx_n_s_reference_end); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 173, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_11 = __Pyx_PyInt_As_int32_t(__pyx_t_6); if (unlikely((__pyx_t_11 == ((int32_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 173, __pyx_L4_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_v_res_align->ref_end1 = __pyx_t_11;

    /* "ssw/sswpy.pyx":174
 *             res_align.ref_begin1 = result.reference_start
 *             res_align.ref_end1 = result.reference_end
 *             res_align.read_begin1 = result.read_start             # <<<<<<<<<<<<<<
 *             res_align.read_end1 = result.read_end
 * 
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_result, __pyx_n_s_read_start); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 174, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_11 = __Pyx_PyInt_As_int32_t(__pyx_t_6); if (unlikely((__pyx_t_11 == ((int32_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 174, __pyx_L4_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_v_res_align->read_begin1 = __pyx_t_11;

    /* "ssw/sswpy.pyx":175
 *             res_align.ref_end1 = result.reference_end
 *             res_align.read_begin1 = result.read_start
 *             res_align.read_end1 = result.read_end             # <<<<<<<<<<<<<<
 * 
 *             self.printResult_c(res_align, start_idx)
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_result, __pyx_n_s_read_end); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 175, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_11 = __Pyx_PyInt_As_int32_t(__pyx_t_6); if (unlikely((__pyx_t_11 == ((int32_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 175, __pyx_L4_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_v_res_align->read_end1 = __pyx_t_11;

    /* "ssw/sswpy.pyx":177
 *             res_align.read_end1 = result.read_end
 * 
 *             self.printResult_c(res_align, start_idx)             # <<<<<<<<<<<<<<
 *         finally:
 *             if cigar is not None:
 */
    __pyx_t_4 = __Pyx_PyIndex_AsSsize_t(__pyx_v_start_idx); if (unlikely((__pyx_t_4 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 177, __pyx_L4_error)
    __pyx_t_12 = ((struct __pyx_vtabstruct_3ssw_5sswpy_SSW *)__pyx_v_self->__pyx_vtab)->printResult_c(__pyx_v_self, __pyx_v_res_align, __pyx_t_4); if (unlikely(__pyx_t_12 == ((int)-1))) __PYX_ERR(0, 177, __pyx_L4_error)
  }

  /* "ssw/sswpy.pyx":179
 *             self.printResult_c(res_align, start_idx)
 *         finally:
 *             if cigar is not None:             # <<<<<<<<<<<<<<
 *                 PyMem_Free(cigar_array)
 *             PyMem_Free(res_align)
 */
  /*finally:*/ {
    /*normal exit:*/{
      __pyx_t_3 = (__pyx_v_cigar != Py_None);
      __pyx_t_1 = (__pyx_t_3 != 0);
      if (__pyx_t_1) {

        /* "ssw/sswpy.pyx":180
 *         finally:
 *             if cigar is not None:
 *                 PyMem_Free(cigar_array)             # <<<<<<<<<<<<<<
 *             PyMem_Free(res_align)
 *     # end def
 */
        PyMem_Free(__pyx_v_cigar_array);

        /* "ssw/sswpy.pyx":179
 *             self.printResult_c(res_align, start_idx)
 *         finally:
 *             if cigar is not None:             # <<<<<<<<<<<<<<
 *                 PyMem_Free(cigar_array)
 *             PyMem_Free(res_align)
 */
      }

      /* "ssw/sswpy.pyx":181
 *             if cigar is not None:
 *                 PyMem_Free(cigar_array)
 *             PyMem_Free(res_align)             # <<<<<<<<<<<<<<
 *     # end def
 * 
 */
      PyMem_Free(__pyx_v_res_align);
      goto __pyx_L5;
    }
    __pyx_L4_error:;
    /*exception exit:*/{
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0; __pyx_t_20 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (PY_MAJOR_VERSION >= 3) __Pyx_ExceptionSwap(&__pyx_t_18, &__pyx_t_19, &__pyx_t_20);
      if ((PY_MAJOR_VERSION < 3) || unlikely(__Pyx_GetException(&__pyx_t_15, &__pyx_t_16, &__pyx_t_17) < 0)) __Pyx_ErrFetch(&__pyx_t_15, &__pyx_t_16, &__pyx_t_17);
      __Pyx_XGOTREF(__pyx_t_15);
      __Pyx_XGOTREF(__pyx_t_16);
      __Pyx_XGOTREF(__pyx_t_17);
      __Pyx_XGOTREF(__pyx_t_18);
      __Pyx_XGOTREF(__pyx_t_19);
      __Pyx_XGOTREF(__pyx_t_20);
      __pyx_t_12 = __pyx_lineno; __pyx_t_13 = __pyx_clineno; __pyx_t_14 = __pyx_filename;
      {

        /* "ssw/sswpy.pyx":179
 *             self.printResult_c(res_align, start_idx)
 *         finally:
 *             if cigar is not None:             # <<<<<<<<<<<<<<
 *                 PyMem_Free(cigar_array)
 *             PyMem_Free(res_align)
 */
        if (unlikely(!__pyx_v_cigar)) { __Pyx_RaiseUnboundLocalError("cigar"); __PYX_ERR(0, 179, __pyx_L13_error) }
        __pyx_t_1 = (__pyx_v_cigar != Py_None);
        __pyx_t_3 = (__pyx_t_1 != 0);
        if (__pyx_t_3) {

          /* "ssw/sswpy.pyx":180
 *         finally:
 *             if cigar is not None:
 *                 PyMem_Free(cigar_array)             # <<<<<<<<<<<<<<
 *             PyMem_Free(res_align)
 *     # end def
 */
          PyMem_Free(__pyx_v_cigar_array);

          /* "ssw/sswpy.pyx":179
 *             self.printResult_c(res_align, start_idx)
 *         finally:
 *             if cigar is not None:             # <<<<<<<<<<<<<<
 *                 PyMem_Free(cigar_array)
 *             PyMem_Free(res_align)
 */
        }

        /* "ssw/sswpy.pyx":181
 *             if cigar is not None:
 *                 PyMem_Free(cigar_array)
 *             PyMem_Free(res_align)             # <<<<<<<<<<<<<<
 *     # end def
 * 
 */
        PyMem_Free(__pyx_v_res_align);
      }
      if (PY_MAJOR_VERSION >= 3) {
        __Pyx_XGIVEREF(__pyx_t_18);
        __Pyx_XGIVEREF(__pyx_t_19);
        __Pyx_XGIVEREF(__pyx_t_20);
        __Pyx_ExceptionReset(__pyx_t_18, __pyx_t_19, __pyx_t_20);
      }
      __Pyx_XGIVEREF(__pyx_t_15);
      __Pyx_XGIVEREF(__pyx_t_16);
      __Pyx_XGIVEREF(__pyx_t_17);
      __Pyx_ErrRestore(__pyx_t_15, __pyx_t_16, __pyx_t_17);
      __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0; __pyx_t_20 = 0;
      __pyx_lineno = __pyx_t_12; __pyx_clineno = __pyx_t_13; __pyx_filename = __pyx_t_14;
      goto __pyx_L1_error;
      __pyx_L13_error:;
      if (PY_MAJOR_VERSION >= 3) {
        __Pyx_XGIVEREF(__pyx_t_18);
        __Pyx_XGIVEREF(__pyx_t_19);
        __Pyx_XGIVEREF(__pyx_t_20);
        __Pyx_ExceptionReset(__pyx_t_18, __pyx_t_19, __pyx_t_20);
      }
      __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
      __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
      __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
      __pyx_t_18 = 0; __pyx_t_19 = 0; __pyx_t_20 = 0;
      goto __pyx_L1_error;
    }
    __pyx_L5:;
  }

  /* "ssw/sswpy.pyx":133
 * 
 * 
 *     def printResult(self, result: Alignment, start_idx: int = 0):             # <<<<<<<<<<<<<<
 *         """ rebuild a s_align struct from a result dictionary
 *         so as to be able to call ssw terminal print functions
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("ssw.sswpy.SSW.printResult", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_cigar);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ssw/sswpy.pyx":184
 *     # end def
 * 
 *     def setRead(self, read: STR_T):             # <<<<<<<<<<<<<<
 *         """ Set the query read string
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_3ssw_5sswpy_3SSW_9setRead(PyObject *__pyx_v_self, PyObject *__pyx_v_read); /*proto*/
static char __pyx_doc_3ssw_5sswpy_3SSW_8setRead[] = " Set the query read string\n\n        Args:\n            read:  String-like (str or bytestring) that represents the read.\n                    Must be set\n        ";
static PyObject *__pyx_pw_3ssw_5sswpy_3SSW_9setRead(PyObject *__pyx_v_self, PyObject *__pyx_v_read) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("setRead (wrapper)", 0);
  __pyx_r = __pyx_pf_3ssw_5sswpy_3SSW_8setRead(((struct __pyx_obj_3ssw_5sswpy_SSW *)__pyx_v_self), ((PyObject *)__pyx_v_read));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3ssw_5sswpy_3SSW_8setRead(struct __pyx_obj_3ssw_5sswpy_SSW *__pyx_v_self, PyObject *__pyx_v_read) {
  Py_ssize_t __pyx_v_read_length;
  char const *__pyx_v_read_cstr;
  int8_t *__pyx_v_read_arr;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("setRead", 0);

  /* "ssw/sswpy.pyx":192
 *         """
 *         cdef Py_ssize_t read_length
 *         cdef const char* read_cstr = c_util.obj_to_cstr_len(read, &read_length)             # <<<<<<<<<<<<<<
 *         cdef int8_t* read_arr = <int8_t*> PyMem_Malloc(read_length*sizeof(char))
 * 
 */
  __pyx_v_read_cstr = __pyx_f_3ssw_6c_util_obj_to_cstr_len(__pyx_v_read, (&__pyx_v_read_length));

  /* "ssw/sswpy.pyx":193
 *         cdef Py_ssize_t read_length
 *         cdef const char* read_cstr = c_util.obj_to_cstr_len(read, &read_length)
 *         cdef int8_t* read_arr = <int8_t*> PyMem_Malloc(read_length*sizeof(char))             # <<<<<<<<<<<<<<
 * 
 *         if self.profile != NULL:
 */
  __pyx_v_read_arr = ((int8_t *)PyMem_Malloc((__pyx_v_read_length * (sizeof(char)))));

  /* "ssw/sswpy.pyx":195
 *         cdef int8_t* read_arr = <int8_t*> PyMem_Malloc(read_length*sizeof(char))
 * 
 *         if self.profile != NULL:             # <<<<<<<<<<<<<<
 *             init_destroy(self.profile)
 *             self.profile = NULL
 */
  __pyx_t_1 = ((__pyx_v_self->profile != NULL) != 0);
  if (__pyx_t_1) {

    /* "ssw/sswpy.pyx":196
 * 
 *         if self.profile != NULL:
 *             init_destroy(self.profile)             # <<<<<<<<<<<<<<
 *             self.profile = NULL
 *         if self.read_arr != NULL:
 */
    init_destroy(__pyx_v_self->profile);

    /* "ssw/sswpy.pyx":197
 *         if self.profile != NULL:
 *             init_destroy(self.profile)
 *             self.profile = NULL             # <<<<<<<<<<<<<<
 *         if self.read_arr != NULL:
 *             PyMem_Free(self.read_arr)
 */
    __pyx_v_self->profile = NULL;

    /* "ssw/sswpy.pyx":195
 *         cdef int8_t* read_arr = <int8_t*> PyMem_Malloc(read_length*sizeof(char))
 * 
 *         if self.profile != NULL:             # <<<<<<<<<<<<<<
 *             init_destroy(self.profile)
 *             self.profile = NULL
 */
  }

  /* "ssw/sswpy.pyx":198
 *             init_destroy(self.profile)
 *             self.profile = NULL
 *         if self.read_arr != NULL:             # <<<<<<<<<<<<<<
 *             PyMem_Free(self.read_arr)
 *             self.read_arr = NULL
 */
  __pyx_t_1 = ((__pyx_v_self->read_arr != NULL) != 0);
  if (__pyx_t_1) {

    /* "ssw/sswpy.pyx":199
 *             self.profile = NULL
 *         if self.read_arr != NULL:
 *             PyMem_Free(self.read_arr)             # <<<<<<<<<<<<<<
 *             self.read_arr = NULL
 * 
 */
    PyMem_Free(__pyx_v_self->read_arr);

    /* "ssw/sswpy.pyx":200
 *         if self.read_arr != NULL:
 *             PyMem_Free(self.read_arr)
 *             self.read_arr = NULL             # <<<<<<<<<<<<<<
 * 
 *         self.read = read
 */
    __pyx_v_self->read_arr = NULL;

    /* "ssw/sswpy.pyx":198
 *             init_destroy(self.profile)
 *             self.profile = NULL
 *         if self.read_arr != NULL:             # <<<<<<<<<<<<<<
 *             PyMem_Free(self.read_arr)
 *             self.read_arr = NULL
 */
  }

  /* "ssw/sswpy.pyx":202
 *             self.read_arr = NULL
 * 
 *         self.read = read             # <<<<<<<<<<<<<<
 *         self.read_arr = read_arr
 *         self.read_length = read_length
 */
  __Pyx_INCREF(__pyx_v_read);
  __Pyx_GIVEREF(__pyx_v_read);
  __Pyx_GOTREF(__pyx_v_self->read);
  __Pyx_DECREF(__pyx_v_self->read);
  __pyx_v_self->read = __pyx_v_read;

  /* "ssw/sswpy.pyx":203
 * 
 *         self.read = read
 *         self.read_arr = read_arr             # <<<<<<<<<<<<<<
 *         self.read_length = read_length
 *         dnaToInt8(read_cstr, read_arr, read_length)
 */
  __pyx_v_self->read_arr = __pyx_v_read_arr;

  /* "ssw/sswpy.pyx":204
 *         self.read = read
 *         self.read_arr = read_arr
 *         self.read_length = read_length             # <<<<<<<<<<<<<<
 *         dnaToInt8(read_cstr, read_arr, read_length)
 * 
 */
  __pyx_v_self->read_length = __pyx_v_read_length;

  /* "ssw/sswpy.pyx":205
 *         self.read_arr = read_arr
 *         self.read_length = read_length
 *         dnaToInt8(read_cstr, read_arr, read_length)             # <<<<<<<<<<<<<<
 * 
 *         self.profile = ssw_init(read_arr,
 */
  dnaToInt8(__pyx_v_read_cstr, __pyx_v_read_arr, __pyx_v_read_length);

  /* "ssw/sswpy.pyx":207
 *         dnaToInt8(read_cstr, read_arr, read_length)
 * 
 *         self.profile = ssw_init(read_arr,             # <<<<<<<<<<<<<<
 *                                 <int32_t> read_length,
 *                                 self.score_matrix,
 */
  __pyx_v_self->profile = ssw_init(__pyx_v_read_arr, ((int32_t)__pyx_v_read_length), __pyx_v_self->score_matrix, 5, 2);

  /* "ssw/sswpy.pyx":184
 *     # end def
 * 
 *     def setRead(self, read: STR_T):             # <<<<<<<<<<<<<<
 *         """ Set the query read string
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ssw/sswpy.pyx":215
 *     # end def
 * 
 *     def setReference(self, reference: STR_T):             # <<<<<<<<<<<<<<
 *         """Set the query reference string
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_3ssw_5sswpy_3SSW_11setReference(PyObject *__pyx_v_self, PyObject *__pyx_v_reference); /*proto*/
static char __pyx_doc_3ssw_5sswpy_3SSW_10setReference[] = "Set the query reference string\n\n        Args:\n            reference:  String-like (str or bytestring) that represents the\n                reference sequence must be set\n        ";
static PyObject *__pyx_pw_3ssw_5sswpy_3SSW_11setReference(PyObject *__pyx_v_self, PyObject *__pyx_v_reference) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("setReference (wrapper)", 0);
  __pyx_r = __pyx_pf_3ssw_5sswpy_3SSW_10setReference(((struct __pyx_obj_3ssw_5sswpy_SSW *)__pyx_v_self), ((PyObject *)__pyx_v_reference));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3ssw_5sswpy_3SSW_10setReference(struct __pyx_obj_3ssw_5sswpy_SSW *__pyx_v_self, PyObject *__pyx_v_reference) {
  Py_ssize_t __pyx_v_ref_length;
  char const *__pyx_v_ref_cstr;
  int8_t *__pyx_v_ref_arr;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("setReference", 0);

  /* "ssw/sswpy.pyx":223
 *         """
 *         cdef Py_ssize_t ref_length
 *         cdef const char* ref_cstr = c_util.obj_to_cstr_len(reference, &ref_length)             # <<<<<<<<<<<<<<
 *         cdef int8_t* ref_arr = <int8_t*> PyMem_Malloc(ref_length*sizeof(char))
 *         dnaToInt8(ref_cstr, ref_arr, ref_length)
 */
  __pyx_v_ref_cstr = __pyx_f_3ssw_6c_util_obj_to_cstr_len(__pyx_v_reference, (&__pyx_v_ref_length));

  /* "ssw/sswpy.pyx":224
 *         cdef Py_ssize_t ref_length
 *         cdef const char* ref_cstr = c_util.obj_to_cstr_len(reference, &ref_length)
 *         cdef int8_t* ref_arr = <int8_t*> PyMem_Malloc(ref_length*sizeof(char))             # <<<<<<<<<<<<<<
 *         dnaToInt8(ref_cstr, ref_arr, ref_length)
 *         self.reference = reference
 */
  __pyx_v_ref_arr = ((int8_t *)PyMem_Malloc((__pyx_v_ref_length * (sizeof(char)))));

  /* "ssw/sswpy.pyx":225
 *         cdef const char* ref_cstr = c_util.obj_to_cstr_len(reference, &ref_length)
 *         cdef int8_t* ref_arr = <int8_t*> PyMem_Malloc(ref_length*sizeof(char))
 *         dnaToInt8(ref_cstr, ref_arr, ref_length)             # <<<<<<<<<<<<<<
 *         self.reference = reference
 *         if self.ref_arr != NULL:
 */
  dnaToInt8(__pyx_v_ref_cstr, __pyx_v_ref_arr, __pyx_v_ref_length);

  /* "ssw/sswpy.pyx":226
 *         cdef int8_t* ref_arr = <int8_t*> PyMem_Malloc(ref_length*sizeof(char))
 *         dnaToInt8(ref_cstr, ref_arr, ref_length)
 *         self.reference = reference             # <<<<<<<<<<<<<<
 *         if self.ref_arr != NULL:
 *             PyMem_Free(self.ref_arr)
 */
  __Pyx_INCREF(__pyx_v_reference);
  __Pyx_GIVEREF(__pyx_v_reference);
  __Pyx_GOTREF(__pyx_v_self->reference);
  __Pyx_DECREF(__pyx_v_self->reference);
  __pyx_v_self->reference = __pyx_v_reference;

  /* "ssw/sswpy.pyx":227
 *         dnaToInt8(ref_cstr, ref_arr, ref_length)
 *         self.reference = reference
 *         if self.ref_arr != NULL:             # <<<<<<<<<<<<<<
 *             PyMem_Free(self.ref_arr)
 *             self.ref_arr = NULL
 */
  __pyx_t_1 = ((__pyx_v_self->ref_arr != NULL) != 0);
  if (__pyx_t_1) {

    /* "ssw/sswpy.pyx":228
 *         self.reference = reference
 *         if self.ref_arr != NULL:
 *             PyMem_Free(self.ref_arr)             # <<<<<<<<<<<<<<
 *             self.ref_arr = NULL
 *         self.ref_arr = ref_arr
 */
    PyMem_Free(__pyx_v_self->ref_arr);

    /* "ssw/sswpy.pyx":229
 *         if self.ref_arr != NULL:
 *             PyMem_Free(self.ref_arr)
 *             self.ref_arr = NULL             # <<<<<<<<<<<<<<
 *         self.ref_arr = ref_arr
 *         self.ref_length = ref_length
 */
    __pyx_v_self->ref_arr = NULL;

    /* "ssw/sswpy.pyx":227
 *         dnaToInt8(ref_cstr, ref_arr, ref_length)
 *         self.reference = reference
 *         if self.ref_arr != NULL:             # <<<<<<<<<<<<<<
 *             PyMem_Free(self.ref_arr)
 *             self.ref_arr = NULL
 */
  }

  /* "ssw/sswpy.pyx":230
 *             PyMem_Free(self.ref_arr)
 *             self.ref_arr = NULL
 *         self.ref_arr = ref_arr             # <<<<<<<<<<<<<<
 *         self.ref_length = ref_length
 *     # end def
 */
  __pyx_v_self->ref_arr = __pyx_v_ref_arr;

  /* "ssw/sswpy.pyx":231
 *             self.ref_arr = NULL
 *         self.ref_arr = ref_arr
 *         self.ref_length = ref_length             # <<<<<<<<<<<<<<
 *     # end def
 * 
 */
  __pyx_v_self->ref_length = __pyx_v_ref_length;

  /* "ssw/sswpy.pyx":215
 *     # end def
 * 
 *     def setReference(self, reference: STR_T):             # <<<<<<<<<<<<<<
 *         """Set the query reference string
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ssw/sswpy.pyx":234
 *     # end def
 * 
 *     cdef s_align* align_c(self,             # <<<<<<<<<<<<<<
 *         int gap_open,
 *         int gap_extension,
 */

static s_align *__pyx_f_3ssw_5sswpy_3SSW_align_c(struct __pyx_obj_3ssw_5sswpy_SSW *__pyx_v_self, int __pyx_v_gap_open, int __pyx_v_gap_extension, Py_ssize_t __pyx_v_start_idx, int32_t __pyx_v_mod_ref_length) {
  Py_ssize_t __pyx_v_read_length;
  CYTHON_UNUSED char const *__pyx_v_read_cstr;
  s_align *__pyx_v_result;
  int32_t __pyx_v_mask_len;
  s_align *__pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int32_t __pyx_t_2;
  int __pyx_t_3;
  __Pyx_RefNannySetupContext("align_c", 0);

  /* "ssw/sswpy.pyx":242
 *         """
 *         cdef Py_ssize_t read_length
 *         cdef const char* read_cstr = c_util.obj_to_cstr_len(self.read, &read_length)             # <<<<<<<<<<<<<<
 *         cdef s_align* result = NULL
 *         cdef int32_t mask_len = read_length / 2
 */
  __pyx_t_1 = __pyx_v_self->read;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_read_cstr = __pyx_f_3ssw_6c_util_obj_to_cstr_len(__pyx_t_1, (&__pyx_v_read_length));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ssw/sswpy.pyx":243
 *         cdef Py_ssize_t read_length
 *         cdef const char* read_cstr = c_util.obj_to_cstr_len(self.read, &read_length)
 *         cdef s_align* result = NULL             # <<<<<<<<<<<<<<
 *         cdef int32_t mask_len = read_length / 2
 * 
 */
  __pyx_v_result = NULL;

  /* "ssw/sswpy.pyx":244
 *         cdef const char* read_cstr = c_util.obj_to_cstr_len(self.read, &read_length)
 *         cdef s_align* result = NULL
 *         cdef int32_t mask_len = read_length / 2             # <<<<<<<<<<<<<<
 * 
 *         mask_len = 15 if mask_len < 15 else mask_len
 */
  __pyx_v_mask_len = __Pyx_div_Py_ssize_t(__pyx_v_read_length, 2);

  /* "ssw/sswpy.pyx":246
 *         cdef int32_t mask_len = read_length / 2
 * 
 *         mask_len = 15 if mask_len < 15 else mask_len             # <<<<<<<<<<<<<<
 * 
 *         if self.profile != NULL:
 */
  if (((__pyx_v_mask_len < 15) != 0)) {
    __pyx_t_2 = 15;
  } else {
    __pyx_t_2 = __pyx_v_mask_len;
  }
  __pyx_v_mask_len = __pyx_t_2;

  /* "ssw/sswpy.pyx":248
 *         mask_len = 15 if mask_len < 15 else mask_len
 * 
 *         if self.profile != NULL:             # <<<<<<<<<<<<<<
 *             result = ssw_align ( self.profile,
 *                                 &self.ref_arr[start_idx],
 */
  __pyx_t_3 = ((__pyx_v_self->profile != NULL) != 0);
  if (__pyx_t_3) {

    /* "ssw/sswpy.pyx":249
 * 
 *         if self.profile != NULL:
 *             result = ssw_align ( self.profile,             # <<<<<<<<<<<<<<
 *                                 &self.ref_arr[start_idx],
 *                                 mod_ref_length,
 */
    __pyx_v_result = ssw_align(__pyx_v_self->profile, (&(__pyx_v_self->ref_arr[__pyx_v_start_idx])), __pyx_v_mod_ref_length, __pyx_v_gap_open, __pyx_v_gap_extension, 1, 0, 0, __pyx_v_mask_len);

    /* "ssw/sswpy.pyx":248
 *         mask_len = 15 if mask_len < 15 else mask_len
 * 
 *         if self.profile != NULL:             # <<<<<<<<<<<<<<
 *             result = ssw_align ( self.profile,
 *                                 &self.ref_arr[start_idx],
 */
    goto __pyx_L3;
  }

  /* "ssw/sswpy.pyx":256
 *                                 1, 0, 0, mask_len)
 *         else:
 *             raise ValueError("Must set profile first")             # <<<<<<<<<<<<<<
 *         if result == NULL:
 *             raise ValueError("Problem Running alignment, see stdout")
 */
  /*else*/ {
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 256, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 256, __pyx_L1_error)
  }
  __pyx_L3:;

  /* "ssw/sswpy.pyx":257
 *         else:
 *             raise ValueError("Must set profile first")
 *         if result == NULL:             # <<<<<<<<<<<<<<
 *             raise ValueError("Problem Running alignment, see stdout")
 *         return result
 */
  __pyx_t_3 = ((__pyx_v_result == NULL) != 0);
  if (__pyx_t_3) {

    /* "ssw/sswpy.pyx":258
 *             raise ValueError("Must set profile first")
 *         if result == NULL:
 *             raise ValueError("Problem Running alignment, see stdout")             # <<<<<<<<<<<<<<
 *         return result
 *     # end def
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 258, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 258, __pyx_L1_error)

    /* "ssw/sswpy.pyx":257
 *         else:
 *             raise ValueError("Must set profile first")
 *         if result == NULL:             # <<<<<<<<<<<<<<
 *             raise ValueError("Problem Running alignment, see stdout")
 *         return result
 */
  }

  /* "ssw/sswpy.pyx":259
 *         if result == NULL:
 *             raise ValueError("Problem Running alignment, see stdout")
 *         return result             # <<<<<<<<<<<<<<
 *     # end def
 * 
 */
  __pyx_r = __pyx_v_result;
  goto __pyx_L0;

  /* "ssw/sswpy.pyx":234
 *     # end def
 * 
 *     cdef s_align* align_c(self,             # <<<<<<<<<<<<<<
 *         int gap_open,
 *         int gap_extension,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ssw.sswpy.SSW.align_c", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ssw/sswpy.pyx":262
 *     # end def
 * 
 *     def align(self,             # <<<<<<<<<<<<<<
 *         int gap_open = 3,
 *         int gap_extension = 1,
 */

/* Python wrapper */
static PyObject *__pyx_pw_3ssw_5sswpy_3SSW_13align(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_3ssw_5sswpy_3SSW_12align[] = "Align a read to the reference with optional index offseting\n\n        returns a dictionary no matter what as align_c can't return\n        NULL\n\n        Args:\n            gap_open (int):         penalty for gap_open. default 3\n            gap_extension (int):    penalty for gap_extension. default 1\n            start_idx (Py_ssize_t): index to start search. default 0\n            end_idx (Py_ssize_t):   index to end search (trying to avoid a target region).\n                                    default 0 means use whole reference length\n\n        Returns:\n            Alignment with keys `CIGAR`,        <for depicting alignment>\n                                `optimal_score`,\n                                `sub-optimal_score`,\n                                `reference_start`, <index into reference>\n                                `reference_end`,   <index into reference>\n                                `read_start`,  <index into read>\n                                `read_end`     <index into read>\n\n        Raises\n            ValueError\n        ";
static PyObject *__pyx_pw_3ssw_5sswpy_3SSW_13align(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_v_gap_open;
  int __pyx_v_gap_extension;
  Py_ssize_t __pyx_v_start_idx;
  Py_ssize_t __pyx_v_end_idx;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("align (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_gap_open,&__pyx_n_s_gap_extension,&__pyx_n_s_start_idx,&__pyx_n_s_end_idx,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_gap_open);
          if (value) { values[0] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_gap_extension);
          if (value) { values[1] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_start_idx);
          if (value) { values[2] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_end_idx);
          if (value) { values[3] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "align") < 0)) __PYX_ERR(0, 262, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    if (values[0]) {
      __pyx_v_gap_open = __Pyx_PyInt_As_int(values[0]); if (unlikely((__pyx_v_gap_open == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 263, __pyx_L3_error)
    } else {
      __pyx_v_gap_open = ((int)3);
    }
    if (values[1]) {
      __pyx_v_gap_extension = __Pyx_PyInt_As_int(values[1]); if (unlikely((__pyx_v_gap_extension == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 264, __pyx_L3_error)
    } else {
      __pyx_v_gap_extension = ((int)1);
    }
    if (values[2]) {
      __pyx_v_start_idx = __Pyx_PyIndex_AsSsize_t(values[2]); if (unlikely((__pyx_v_start_idx == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 265, __pyx_L3_error)
    } else {
      __pyx_v_start_idx = ((Py_ssize_t)0);
    }
    if (values[3]) {
      __pyx_v_end_idx = __Pyx_PyIndex_AsSsize_t(values[3]); if (unlikely((__pyx_v_end_idx == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 266, __pyx_L3_error)
    } else {
      __pyx_v_end_idx = ((Py_ssize_t)0);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("align", 0, 0, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 262, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ssw.sswpy.SSW.align", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_3ssw_5sswpy_3SSW_12align(((struct __pyx_obj_3ssw_5sswpy_SSW *)__pyx_v_self), __pyx_v_gap_open, __pyx_v_gap_extension, __pyx_v_start_idx, __pyx_v_end_idx);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3ssw_5sswpy_3SSW_12align(struct __pyx_obj_3ssw_5sswpy_SSW *__pyx_v_self, int __pyx_v_gap_open, int __pyx_v_gap_extension, Py_ssize_t __pyx_v_start_idx, Py_ssize_t __pyx_v_end_idx) {
  Py_ssize_t __pyx_v_c;
  char __pyx_v_letter;
  int __pyx_v_letter_int;
  uint32_t __pyx_v_length;
  int32_t __pyx_v_search_length;
  Py_ssize_t __pyx_v_end_idx_final;
  PyObject *__pyx_v_err = NULL;
  PyObject *__pyx_v_cigar = NULL;
  s_align *__pyx_v_result;
  PyObject *__pyx_v_out = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  Py_ssize_t __pyx_t_11;
  s_align *__pyx_t_12;
  int32_t __pyx_t_13;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  __Pyx_RefNannySetupContext("align", 0);

  /* "ssw/sswpy.pyx":298
 *         cdef Py_ssize_t end_idx_final
 * 
 *         if start_idx < 0 or end_idx < 0:             # <<<<<<<<<<<<<<
 *             raise ValueError("negative indexing not supported")
 *         if end_idx > self.ref_length or start_idx > self.ref_length:
 */
  __pyx_t_2 = ((__pyx_v_start_idx < 0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_end_idx < 0) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "ssw/sswpy.pyx":299
 * 
 *         if start_idx < 0 or end_idx < 0:
 *             raise ValueError("negative indexing not supported")             # <<<<<<<<<<<<<<
 *         if end_idx > self.ref_length or start_idx > self.ref_length:
 *             err = "start_idx: {} or end_idx: {} can't be greater than ref_length: {}".format(
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__5, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 299, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 299, __pyx_L1_error)

    /* "ssw/sswpy.pyx":298
 *         cdef Py_ssize_t end_idx_final
 * 
 *         if start_idx < 0 or end_idx < 0:             # <<<<<<<<<<<<<<
 *             raise ValueError("negative indexing not supported")
 *         if end_idx > self.ref_length or start_idx > self.ref_length:
 */
  }

  /* "ssw/sswpy.pyx":300
 *         if start_idx < 0 or end_idx < 0:
 *             raise ValueError("negative indexing not supported")
 *         if end_idx > self.ref_length or start_idx > self.ref_length:             # <<<<<<<<<<<<<<
 *             err = "start_idx: {} or end_idx: {} can't be greater than ref_length: {}".format(
 *                                                 start_idx,
 */
  __pyx_t_2 = ((__pyx_v_end_idx > __pyx_v_self->ref_length) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L7_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_start_idx > __pyx_v_self->ref_length) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L7_bool_binop_done:;
  if (__pyx_t_1) {

    /* "ssw/sswpy.pyx":301
 *             raise ValueError("negative indexing not supported")
 *         if end_idx > self.ref_length or start_idx > self.ref_length:
 *             err = "start_idx: {} or end_idx: {} can't be greater than ref_length: {}".format(             # <<<<<<<<<<<<<<
 *                                                 start_idx,
 *                                                 end_idx,
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_start_idx_or_end_idx_can_t_be_gr, __pyx_n_s_format); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 301, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);

    /* "ssw/sswpy.pyx":302
 *         if end_idx > self.ref_length or start_idx > self.ref_length:
 *             err = "start_idx: {} or end_idx: {} can't be greater than ref_length: {}".format(
 *                                                 start_idx,             # <<<<<<<<<<<<<<
 *                                                 end_idx,
 *                                                 self.ref_length)
 */
    __pyx_t_5 = PyInt_FromSsize_t(__pyx_v_start_idx); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 302, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);

    /* "ssw/sswpy.pyx":303
 *             err = "start_idx: {} or end_idx: {} can't be greater than ref_length: {}".format(
 *                                                 start_idx,
 *                                                 end_idx,             # <<<<<<<<<<<<<<
 *                                                 self.ref_length)
 *             raise ValueError(err)
 */
    __pyx_t_6 = PyInt_FromSsize_t(__pyx_v_end_idx); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 303, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);

    /* "ssw/sswpy.pyx":304
 *                                                 start_idx,
 *                                                 end_idx,
 *                                                 self.ref_length)             # <<<<<<<<<<<<<<
 *             raise ValueError(err)
 *         if end_idx == 0:
 */
    __pyx_t_7 = PyInt_FromSsize_t(__pyx_v_self->ref_length); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 304, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = NULL;
    __pyx_t_9 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
        __pyx_t_9 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[4] = {__pyx_t_8, __pyx_t_5, __pyx_t_6, __pyx_t_7};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_9, 3+__pyx_t_9); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 301, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[4] = {__pyx_t_8, __pyx_t_5, __pyx_t_6, __pyx_t_7};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_9, 3+__pyx_t_9); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 301, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    } else
    #endif
    {
      __pyx_t_10 = PyTuple_New(3+__pyx_t_9); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 301, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      if (__pyx_t_8) {
        __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_8); __pyx_t_8 = NULL;
      }
      __Pyx_GIVEREF(__pyx_t_5);
      PyTuple_SET_ITEM(__pyx_t_10, 0+__pyx_t_9, __pyx_t_5);
      __Pyx_GIVEREF(__pyx_t_6);
      PyTuple_SET_ITEM(__pyx_t_10, 1+__pyx_t_9, __pyx_t_6);
      __Pyx_GIVEREF(__pyx_t_7);
      PyTuple_SET_ITEM(__pyx_t_10, 2+__pyx_t_9, __pyx_t_7);
      __pyx_t_5 = 0;
      __pyx_t_6 = 0;
      __pyx_t_7 = 0;
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_10, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 301, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_err = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "ssw/sswpy.pyx":305
 *                                                 end_idx,
 *                                                 self.ref_length)
 *             raise ValueError(err)             # <<<<<<<<<<<<<<
 *         if end_idx == 0:
 *             end_idx_final = self.ref_length
 */
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 305, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v_err);
    __Pyx_GIVEREF(__pyx_v_err);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_err);
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_t_3, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 305, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 305, __pyx_L1_error)

    /* "ssw/sswpy.pyx":300
 *         if start_idx < 0 or end_idx < 0:
 *             raise ValueError("negative indexing not supported")
 *         if end_idx > self.ref_length or start_idx > self.ref_length:             # <<<<<<<<<<<<<<
 *             err = "start_idx: {} or end_idx: {} can't be greater than ref_length: {}".format(
 *                                                 start_idx,
 */
  }

  /* "ssw/sswpy.pyx":306
 *                                                 self.ref_length)
 *             raise ValueError(err)
 *         if end_idx == 0:             # <<<<<<<<<<<<<<
 *             end_idx_final = self.ref_length
 *         else:
 */
  __pyx_t_1 = ((__pyx_v_end_idx == 0) != 0);
  if (__pyx_t_1) {

    /* "ssw/sswpy.pyx":307
 *             raise ValueError(err)
 *         if end_idx == 0:
 *             end_idx_final = self.ref_length             # <<<<<<<<<<<<<<
 *         else:
 *             end_idx_final = end_idx
 */
    __pyx_t_11 = __pyx_v_self->ref_length;
    __pyx_v_end_idx_final = __pyx_t_11;

    /* "ssw/sswpy.pyx":306
 *                                                 self.ref_length)
 *             raise ValueError(err)
 *         if end_idx == 0:             # <<<<<<<<<<<<<<
 *             end_idx_final = self.ref_length
 *         else:
 */
    goto __pyx_L9;
  }

  /* "ssw/sswpy.pyx":309
 *             end_idx_final = self.ref_length
 *         else:
 *             end_idx_final = end_idx             # <<<<<<<<<<<<<<
 *         search_length = end_idx_final - start_idx
 * 
 */
  /*else*/ {
    __pyx_v_end_idx_final = __pyx_v_end_idx;
  }
  __pyx_L9:;

  /* "ssw/sswpy.pyx":310
 *         else:
 *             end_idx_final = end_idx
 *         search_length = end_idx_final - start_idx             # <<<<<<<<<<<<<<
 * 
 *         cigar = None
 */
  __pyx_v_search_length = (__pyx_v_end_idx_final - __pyx_v_start_idx);

  /* "ssw/sswpy.pyx":312
 *         search_length = end_idx_final - start_idx
 * 
 *         cigar = None             # <<<<<<<<<<<<<<
 * 
 *         if self.reference is None:
 */
  __Pyx_INCREF(Py_None);
  __pyx_v_cigar = Py_None;

  /* "ssw/sswpy.pyx":314
 *         cigar = None
 * 
 *         if self.reference is None:             # <<<<<<<<<<<<<<
 *             raise ValueError("call setReference first")
 * 
 */
  __pyx_t_1 = (__pyx_v_self->reference == Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "ssw/sswpy.pyx":315
 * 
 *         if self.reference is None:
 *             raise ValueError("call setReference first")             # <<<<<<<<<<<<<<
 * 
 *         cdef s_align* result =  self.align_c(gap_open, gap_extension, start_idx, search_length)
 */
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__6, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 315, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 315, __pyx_L1_error)

    /* "ssw/sswpy.pyx":314
 *         cigar = None
 * 
 *         if self.reference is None:             # <<<<<<<<<<<<<<
 *             raise ValueError("call setReference first")
 * 
 */
  }

  /* "ssw/sswpy.pyx":317
 *             raise ValueError("call setReference first")
 * 
 *         cdef s_align* result =  self.align_c(gap_open, gap_extension, start_idx, search_length)             # <<<<<<<<<<<<<<
 *         if result.cigar != NULL:
 *             cigar = ""
 */
  __pyx_t_12 = ((struct __pyx_vtabstruct_3ssw_5sswpy_SSW *)__pyx_v_self->__pyx_vtab)->align_c(__pyx_v_self, __pyx_v_gap_open, __pyx_v_gap_extension, __pyx_v_start_idx, __pyx_v_search_length); if (unlikely(__pyx_t_12 == ((s_align *)NULL))) __PYX_ERR(0, 317, __pyx_L1_error)
  __pyx_v_result = __pyx_t_12;

  /* "ssw/sswpy.pyx":318
 * 
 *         cdef s_align* result =  self.align_c(gap_open, gap_extension, start_idx, search_length)
 *         if result.cigar != NULL:             # <<<<<<<<<<<<<<
 *             cigar = ""
 *             for c in range(result.cigarLen):
 */
  __pyx_t_2 = ((__pyx_v_result->cigar != NULL) != 0);
  if (__pyx_t_2) {

    /* "ssw/sswpy.pyx":319
 *         cdef s_align* result =  self.align_c(gap_open, gap_extension, start_idx, search_length)
 *         if result.cigar != NULL:
 *             cigar = ""             # <<<<<<<<<<<<<<
 *             for c in range(result.cigarLen):
 *                 letter = cigar_int_to_op(result.cigar[c])
 */
    __Pyx_INCREF(__pyx_kp_s__7);
    __Pyx_DECREF_SET(__pyx_v_cigar, __pyx_kp_s__7);

    /* "ssw/sswpy.pyx":320
 *         if result.cigar != NULL:
 *             cigar = ""
 *             for c in range(result.cigarLen):             # <<<<<<<<<<<<<<
 *                 letter = cigar_int_to_op(result.cigar[c])
 *                 letter_int = letter
 */
    __pyx_t_13 = __pyx_v_result->cigarLen;
    for (__pyx_t_11 = 0; __pyx_t_11 < __pyx_t_13; __pyx_t_11+=1) {
      __pyx_v_c = __pyx_t_11;

      /* "ssw/sswpy.pyx":321
 *             cigar = ""
 *             for c in range(result.cigarLen):
 *                 letter = cigar_int_to_op(result.cigar[c])             # <<<<<<<<<<<<<<
 *                 letter_int = letter
 *                 length = cigar_int_to_len(result.cigar[c])
 */
      __pyx_v_letter = cigar_int_to_op((__pyx_v_result->cigar[__pyx_v_c]));

      /* "ssw/sswpy.pyx":322
 *             for c in range(result.cigarLen):
 *                 letter = cigar_int_to_op(result.cigar[c])
 *                 letter_int = letter             # <<<<<<<<<<<<<<
 *                 length = cigar_int_to_len(result.cigar[c])
 *                 cigar += "%d%s" % (<int>length, chr(letter_int))
 */
      __pyx_v_letter_int = __pyx_v_letter;

      /* "ssw/sswpy.pyx":323
 *                 letter = cigar_int_to_op(result.cigar[c])
 *                 letter_int = letter
 *                 length = cigar_int_to_len(result.cigar[c])             # <<<<<<<<<<<<<<
 *                 cigar += "%d%s" % (<int>length, chr(letter_int))
 *         out = Alignment(
 */
      __pyx_v_length = cigar_int_to_len((__pyx_v_result->cigar[__pyx_v_c]));

      /* "ssw/sswpy.pyx":324
 *                 letter_int = letter
 *                 length = cigar_int_to_len(result.cigar[c])
 *                 cigar += "%d%s" % (<int>length, chr(letter_int))             # <<<<<<<<<<<<<<
 *         out = Alignment(
 *                 cigar,
 */
      __pyx_t_4 = __Pyx_PyInt_From_int(((int)__pyx_v_length)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 324, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_letter_int); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 324, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_10 = PyTuple_New(1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 324, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_GIVEREF(__pyx_t_3);
      PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_3);
      __pyx_t_3 = 0;
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_chr, __pyx_t_10, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 324, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __pyx_t_10 = PyTuple_New(2); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 324, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_GIVEREF(__pyx_t_4);
      PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3);
      PyTuple_SET_ITEM(__pyx_t_10, 1, __pyx_t_3);
      __pyx_t_4 = 0;
      __pyx_t_3 = 0;
      __pyx_t_3 = __Pyx_PyString_Format(__pyx_kp_s_d_s, __pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 324, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __pyx_t_10 = PyNumber_InPlaceAdd(__pyx_v_cigar, __pyx_t_3); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 324, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF_SET(__pyx_v_cigar, __pyx_t_10);
      __pyx_t_10 = 0;
    }

    /* "ssw/sswpy.pyx":318
 * 
 *         cdef s_align* result =  self.align_c(gap_open, gap_extension, start_idx, search_length)
 *         if result.cigar != NULL:             # <<<<<<<<<<<<<<
 *             cigar = ""
 *             for c in range(result.cigarLen):
 */
  }

  /* "ssw/sswpy.pyx":325
 *                 length = cigar_int_to_len(result.cigar[c])
 *                 cigar += "%d%s" % (<int>length, chr(letter_int))
 *         out = Alignment(             # <<<<<<<<<<<<<<
 *                 cigar,
 *                 result.score1,
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_Alignment); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 325, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "ssw/sswpy.pyx":327
 *         out = Alignment(
 *                 cigar,
 *                 result.score1,             # <<<<<<<<<<<<<<
 *                 result.score2,
 *                 result.ref_begin1,
 */
  __pyx_t_4 = __Pyx_PyInt_From_uint16_t(__pyx_v_result->score1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 327, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);

  /* "ssw/sswpy.pyx":328
 *                 cigar,
 *                 result.score1,
 *                 result.score2,             # <<<<<<<<<<<<<<
 *                 result.ref_begin1,
 *                 result.ref_end1,
 */
  __pyx_t_7 = __Pyx_PyInt_From_uint16_t(__pyx_v_result->score2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 328, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);

  /* "ssw/sswpy.pyx":329
 *                 result.score1,
 *                 result.score2,
 *                 result.ref_begin1,             # <<<<<<<<<<<<<<
 *                 result.ref_end1,
 *                 result.read_begin1,
 */
  __pyx_t_6 = __Pyx_PyInt_From_int32_t(__pyx_v_result->ref_begin1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 329, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);

  /* "ssw/sswpy.pyx":330
 *                 result.score2,
 *                 result.ref_begin1,
 *                 result.ref_end1,             # <<<<<<<<<<<<<<
 *                 result.read_begin1,
 *                 result.read_end1
 */
  __pyx_t_5 = __Pyx_PyInt_From_int32_t(__pyx_v_result->ref_end1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 330, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);

  /* "ssw/sswpy.pyx":331
 *                 result.ref_begin1,
 *                 result.ref_end1,
 *                 result.read_begin1,             # <<<<<<<<<<<<<<
 *                 result.read_end1
 *         )
 */
  __pyx_t_8 = __Pyx_PyInt_From_int32_t(__pyx_v_result->read_begin1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 331, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);

  /* "ssw/sswpy.pyx":332
 *                 result.ref_end1,
 *                 result.read_begin1,
 *                 result.read_end1             # <<<<<<<<<<<<<<
 *         )
 *         #print("RAW BEGIN")
 */
  __pyx_t_14 = __Pyx_PyInt_From_int32_t(__pyx_v_result->read_end1); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 332, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __pyx_t_15 = NULL;
  __pyx_t_9 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_15)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_15);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_9 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[8] = {__pyx_t_15, __pyx_v_cigar, __pyx_t_4, __pyx_t_7, __pyx_t_6, __pyx_t_5, __pyx_t_8, __pyx_t_14};
    __pyx_t_10 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_9, 7+__pyx_t_9); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 325, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[8] = {__pyx_t_15, __pyx_v_cigar, __pyx_t_4, __pyx_t_7, __pyx_t_6, __pyx_t_5, __pyx_t_8, __pyx_t_14};
    __pyx_t_10 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_9, 7+__pyx_t_9); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 325, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
  } else
  #endif
  {
    __pyx_t_16 = PyTuple_New(7+__pyx_t_9); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 325, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_16);
    if (__pyx_t_15) {
      __Pyx_GIVEREF(__pyx_t_15); PyTuple_SET_ITEM(__pyx_t_16, 0, __pyx_t_15); __pyx_t_15 = NULL;
    }
    __Pyx_INCREF(__pyx_v_cigar);
    __Pyx_GIVEREF(__pyx_v_cigar);
    PyTuple_SET_ITEM(__pyx_t_16, 0+__pyx_t_9, __pyx_v_cigar);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_16, 1+__pyx_t_9, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_16, 2+__pyx_t_9, __pyx_t_7);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_16, 3+__pyx_t_9, __pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_16, 4+__pyx_t_9, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_16, 5+__pyx_t_9, __pyx_t_8);
    __Pyx_GIVEREF(__pyx_t_14);
    PyTuple_SET_ITEM(__pyx_t_16, 6+__pyx_t_9, __pyx_t_14);
    __pyx_t_4 = 0;
    __pyx_t_7 = 0;
    __pyx_t_6 = 0;
    __pyx_t_5 = 0;
    __pyx_t_8 = 0;
    __pyx_t_14 = 0;
    __pyx_t_10 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_16, NULL); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 325, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_out = __pyx_t_10;
  __pyx_t_10 = 0;

  /* "ssw/sswpy.pyx":337
 *         #self.printResult_c(result)
 *         #print("RAW END")
 *         align_destroy(result)             # <<<<<<<<<<<<<<
 *         return out
 *     # end def
 */
  align_destroy(__pyx_v_result);

  /* "ssw/sswpy.pyx":338
 *         #print("RAW END")
 *         align_destroy(result)
 *         return out             # <<<<<<<<<<<<<<
 *     # end def
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_out);
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "ssw/sswpy.pyx":262
 *     # end def
 * 
 *     def align(self,             # <<<<<<<<<<<<<<
 *         int gap_open = 3,
 *         int gap_extension = 1,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_AddTraceback("ssw.sswpy.SSW.align", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_err);
  __Pyx_XDECREF(__pyx_v_cigar);
  __Pyx_XDECREF(__pyx_v_out);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ssw/sswpy.pyx":341
 *     # end def
 * 
 *     cdef int buildDNAScoreMatrix(self,             # <<<<<<<<<<<<<<
 *         const uint8_t match_score,
 *         const uint8_t mismatch_penalty,
 */

static int __pyx_f_3ssw_5sswpy_3SSW_buildDNAScoreMatrix(CYTHON_UNUSED struct __pyx_obj_3ssw_5sswpy_SSW *__pyx_v_self, uint8_t const __pyx_v_match_score, uint8_t const __pyx_v_mismatch_penalty, int8_t *__pyx_v_matrix) {
  Py_ssize_t __pyx_v_i;
  Py_ssize_t __pyx_v_j;
  Py_ssize_t __pyx_v_idx;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  int __pyx_t_3;
  __Pyx_RefNannySetupContext("buildDNAScoreMatrix", 0);

  /* "ssw/sswpy.pyx":358
 *         """
 *         cdef Py_ssize_t i, j
 *         cdef Py_ssize_t idx = 0;             # <<<<<<<<<<<<<<
 *         for i in range(4):
 *             for j in range(4):
 */
  __pyx_v_idx = 0;

  /* "ssw/sswpy.pyx":359
 *         cdef Py_ssize_t i, j
 *         cdef Py_ssize_t idx = 0;
 *         for i in range(4):             # <<<<<<<<<<<<<<
 *             for j in range(4):
 *                 if i == j:
 */
  for (__pyx_t_1 = 0; __pyx_t_1 < 4; __pyx_t_1+=1) {
    __pyx_v_i = __pyx_t_1;

    /* "ssw/sswpy.pyx":360
 *         cdef Py_ssize_t idx = 0;
 *         for i in range(4):
 *             for j in range(4):             # <<<<<<<<<<<<<<
 *                 if i == j:
 *                     matrix[idx] =  <int8_t> match_score
 */
    for (__pyx_t_2 = 0; __pyx_t_2 < 4; __pyx_t_2+=1) {
      __pyx_v_j = __pyx_t_2;

      /* "ssw/sswpy.pyx":361
 *         for i in range(4):
 *             for j in range(4):
 *                 if i == j:             # <<<<<<<<<<<<<<
 *                     matrix[idx] =  <int8_t> match_score
 *                 else:
 */
      __pyx_t_3 = ((__pyx_v_i == __pyx_v_j) != 0);
      if (__pyx_t_3) {

        /* "ssw/sswpy.pyx":362
 *             for j in range(4):
 *                 if i == j:
 *                     matrix[idx] =  <int8_t> match_score             # <<<<<<<<<<<<<<
 *                 else:
 *                     matrix[idx] = <int8_t> (-mismatch_penalty)
 */
        (__pyx_v_matrix[__pyx_v_idx]) = ((int8_t)__pyx_v_match_score);

        /* "ssw/sswpy.pyx":361
 *         for i in range(4):
 *             for j in range(4):
 *                 if i == j:             # <<<<<<<<<<<<<<
 *                     matrix[idx] =  <int8_t> match_score
 *                 else:
 */
        goto __pyx_L7;
      }

      /* "ssw/sswpy.pyx":364
 *                     matrix[idx] =  <int8_t> match_score
 *                 else:
 *                     matrix[idx] = <int8_t> (-mismatch_penalty)             # <<<<<<<<<<<<<<
 *                 inc(idx)
 *             matrix[idx] = 0;
 */
      /*else*/ {
        (__pyx_v_matrix[__pyx_v_idx]) = ((int8_t)(-__pyx_v_mismatch_penalty));
      }
      __pyx_L7:;

      /* "ssw/sswpy.pyx":365
 *                 else:
 *                     matrix[idx] = <int8_t> (-mismatch_penalty)
 *                 inc(idx)             # <<<<<<<<<<<<<<
 *             matrix[idx] = 0;
 *             inc(idx)
 */
      (__pyx_v_idx++);
    }

    /* "ssw/sswpy.pyx":366
 *                     matrix[idx] = <int8_t> (-mismatch_penalty)
 *                 inc(idx)
 *             matrix[idx] = 0;             # <<<<<<<<<<<<<<
 *             inc(idx)
 *         for i in range(5):
 */
    (__pyx_v_matrix[__pyx_v_idx]) = 0;

    /* "ssw/sswpy.pyx":367
 *                 inc(idx)
 *             matrix[idx] = 0;
 *             inc(idx)             # <<<<<<<<<<<<<<
 *         for i in range(5):
 *             matrix[inc(idx)] = 0
 */
    (__pyx_v_idx++);
  }

  /* "ssw/sswpy.pyx":368
 *             matrix[idx] = 0;
 *             inc(idx)
 *         for i in range(5):             # <<<<<<<<<<<<<<
 *             matrix[inc(idx)] = 0
 *         return 0
 */
  for (__pyx_t_1 = 0; __pyx_t_1 < 5; __pyx_t_1+=1) {
    __pyx_v_i = __pyx_t_1;

    /* "ssw/sswpy.pyx":369
 *             inc(idx)
 *         for i in range(5):
 *             matrix[inc(idx)] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     # end def
 */
    (__pyx_v_matrix[(__pyx_v_idx++)]) = 0;
  }

  /* "ssw/sswpy.pyx":370
 *         for i in range(5):
 *             matrix[inc(idx)] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     # end def
 * # end class
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "ssw/sswpy.pyx":341
 *     # end def
 * 
 *     cdef int buildDNAScoreMatrix(self,             # <<<<<<<<<<<<<<
 *         const uint8_t match_score,
 *         const uint8_t mismatch_penalty,
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_3ssw_5sswpy_3SSW_15__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_3ssw_5sswpy_3SSW_15__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_3ssw_5sswpy_3SSW_14__reduce_cython__(((struct __pyx_obj_3ssw_5sswpy_SSW *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3ssw_5sswpy_3SSW_14__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_3ssw_5sswpy_SSW *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__8, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ssw.sswpy.SSW.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_3ssw_5sswpy_3SSW_17__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_3ssw_5sswpy_3SSW_17__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_3ssw_5sswpy_3SSW_16__setstate_cython__(((struct __pyx_obj_3ssw_5sswpy_SSW *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3ssw_5sswpy_3SSW_16__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_3ssw_5sswpy_SSW *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__9, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ssw.sswpy.SSW.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ssw/sswpy.pyx":374
 * # end class
 * 
 * def force_align( read: STR_T,             # <<<<<<<<<<<<<<
 *                 reference: STR_T,
 *                 force_overhang: bool = False,
 */

/* Python wrapper */
static PyObject *__pyx_pw_3ssw_5sswpy_1force_align(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_3ssw_5sswpy_force_align[] = "Enforces no gaps by raising the ``gap_open`` penalty\n\n    Args:\n        read:\n        reference:\n        force_overhang: Make sure only one end overhangs\n        aligner: pass an existing :class:`SSW` object\n    Raises:\n        ValueError for no solution found\n    ";
static PyMethodDef __pyx_mdef_3ssw_5sswpy_1force_align = {"force_align", (PyCFunction)__pyx_pw_3ssw_5sswpy_1force_align, METH_VARARGS|METH_KEYWORDS, __pyx_doc_3ssw_5sswpy_force_align};
static PyObject *__pyx_pw_3ssw_5sswpy_1force_align(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_read = 0;
  PyObject *__pyx_v_reference = 0;
  PyObject *__pyx_v_force_overhang = 0;
  struct __pyx_obj_3ssw_5sswpy_SSW *__pyx_v_aligner = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("force_align (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_read,&__pyx_n_s_reference,&__pyx_n_s_force_overhang,&__pyx_n_s_aligner,0};
    PyObject* values[4] = {0,0,0,0};

    /* "ssw/sswpy.pyx":376
 * def force_align( read: STR_T,
 *                 reference: STR_T,
 *                 force_overhang: bool = False,             # <<<<<<<<<<<<<<
 *                 aligner: SSW = None) -> Alignment:
 *     '''Enforces no gaps by raising the ``gap_open`` penalty
 */
    values[2] = ((PyObject *)Py_False);

    /* "ssw/sswpy.pyx":377
 *                 reference: STR_T,
 *                 force_overhang: bool = False,
 *                 aligner: SSW = None) -> Alignment:             # <<<<<<<<<<<<<<
 *     '''Enforces no gaps by raising the ``gap_open`` penalty
 * 
 */
    values[3] = (PyObject *)((struct __pyx_obj_3ssw_5sswpy_SSW *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_read)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_reference)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("force_align", 0, 2, 4, 1); __PYX_ERR(0, 374, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_force_overhang);
          if (value) { values[2] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_aligner);
          if (value) { values[3] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "force_align") < 0)) __PYX_ERR(0, 374, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_read = values[0];
    __pyx_v_reference = values[1];
    __pyx_v_force_overhang = values[2];
    __pyx_v_aligner = ((struct __pyx_obj_3ssw_5sswpy_SSW *)values[3]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("force_align", 0, 2, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 374, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ssw.sswpy.force_align", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_aligner), __pyx_ptype_3ssw_5sswpy_SSW, 1, "aligner", 0))) __PYX_ERR(0, 377, __pyx_L1_error)
  __pyx_r = __pyx_pf_3ssw_5sswpy_force_align(__pyx_self, __pyx_v_read, __pyx_v_reference, __pyx_v_force_overhang, __pyx_v_aligner);

  /* "ssw/sswpy.pyx":374
 * # end class
 * 
 * def force_align( read: STR_T,             # <<<<<<<<<<<<<<
 *                 reference: STR_T,
 *                 force_overhang: bool = False,
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3ssw_5sswpy_force_align(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_read, PyObject *__pyx_v_reference, PyObject *__pyx_v_force_overhang, struct __pyx_obj_3ssw_5sswpy_SSW *__pyx_v_aligner) {
  struct __pyx_obj_3ssw_5sswpy_SSW *__pyx_v_a = 0;
  PyObject *__pyx_v_len_x = 0;
  PyObject *__pyx_v_res = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  Py_ssize_t __pyx_t_6;
  int __pyx_t_7;
  __Pyx_RefNannySetupContext("force_align", 0);

  /* "ssw/sswpy.pyx":388
 *         ValueError for no solution found
 *     '''
 *     a: SSW = SSW() if aligner is None else aligner             # <<<<<<<<<<<<<<
 *     a.setRead(read)
 *     a.setReference(reference)
 */
  __pyx_t_2 = (((PyObject *)__pyx_v_aligner) == Py_None);
  if ((__pyx_t_2 != 0)) {
    __pyx_t_3 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_3ssw_5sswpy_SSW), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 388, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = __pyx_t_3;
    __pyx_t_3 = 0;
  } else {
    __Pyx_INCREF(((PyObject *)__pyx_v_aligner));
    __pyx_t_1 = ((PyObject *)__pyx_v_aligner);
  }
  __pyx_v_a = ((struct __pyx_obj_3ssw_5sswpy_SSW *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ssw/sswpy.pyx":389
 *     '''
 *     a: SSW = SSW() if aligner is None else aligner
 *     a.setRead(read)             # <<<<<<<<<<<<<<
 *     a.setReference(reference)
 *     len_x: int = len(read)
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_a), __pyx_n_s_setRead); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 389, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_read); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 389, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_v_read};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 389, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_v_read};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 389, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 389, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
      __Pyx_INCREF(__pyx_v_read);
      __Pyx_GIVEREF(__pyx_v_read);
      PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_v_read);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 389, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ssw/sswpy.pyx":390
 *     a: SSW = SSW() if aligner is None else aligner
 *     a.setRead(read)
 *     a.setReference(reference)             # <<<<<<<<<<<<<<
 *     len_x: int = len(read)
 *     # set the gap_open penalty high to drop all splits in alignment
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_a), __pyx_n_s_setReference); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 390, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_5) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_reference); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 390, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_v_reference};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 390, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_v_reference};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 390, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 390, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5); __pyx_t_5 = NULL;
      __Pyx_INCREF(__pyx_v_reference);
      __Pyx_GIVEREF(__pyx_v_reference);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_reference);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 390, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ssw/sswpy.pyx":391
 *     a.setRead(read)
 *     a.setReference(reference)
 *     len_x: int = len(read)             # <<<<<<<<<<<<<<
 *     # set the gap_open penalty high to drop all splits in alignment
 *     # pick the first max hit
 */
  __pyx_t_6 = PyObject_Length(__pyx_v_read); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(0, 391, __pyx_L1_error)
  __pyx_t_1 = PyInt_FromSsize_t(__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 391, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_len_x = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "ssw/sswpy.pyx":394
 *     # set the gap_open penalty high to drop all splits in alignment
 *     # pick the first max hit
 *     res: Alignment = a.align(gap_open=len_x)             # <<<<<<<<<<<<<<
 *     if res.optimal_score < 4:
 *         raise ValueError("No solution found")
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_a), __pyx_n_s_align); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 394, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 394, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_gap_open, __pyx_v_len_x) < 0) __PYX_ERR(0, 394, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 394, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_res = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "ssw/sswpy.pyx":395
 *     # pick the first max hit
 *     res: Alignment = a.align(gap_open=len_x)
 *     if res.optimal_score < 4:             # <<<<<<<<<<<<<<
 *         raise ValueError("No solution found")
 *     if force_overhang:
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_res, __pyx_n_s_optimal_score); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 395, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = PyObject_RichCompare(__pyx_t_4, __pyx_int_4, Py_LT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 395, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 395, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_2) {

    /* "ssw/sswpy.pyx":396
 *     res: Alignment = a.align(gap_open=len_x)
 *     if res.optimal_score < 4:
 *         raise ValueError("No solution found")             # <<<<<<<<<<<<<<
 *     if force_overhang:
 *         # read must align to either the beginning or end of the reference string
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__10, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 396, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 396, __pyx_L1_error)

    /* "ssw/sswpy.pyx":395
 *     # pick the first max hit
 *     res: Alignment = a.align(gap_open=len_x)
 *     if res.optimal_score < 4:             # <<<<<<<<<<<<<<
 *         raise ValueError("No solution found")
 *     if force_overhang:
 */
  }

  /* "ssw/sswpy.pyx":397
 *     if res.optimal_score < 4:
 *         raise ValueError("No solution found")
 *     if force_overhang:             # <<<<<<<<<<<<<<
 *         # read must align to either the beginning or end of the reference string
 *         if (res.reference_start != 0 or
 */
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_force_overhang); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 397, __pyx_L1_error)
  if (__pyx_t_2) {

    /* "ssw/sswpy.pyx":399
 *     if force_overhang:
 *         # read must align to either the beginning or end of the reference string
 *         if (res.reference_start != 0 or             # <<<<<<<<<<<<<<
 *             res.reference_end != len(reference) - 1):
 *             raise ValueError("Read does not align to one overhang")
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_res, __pyx_n_s_reference_start); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 399, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyObject_RichCompare(__pyx_t_3, __pyx_int_0, Py_NE); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 399, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 399, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (!__pyx_t_7) {
    } else {
      __pyx_t_2 = __pyx_t_7;
      goto __pyx_L6_bool_binop_done;
    }

    /* "ssw/sswpy.pyx":400
 *         # read must align to either the beginning or end of the reference string
 *         if (res.reference_start != 0 or
 *             res.reference_end != len(reference) - 1):             # <<<<<<<<<<<<<<
 *             raise ValueError("Read does not align to one overhang")
 *     return res
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_res, __pyx_n_s_reference_end); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 400, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = PyObject_Length(__pyx_v_reference); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(0, 400, __pyx_L1_error)
    __pyx_t_3 = PyInt_FromSsize_t((__pyx_t_6 - 1)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 400, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = PyObject_RichCompare(__pyx_t_4, __pyx_t_3, Py_NE); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 400, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 400, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_2 = __pyx_t_7;
    __pyx_L6_bool_binop_done:;

    /* "ssw/sswpy.pyx":399
 *     if force_overhang:
 *         # read must align to either the beginning or end of the reference string
 *         if (res.reference_start != 0 or             # <<<<<<<<<<<<<<
 *             res.reference_end != len(reference) - 1):
 *             raise ValueError("Read does not align to one overhang")
 */
    if (__pyx_t_2) {

      /* "ssw/sswpy.pyx":401
 *         if (res.reference_start != 0 or
 *             res.reference_end != len(reference) - 1):
 *             raise ValueError("Read does not align to one overhang")             # <<<<<<<<<<<<<<
 *     return res
 * # end def
 */
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__11, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 401, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_Raise(__pyx_t_1, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __PYX_ERR(0, 401, __pyx_L1_error)

      /* "ssw/sswpy.pyx":399
 *     if force_overhang:
 *         # read must align to either the beginning or end of the reference string
 *         if (res.reference_start != 0 or             # <<<<<<<<<<<<<<
 *             res.reference_end != len(reference) - 1):
 *             raise ValueError("Read does not align to one overhang")
 */
    }

    /* "ssw/sswpy.pyx":397
 *     if res.optimal_score < 4:
 *         raise ValueError("No solution found")
 *     if force_overhang:             # <<<<<<<<<<<<<<
 *         # read must align to either the beginning or end of the reference string
 *         if (res.reference_start != 0 or
 */
  }

  /* "ssw/sswpy.pyx":402
 *             res.reference_end != len(reference) - 1):
 *             raise ValueError("Read does not align to one overhang")
 *     return res             # <<<<<<<<<<<<<<
 * # end def
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_res);
  __pyx_r = __pyx_v_res;
  goto __pyx_L0;

  /* "ssw/sswpy.pyx":374
 * # end class
 * 
 * def force_align( read: STR_T,             # <<<<<<<<<<<<<<
 *                 reference: STR_T,
 *                 force_overhang: bool = False,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ssw.sswpy.force_align", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_a);
  __Pyx_XDECREF(__pyx_v_len_x);
  __Pyx_XDECREF(__pyx_v_res);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ssw/sswpy.pyx":405
 * # end def
 * 
 * def format_force_align(  read: STR_T,             # <<<<<<<<<<<<<<
 *                         reference: STR_T,
 *                         alignment: Alignment,
 */

/* Python wrapper */
static PyObject *__pyx_pw_3ssw_5sswpy_3format_force_align(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_3ssw_5sswpy_2format_force_align[] = "Does not truncate strings\n\n    Args:\n        read:\n        reference:\n        alignment:\n        do_print: default is False\n    ";
static PyMethodDef __pyx_mdef_3ssw_5sswpy_3format_force_align = {"format_force_align", (PyCFunction)__pyx_pw_3ssw_5sswpy_3format_force_align, METH_VARARGS|METH_KEYWORDS, __pyx_doc_3ssw_5sswpy_2format_force_align};
static PyObject *__pyx_pw_3ssw_5sswpy_3format_force_align(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_read = 0;
  PyObject *__pyx_v_reference = 0;
  PyObject *__pyx_v_alignment = 0;
  PyObject *__pyx_v_do_print = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("format_force_align (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_read,&__pyx_n_s_reference,&__pyx_n_s_alignment,&__pyx_n_s_do_print,0};
    PyObject* values[4] = {0,0,0,0};

    /* "ssw/sswpy.pyx":408
 *                         reference: STR_T,
 *                         alignment: Alignment,
 *                         do_print: bool = False):             # <<<<<<<<<<<<<<
 *     '''Does not truncate strings
 * 
 */
    values[3] = ((PyObject *)Py_False);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_read)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_reference)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("format_force_align", 0, 3, 4, 1); __PYX_ERR(0, 405, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_alignment)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("format_force_align", 0, 3, 4, 2); __PYX_ERR(0, 405, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_do_print);
          if (value) { values[3] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "format_force_align") < 0)) __PYX_ERR(0, 405, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_read = values[0];
    __pyx_v_reference = values[1];
    __pyx_v_alignment = values[2];
    __pyx_v_do_print = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("format_force_align", 0, 3, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 405, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ssw.sswpy.format_force_align", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_3ssw_5sswpy_2format_force_align(__pyx_self, __pyx_v_read, __pyx_v_reference, __pyx_v_alignment, __pyx_v_do_print);

  /* "ssw/sswpy.pyx":405
 * # end def
 * 
 * def format_force_align(  read: STR_T,             # <<<<<<<<<<<<<<
 *                         reference: STR_T,
 *                         alignment: Alignment,
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_3ssw_5sswpy_2format_force_align(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_read, PyObject *__pyx_v_reference, PyObject *__pyx_v_alignment, PyObject *__pyx_v_do_print) {
  PyObject *__pyx_v_start_ref = 0;
  PyObject *__pyx_v_start_read = 0;
  PyObject *__pyx_v_buffer_ref = 0;
  PyObject *__pyx_v_buffer_read = 0;
  PyObject *__pyx_v_ref_out = NULL;
  PyObject *__pyx_v_read_out = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("format_force_align", 0);

  /* "ssw/sswpy.pyx":417
 *         do_print: default is False
 *     '''
 *     start_ref: int = alignment.reference_start             # <<<<<<<<<<<<<<
 *     start_read: int = alignment.read_start
 *     buffer_ref: str = ''
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_alignment, __pyx_n_s_reference_start); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 417, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_start_ref = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "ssw/sswpy.pyx":418
 *     '''
 *     start_ref: int = alignment.reference_start
 *     start_read: int = alignment.read_start             # <<<<<<<<<<<<<<
 *     buffer_ref: str = ''
 *     buffer_read: str = ''
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_alignment, __pyx_n_s_read_start); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 418, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_start_read = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "ssw/sswpy.pyx":419
 *     start_ref: int = alignment.reference_start
 *     start_read: int = alignment.read_start
 *     buffer_ref: str = ''             # <<<<<<<<<<<<<<
 *     buffer_read: str = ''
 *     if start_ref < start_read:
 */
  __Pyx_INCREF(__pyx_kp_s__7);
  __pyx_v_buffer_ref = __pyx_kp_s__7;

  /* "ssw/sswpy.pyx":420
 *     start_read: int = alignment.read_start
 *     buffer_ref: str = ''
 *     buffer_read: str = ''             # <<<<<<<<<<<<<<
 *     if start_ref < start_read:
 *         buffer_ref = ' '*(start_read - start_ref)
 */
  __Pyx_INCREF(__pyx_kp_s__7);
  __pyx_v_buffer_read = __pyx_kp_s__7;

  /* "ssw/sswpy.pyx":421
 *     buffer_ref: str = ''
 *     buffer_read: str = ''
 *     if start_ref < start_read:             # <<<<<<<<<<<<<<
 *         buffer_ref = ' '*(start_read - start_ref)
 *     else:
 */
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_start_ref, __pyx_v_start_read, Py_LT); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 421, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 421, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "ssw/sswpy.pyx":422
 *     buffer_read: str = ''
 *     if start_ref < start_read:
 *         buffer_ref = ' '*(start_read - start_ref)             # <<<<<<<<<<<<<<
 *     else:
 *         buffer_read = ' '*(start_ref - start_read)
 */
    __pyx_t_1 = PyNumber_Subtract(__pyx_v_start_read, __pyx_v_start_ref); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 422, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = PyNumber_Multiply(__pyx_kp_s__12, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 422, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (!(likely(PyString_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_3)->tp_name), 0))) __PYX_ERR(0, 422, __pyx_L1_error)
    __Pyx_DECREF_SET(__pyx_v_buffer_ref, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "ssw/sswpy.pyx":421
 *     buffer_ref: str = ''
 *     buffer_read: str = ''
 *     if start_ref < start_read:             # <<<<<<<<<<<<<<
 *         buffer_ref = ' '*(start_read - start_ref)
 *     else:
 */
    goto __pyx_L3;
  }

  /* "ssw/sswpy.pyx":424
 *         buffer_ref = ' '*(start_read - start_ref)
 *     else:
 *         buffer_read = ' '*(start_ref - start_read)             # <<<<<<<<<<<<<<
 *     ref_out = buffer_ref + c_util._str(reference)
 *     read_out = buffer_read + c_util._str(read)
 */
  /*else*/ {
    __pyx_t_3 = PyNumber_Subtract(__pyx_v_start_ref, __pyx_v_start_read); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 424, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = PyNumber_Multiply(__pyx_kp_s__12, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 424, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 424, __pyx_L1_error)
    __Pyx_DECREF_SET(__pyx_v_buffer_read, ((PyObject*)__pyx_t_1));
    __pyx_t_1 = 0;
  }
  __pyx_L3:;

  /* "ssw/sswpy.pyx":425
 *     else:
 *         buffer_read = ' '*(start_ref - start_read)
 *     ref_out = buffer_ref + c_util._str(reference)             # <<<<<<<<<<<<<<
 *     read_out = buffer_read + c_util._str(read)
 *     if do_print:
 */
  __pyx_t_1 = __pyx_f_3ssw_6c_util__str(__pyx_v_reference); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 425, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyNumber_Add(__pyx_v_buffer_ref, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 425, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_ref_out = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "ssw/sswpy.pyx":426
 *         buffer_read = ' '*(start_ref - start_read)
 *     ref_out = buffer_ref + c_util._str(reference)
 *     read_out = buffer_read + c_util._str(read)             # <<<<<<<<<<<<<<
 *     if do_print:
 *         print(ref_out)
 */
  __pyx_t_3 = __pyx_f_3ssw_6c_util__str(__pyx_v_read); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 426, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = PyNumber_Add(__pyx_v_buffer_read, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 426, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_read_out = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "ssw/sswpy.pyx":427
 *     ref_out = buffer_ref + c_util._str(reference)
 *     read_out = buffer_read + c_util._str(read)
 *     if do_print:             # <<<<<<<<<<<<<<
 *         print(ref_out)
 *         print(read_out)
 */
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_do_print); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 427, __pyx_L1_error)
  if (__pyx_t_2) {

    /* "ssw/sswpy.pyx":428
 *     read_out = buffer_read + c_util._str(read)
 *     if do_print:
 *         print(ref_out)             # <<<<<<<<<<<<<<
 *         print(read_out)
 *     return ref_out, read_out
 */
    if (__Pyx_PrintOne(0, __pyx_v_ref_out) < 0) __PYX_ERR(0, 428, __pyx_L1_error)

    /* "ssw/sswpy.pyx":429
 *     if do_print:
 *         print(ref_out)
 *         print(read_out)             # <<<<<<<<<<<<<<
 *     return ref_out, read_out
 * # end def
 */
    if (__Pyx_PrintOne(0, __pyx_v_read_out) < 0) __PYX_ERR(0, 429, __pyx_L1_error)

    /* "ssw/sswpy.pyx":427
 *     ref_out = buffer_ref + c_util._str(reference)
 *     read_out = buffer_read + c_util._str(read)
 *     if do_print:             # <<<<<<<<<<<<<<
 *         print(ref_out)
 *         print(read_out)
 */
  }

  /* "ssw/sswpy.pyx":430
 *         print(ref_out)
 *         print(read_out)
 *     return ref_out, read_out             # <<<<<<<<<<<<<<
 * # end def
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 430, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_ref_out);
  __Pyx_GIVEREF(__pyx_v_ref_out);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_ref_out);
  __Pyx_INCREF(__pyx_v_read_out);
  __Pyx_GIVEREF(__pyx_v_read_out);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_read_out);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ssw/sswpy.pyx":405
 * # end def
 * 
 * def format_force_align(  read: STR_T,             # <<<<<<<<<<<<<<
 *                         reference: STR_T,
 *                         alignment: Alignment,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ssw.sswpy.format_force_align", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_start_ref);
  __Pyx_XDECREF(__pyx_v_start_read);
  __Pyx_XDECREF(__pyx_v_buffer_ref);
  __Pyx_XDECREF(__pyx_v_buffer_read);
  __Pyx_XDECREF(__pyx_v_ref_out);
  __Pyx_XDECREF(__pyx_v_read_out);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "c_util.pxd":5
 * from libc.string cimport memcpy
 * 
 * cdef inline bytes _bytes(s):             # <<<<<<<<<<<<<<
 *     if isinstance(s, str):
 *         # encode to the specific encoding used inside of the module
 */

static CYTHON_INLINE PyObject *__pyx_f_3ssw_6c_util__bytes(PyObject *__pyx_v_s) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("_bytes", 0);

  /* "c_util.pxd":6
 * 
 * cdef inline bytes _bytes(s):
 *     if isinstance(s, str):             # <<<<<<<<<<<<<<
 *         # encode to the specific encoding used inside of the module
 *         return (<str>s).encode('utf8')
 */
  __pyx_t_1 = PyString_Check(__pyx_v_s); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "c_util.pxd":8
 *     if isinstance(s, str):
 *         # encode to the specific encoding used inside of the module
 *         return (<str>s).encode('utf8')             # <<<<<<<<<<<<<<
 *     else:
 *         return s
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_s, __pyx_n_s_encode); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_tuple__13, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (!(likely(PyBytes_CheckExact(__pyx_t_4))||((__pyx_t_4) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_t_4)->tp_name), 0))) __PYX_ERR(2, 8, __pyx_L1_error)
    __pyx_r = ((PyObject*)__pyx_t_4);
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "c_util.pxd":6
 * 
 * cdef inline bytes _bytes(s):
 *     if isinstance(s, str):             # <<<<<<<<<<<<<<
 *         # encode to the specific encoding used inside of the module
 *         return (<str>s).encode('utf8')
 */
  }

  /* "c_util.pxd":10
 *         return (<str>s).encode('utf8')
 *     else:
 *         return s             # <<<<<<<<<<<<<<
 * 
 * cdef inline str _str(s):
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    if (!(likely(PyBytes_CheckExact(__pyx_v_s))||((__pyx_v_s) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_v_s)->tp_name), 0))) __PYX_ERR(2, 10, __pyx_L1_error)
    __Pyx_INCREF(__pyx_v_s);
    __pyx_r = ((PyObject*)__pyx_v_s);
    goto __pyx_L0;
  }

  /* "c_util.pxd":5
 * from libc.string cimport memcpy
 * 
 * cdef inline bytes _bytes(s):             # <<<<<<<<<<<<<<
 *     if isinstance(s, str):
 *         # encode to the specific encoding used inside of the module
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ssw.c_util._bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "c_util.pxd":12
 *         return s
 * 
 * cdef inline str _str(s):             # <<<<<<<<<<<<<<
 *     if isinstance(s, bytes):
 *         # encode to the specific encoding used inside of the module
 */

static CYTHON_INLINE PyObject *__pyx_f_3ssw_6c_util__str(PyObject *__pyx_v_s) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("_str", 0);

  /* "c_util.pxd":13
 * 
 * cdef inline str _str(s):
 *     if isinstance(s, bytes):             # <<<<<<<<<<<<<<
 *         # encode to the specific encoding used inside of the module
 *         return (<bytes>s).decode('utf8')
 */
  __pyx_t_1 = PyBytes_Check(__pyx_v_s); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "c_util.pxd":15
 *     if isinstance(s, bytes):
 *         # encode to the specific encoding used inside of the module
 *         return (<bytes>s).decode('utf8')             # <<<<<<<<<<<<<<
 *     else:
 *         return s
 */
    __Pyx_XDECREF(__pyx_r);
    if (unlikely(__pyx_v_s == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "decode");
      __PYX_ERR(2, 15, __pyx_L1_error)
    }
    __pyx_t_3 = __Pyx_decode_bytes(((PyObject*)__pyx_v_s), 0, PY_SSIZE_T_MAX, NULL, NULL, PyUnicode_DecodeUTF8); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (!(likely(PyString_CheckExact(__pyx_t_3))||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_3)->tp_name), 0))) __PYX_ERR(2, 15, __pyx_L1_error)
    __pyx_r = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "c_util.pxd":13
 * 
 * cdef inline str _str(s):
 *     if isinstance(s, bytes):             # <<<<<<<<<<<<<<
 *         # encode to the specific encoding used inside of the module
 *         return (<bytes>s).decode('utf8')
 */
  }

  /* "c_util.pxd":17
 *         return (<bytes>s).decode('utf8')
 *     else:
 *         return s             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "Python.h":
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    if (!(likely(PyString_CheckExact(__pyx_v_s))||((__pyx_v_s) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_v_s)->tp_name), 0))) __PYX_ERR(2, 17, __pyx_L1_error)
    __Pyx_INCREF(__pyx_v_s);
    __pyx_r = ((PyObject*)__pyx_v_s);
    goto __pyx_L0;
  }

  /* "c_util.pxd":12
 *         return s
 * 
 * cdef inline str _str(s):             # <<<<<<<<<<<<<<
 *     if isinstance(s, bytes):
 *         # encode to the specific encoding used inside of the module
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ssw.c_util._str", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "c_util.pxd":45
 * https://github.com/python/cpython/blob/master/Objects/unicodeobject.c#L4688
 * """
 * cdef inline object copy_obj_to_cstr_unsafe(    object o1,             # <<<<<<<<<<<<<<
 *                                         Py_ssize_t *length,
 *                                         char** c_str2):
 */

static CYTHON_INLINE PyObject *__pyx_f_3ssw_6c_util_copy_obj_to_cstr_unsafe(PyObject *__pyx_v_o1, Py_ssize_t *__pyx_v_length, char **__pyx_v_c_str2) {
  PyObject *__pyx_v_o2 = 0;
  char *__pyx_v_c_str1;
  size_t __pyx_v_b_length;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("copy_obj_to_cstr_unsafe", 0);

  /* "c_util.pxd":51
 *     cdef char* c_str1
 *     cdef size_t b_length
 *     if PyBytes_Check(o1):             # <<<<<<<<<<<<<<
 *         if PyBytes_AsStringAndSize(o1, &(c_str1), length) == -1:
 *             raise TypeError("copy_obj_to_cstr:")
 */
  __pyx_t_1 = (PyBytes_Check(__pyx_v_o1) != 0);
  if (__pyx_t_1) {

    /* "c_util.pxd":52
 *     cdef size_t b_length
 *     if PyBytes_Check(o1):
 *         if PyBytes_AsStringAndSize(o1, &(c_str1), length) == -1:             # <<<<<<<<<<<<<<
 *             raise TypeError("copy_obj_to_cstr:")
 *         b_length = length[0]
 */
    __pyx_t_1 = ((PyBytes_AsStringAndSize(__pyx_v_o1, (&__pyx_v_c_str1), __pyx_v_length) == -1L) != 0);
    if (__pyx_t_1) {

      /* "c_util.pxd":53
 *     if PyBytes_Check(o1):
 *         if PyBytes_AsStringAndSize(o1, &(c_str1), length) == -1:
 *             raise TypeError("copy_obj_to_cstr:")             # <<<<<<<<<<<<<<
 *         b_length = length[0]
 *         o2 = PyBytes_FromStringAndSize(c_str1, b_length)
 */
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__14, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 53, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_Raise(__pyx_t_2, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __PYX_ERR(2, 53, __pyx_L1_error)

      /* "c_util.pxd":52
 *     cdef size_t b_length
 *     if PyBytes_Check(o1):
 *         if PyBytes_AsStringAndSize(o1, &(c_str1), length) == -1:             # <<<<<<<<<<<<<<
 *             raise TypeError("copy_obj_to_cstr:")
 *         b_length = length[0]
 */
    }

    /* "c_util.pxd":54
 *         if PyBytes_AsStringAndSize(o1, &(c_str1), length) == -1:
 *             raise TypeError("copy_obj_to_cstr:")
 *         b_length = length[0]             # <<<<<<<<<<<<<<
 *         o2 = PyBytes_FromStringAndSize(c_str1, b_length)
 *         #if o2 == NULL:
 */
    __pyx_v_b_length = (__pyx_v_length[0]);

    /* "c_util.pxd":55
 *             raise TypeError("copy_obj_to_cstr:")
 *         b_length = length[0]
 *         o2 = PyBytes_FromStringAndSize(c_str1, b_length)             # <<<<<<<<<<<<<<
 *         #if o2 == NULL:
 *         #    raise OSError("copy_obj_to_cstr:")
 */
    __pyx_t_2 = PyBytes_FromStringAndSize(__pyx_v_c_str1, __pyx_v_b_length); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 55, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_v_o2 = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "c_util.pxd":58
 *         #if o2 == NULL:
 *         #    raise OSError("copy_obj_to_cstr:")
 *         c_str2[0] = PyBytes_AsString(o2)             # <<<<<<<<<<<<<<
 *     else:
 *         c_str1 = PyUnicode_AsUTF8AndSize(o1, length)
 */
    (__pyx_v_c_str2[0]) = PyBytes_AsString(__pyx_v_o2);

    /* "c_util.pxd":51
 *     cdef char* c_str1
 *     cdef size_t b_length
 *     if PyBytes_Check(o1):             # <<<<<<<<<<<<<<
 *         if PyBytes_AsStringAndSize(o1, &(c_str1), length) == -1:
 *             raise TypeError("copy_obj_to_cstr:")
 */
    goto __pyx_L3;
  }

  /* "c_util.pxd":60
 *         c_str2[0] = PyBytes_AsString(o2)
 *     else:
 *         c_str1 = PyUnicode_AsUTF8AndSize(o1, length)             # <<<<<<<<<<<<<<
 *         if c_str1 == NULL:
 *             raise OSError("copy_obj_to_cstr:")
 */
  /*else*/ {
    __pyx_v_c_str1 = PyUnicode_AsUTF8AndSize(__pyx_v_o1, __pyx_v_length);

    /* "c_util.pxd":61
 *     else:
 *         c_str1 = PyUnicode_AsUTF8AndSize(o1, length)
 *         if c_str1 == NULL:             # <<<<<<<<<<<<<<
 *             raise OSError("copy_obj_to_cstr:")
 *         b_length = length[0]
 */
    __pyx_t_1 = ((__pyx_v_c_str1 == NULL) != 0);
    if (__pyx_t_1) {

      /* "c_util.pxd":62
 *         c_str1 = PyUnicode_AsUTF8AndSize(o1, length)
 *         if c_str1 == NULL:
 *             raise OSError("copy_obj_to_cstr:")             # <<<<<<<<<<<<<<
 *         b_length = length[0]
 *         o2 = PyUnicode_FromStringAndSize(<const char *>c_str1, b_length)
 */
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_OSError, __pyx_tuple__15, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 62, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_Raise(__pyx_t_2, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __PYX_ERR(2, 62, __pyx_L1_error)

      /* "c_util.pxd":61
 *     else:
 *         c_str1 = PyUnicode_AsUTF8AndSize(o1, length)
 *         if c_str1 == NULL:             # <<<<<<<<<<<<<<
 *             raise OSError("copy_obj_to_cstr:")
 *         b_length = length[0]
 */
    }

    /* "c_util.pxd":63
 *         if c_str1 == NULL:
 *             raise OSError("copy_obj_to_cstr:")
 *         b_length = length[0]             # <<<<<<<<<<<<<<
 *         o2 = PyUnicode_FromStringAndSize(<const char *>c_str1, b_length)
 *         #if o2 == NULL:
 */
    __pyx_v_b_length = (__pyx_v_length[0]);

    /* "c_util.pxd":64
 *             raise OSError("copy_obj_to_cstr:")
 *         b_length = length[0]
 *         o2 = PyUnicode_FromStringAndSize(<const char *>c_str1, b_length)             # <<<<<<<<<<<<<<
 *         #if o2 == NULL:
 *         #    raise OSError("copy_obj_to_cstr:")
 */
    __pyx_t_2 = PyUnicode_FromStringAndSize(((char const *)__pyx_v_c_str1), __pyx_v_b_length); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 64, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_v_o2 = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "c_util.pxd":67
 *         #if o2 == NULL:
 *         #    raise OSError("copy_obj_to_cstr:")
 *         c_str2[0] = PyUnicode_AsUTF8(o2)             # <<<<<<<<<<<<<<
 * 
 *     return o2
 */
    (__pyx_v_c_str2[0]) = PyUnicode_AsUTF8(__pyx_v_o2);
  }
  __pyx_L3:;

  /* "c_util.pxd":69
 *         c_str2[0] = PyUnicode_AsUTF8(o2)
 * 
 *     return o2             # <<<<<<<<<<<<<<
 * # end def
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_o2);
  __pyx_r = __pyx_v_o2;
  goto __pyx_L0;

  /* "c_util.pxd":45
 * https://github.com/python/cpython/blob/master/Objects/unicodeobject.c#L4688
 * """
 * cdef inline object copy_obj_to_cstr_unsafe(    object o1,             # <<<<<<<<<<<<<<
 *                                         Py_ssize_t *length,
 *                                         char** c_str2):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("ssw.c_util.copy_obj_to_cstr_unsafe", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_o2);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "c_util.pxd":86
 * 
 * """
 * cdef inline int copy_obj_to_cstr(object o1,             # <<<<<<<<<<<<<<
 *                             Py_ssize_t *length,
 *                             char** c_str2) except -1:
 */

static CYTHON_INLINE int __pyx_f_3ssw_6c_util_copy_obj_to_cstr(PyObject *__pyx_v_o1, Py_ssize_t *__pyx_v_length, char **__pyx_v_c_str2) {
  char *__pyx_v_c_str1;
  char *__pyx_v_temp;
  int __pyx_v_obj_type;
  size_t __pyx_v_b_length;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("copy_obj_to_cstr", 0);

  /* "c_util.pxd":90
 *                             char** c_str2) except -1:
 *     cdef char* c_str1
 *     cdef char* temp = NULL             # <<<<<<<<<<<<<<
 *     cdef int obj_type
 *     cdef size_t b_length
 */
  __pyx_v_temp = NULL;

  /* "c_util.pxd":93
 *     cdef int obj_type
 *     cdef size_t b_length
 *     if PyBytes_Check(o1):             # <<<<<<<<<<<<<<
 *         obj_type = 1
 *         if PyBytes_AsStringAndSize(o1, &(c_str1), length) == -1:
 */
  __pyx_t_1 = (PyBytes_Check(__pyx_v_o1) != 0);
  if (__pyx_t_1) {

    /* "c_util.pxd":94
 *     cdef size_t b_length
 *     if PyBytes_Check(o1):
 *         obj_type = 1             # <<<<<<<<<<<<<<
 *         if PyBytes_AsStringAndSize(o1, &(c_str1), length) == -1:
 *             #raise TypeError("copy_obj_to_cstr:")
 */
    __pyx_v_obj_type = 1;

    /* "c_util.pxd":95
 *     if PyBytes_Check(o1):
 *         obj_type = 1
 *         if PyBytes_AsStringAndSize(o1, &(c_str1), length) == -1:             # <<<<<<<<<<<<<<
 *             #raise TypeError("copy_obj_to_cstr:")
 *             return -1
 */
    __pyx_t_1 = ((PyBytes_AsStringAndSize(__pyx_v_o1, (&__pyx_v_c_str1), __pyx_v_length) == -1L) != 0);
    if (__pyx_t_1) {

      /* "c_util.pxd":97
 *         if PyBytes_AsStringAndSize(o1, &(c_str1), length) == -1:
 *             #raise TypeError("copy_obj_to_cstr:")
 *             return -1             # <<<<<<<<<<<<<<
 *     else:
 *         obj_type = 0
 */
      __pyx_r = -1;
      goto __pyx_L0;

      /* "c_util.pxd":95
 *     if PyBytes_Check(o1):
 *         obj_type = 1
 *         if PyBytes_AsStringAndSize(o1, &(c_str1), length) == -1:             # <<<<<<<<<<<<<<
 *             #raise TypeError("copy_obj_to_cstr:")
 *             return -1
 */
    }

    /* "c_util.pxd":93
 *     cdef int obj_type
 *     cdef size_t b_length
 *     if PyBytes_Check(o1):             # <<<<<<<<<<<<<<
 *         obj_type = 1
 *         if PyBytes_AsStringAndSize(o1, &(c_str1), length) == -1:
 */
    goto __pyx_L3;
  }

  /* "c_util.pxd":99
 *             return -1
 *     else:
 *         obj_type = 0             # <<<<<<<<<<<<<<
 *         c_str1 = PyUnicode_AsUTF8AndSize(o1, length)
 *         if c_str1 == NULL:
 */
  /*else*/ {
    __pyx_v_obj_type = 0;

    /* "c_util.pxd":100
 *     else:
 *         obj_type = 0
 *         c_str1 = PyUnicode_AsUTF8AndSize(o1, length)             # <<<<<<<<<<<<<<
 *         if c_str1 == NULL:
 *             #raise OSError("copy_obj_to_cstr:")
 */
    __pyx_v_c_str1 = PyUnicode_AsUTF8AndSize(__pyx_v_o1, __pyx_v_length);

    /* "c_util.pxd":101
 *         obj_type = 0
 *         c_str1 = PyUnicode_AsUTF8AndSize(o1, length)
 *         if c_str1 == NULL:             # <<<<<<<<<<<<<<
 *             #raise OSError("copy_obj_to_cstr:")
 *             return -1
 */
    __pyx_t_1 = ((__pyx_v_c_str1 == NULL) != 0);
    if (__pyx_t_1) {

      /* "c_util.pxd":103
 *         if c_str1 == NULL:
 *             #raise OSError("copy_obj_to_cstr:")
 *             return -1             # <<<<<<<<<<<<<<
 *     b_length = length[0]+1
 *     temp = <char *> PyMem_Malloc(b_length*sizeof(char))
 */
      __pyx_r = -1;
      goto __pyx_L0;

      /* "c_util.pxd":101
 *         obj_type = 0
 *         c_str1 = PyUnicode_AsUTF8AndSize(o1, length)
 *         if c_str1 == NULL:             # <<<<<<<<<<<<<<
 *             #raise OSError("copy_obj_to_cstr:")
 *             return -1
 */
    }
  }
  __pyx_L3:;

  /* "c_util.pxd":104
 *             #raise OSError("copy_obj_to_cstr:")
 *             return -1
 *     b_length = length[0]+1             # <<<<<<<<<<<<<<
 *     temp = <char *> PyMem_Malloc(b_length*sizeof(char))
 *     if temp == NULL:
 */
  __pyx_v_b_length = ((__pyx_v_length[0]) + 1);

  /* "c_util.pxd":105
 *             return -1
 *     b_length = length[0]+1
 *     temp = <char *> PyMem_Malloc(b_length*sizeof(char))             # <<<<<<<<<<<<<<
 *     if temp == NULL:
 *         return -1
 */
  __pyx_v_temp = ((char *)PyMem_Malloc((__pyx_v_b_length * (sizeof(char)))));

  /* "c_util.pxd":106
 *     b_length = length[0]+1
 *     temp = <char *> PyMem_Malloc(b_length*sizeof(char))
 *     if temp == NULL:             # <<<<<<<<<<<<<<
 *         return -1
 *     memcpy(temp, c_str1, b_length)
 */
  __pyx_t_1 = ((__pyx_v_temp == NULL) != 0);
  if (__pyx_t_1) {

    /* "c_util.pxd":107
 *     temp = <char *> PyMem_Malloc(b_length*sizeof(char))
 *     if temp == NULL:
 *         return -1             # <<<<<<<<<<<<<<
 *     memcpy(temp, c_str1, b_length)
 *     c_str2[0] = temp
 */
    __pyx_r = -1;
    goto __pyx_L0;

    /* "c_util.pxd":106
 *     b_length = length[0]+1
 *     temp = <char *> PyMem_Malloc(b_length*sizeof(char))
 *     if temp == NULL:             # <<<<<<<<<<<<<<
 *         return -1
 *     memcpy(temp, c_str1, b_length)
 */
  }

  /* "c_util.pxd":108
 *     if temp == NULL:
 *         return -1
 *     memcpy(temp, c_str1, b_length)             # <<<<<<<<<<<<<<
 *     c_str2[0] = temp
 *     return obj_type
 */
  memcpy(__pyx_v_temp, __pyx_v_c_str1, __pyx_v_b_length);

  /* "c_util.pxd":109
 *         return -1
 *     memcpy(temp, c_str1, b_length)
 *     c_str2[0] = temp             # <<<<<<<<<<<<<<
 *     return obj_type
 * # end def
 */
  (__pyx_v_c_str2[0]) = __pyx_v_temp;

  /* "c_util.pxd":110
 *     memcpy(temp, c_str1, b_length)
 *     c_str2[0] = temp
 *     return obj_type             # <<<<<<<<<<<<<<
 * # end def
 * 
 */
  __pyx_r = __pyx_v_obj_type;
  goto __pyx_L0;

  /* "c_util.pxd":86
 * 
 * """
 * cdef inline int copy_obj_to_cstr(object o1,             # <<<<<<<<<<<<<<
 *                             Py_ssize_t *length,
 *                             char** c_str2) except -1:
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "c_util.pxd":125
 *     obj - a python string object and frees the memory associated with c_str
 * """
 * cdef inline cstr_to_obj(char* c_str,             # <<<<<<<<<<<<<<
 *                         Py_ssize_t length,
 *                         int obj_type):
 */

static CYTHON_INLINE PyObject *__pyx_f_3ssw_6c_util_cstr_to_obj(char *__pyx_v_c_str, Py_ssize_t __pyx_v_length, int __pyx_v_obj_type) {
  PyObject *__pyx_v_obj = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("cstr_to_obj", 0);

  /* "c_util.pxd":130
 *     cdef object obj
 * 
 *     if obj_type:             # <<<<<<<<<<<<<<
 *         obj = PyBytes_FromStringAndSize(c_str, length)
 *     else:
 */
  __pyx_t_1 = (__pyx_v_obj_type != 0);
  if (__pyx_t_1) {

    /* "c_util.pxd":131
 * 
 *     if obj_type:
 *         obj = PyBytes_FromStringAndSize(c_str, length)             # <<<<<<<<<<<<<<
 *     else:
 *         obj = PyUnicode_FromStringAndSize(<const char *>c_str, length)
 */
    __pyx_t_2 = PyBytes_FromStringAndSize(__pyx_v_c_str, __pyx_v_length); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 131, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_v_obj = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "c_util.pxd":130
 *     cdef object obj
 * 
 *     if obj_type:             # <<<<<<<<<<<<<<
 *         obj = PyBytes_FromStringAndSize(c_str, length)
 *     else:
 */
    goto __pyx_L3;
  }

  /* "c_util.pxd":133
 *         obj = PyBytes_FromStringAndSize(c_str, length)
 *     else:
 *         obj = PyUnicode_FromStringAndSize(<const char *>c_str, length)             # <<<<<<<<<<<<<<
 *     PyMem_Free(c_str)
 *     return obj
 */
  /*else*/ {
    __pyx_t_2 = PyUnicode_FromStringAndSize(((char const *)__pyx_v_c_str), __pyx_v_length); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 133, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_v_obj = __pyx_t_2;
    __pyx_t_2 = 0;
  }
  __pyx_L3:;

  /* "c_util.pxd":134
 *     else:
 *         obj = PyUnicode_FromStringAndSize(<const char *>c_str, length)
 *     PyMem_Free(c_str)             # <<<<<<<<<<<<<<
 *     return obj
 * # end def
 */
  PyMem_Free(__pyx_v_c_str);

  /* "c_util.pxd":135
 *         obj = PyUnicode_FromStringAndSize(<const char *>c_str, length)
 *     PyMem_Free(c_str)
 *     return obj             # <<<<<<<<<<<<<<
 * # end def
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_obj);
  __pyx_r = __pyx_v_obj;
  goto __pyx_L0;

  /* "c_util.pxd":125
 *     obj - a python string object and frees the memory associated with c_str
 * """
 * cdef inline cstr_to_obj(char* c_str,             # <<<<<<<<<<<<<<
 *                         Py_ssize_t length,
 *                         int obj_type):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("ssw.c_util.cstr_to_obj", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_obj);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "c_util.pxd":138
 * # end def
 * 
 * cdef inline cstr_to_obj_nofree(char* c_str,             # <<<<<<<<<<<<<<
 *                         Py_ssize_t length,
 *                         int obj_type):
 */

static CYTHON_INLINE PyObject *__pyx_f_3ssw_6c_util_cstr_to_obj_nofree(char *__pyx_v_c_str, Py_ssize_t __pyx_v_length, int __pyx_v_obj_type) {
  PyObject *__pyx_v_obj = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("cstr_to_obj_nofree", 0);

  /* "c_util.pxd":143
 *     cdef object obj
 * 
 *     if obj_type:             # <<<<<<<<<<<<<<
 *         obj = PyBytes_FromStringAndSize(c_str, length)
 *     else:
 */
  __pyx_t_1 = (__pyx_v_obj_type != 0);
  if (__pyx_t_1) {

    /* "c_util.pxd":144
 * 
 *     if obj_type:
 *         obj = PyBytes_FromStringAndSize(c_str, length)             # <<<<<<<<<<<<<<
 *     else:
 *         obj = PyUnicode_FromStringAndSize(<const char *>c_str, length)
 */
    __pyx_t_2 = PyBytes_FromStringAndSize(__pyx_v_c_str, __pyx_v_length); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 144, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_v_obj = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "c_util.pxd":143
 *     cdef object obj
 * 
 *     if obj_type:             # <<<<<<<<<<<<<<
 *         obj = PyBytes_FromStringAndSize(c_str, length)
 *     else:
 */
    goto __pyx_L3;
  }

  /* "c_util.pxd":146
 *         obj = PyBytes_FromStringAndSize(c_str, length)
 *     else:
 *         obj = PyUnicode_FromStringAndSize(<const char *>c_str, length)             # <<<<<<<<<<<<<<
 *     return obj
 * # end def
 */
  /*else*/ {
    __pyx_t_2 = PyUnicode_FromStringAndSize(((char const *)__pyx_v_c_str), __pyx_v_length); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 146, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_v_obj = __pyx_t_2;
    __pyx_t_2 = 0;
  }
  __pyx_L3:;

  /* "c_util.pxd":147
 *     else:
 *         obj = PyUnicode_FromStringAndSize(<const char *>c_str, length)
 *     return obj             # <<<<<<<<<<<<<<
 * # end def
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_obj);
  __pyx_r = __pyx_v_obj;
  goto __pyx_L0;

  /* "c_util.pxd":138
 * # end def
 * 
 * cdef inline cstr_to_obj_nofree(char* c_str,             # <<<<<<<<<<<<<<
 *                         Py_ssize_t length,
 *                         int obj_type):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("ssw.c_util.cstr_to_obj_nofree", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_obj);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "c_util.pxd":150
 * # end def
 * 
 * cdef inline cstr_to_obj_nolength(char* c_str,             # <<<<<<<<<<<<<<
 *                                 int obj_type):
 *     cdef object obj
 */

static CYTHON_INLINE PyObject *__pyx_f_3ssw_6c_util_cstr_to_obj_nolength(char *__pyx_v_c_str, int __pyx_v_obj_type) {
  PyObject *__pyx_v_obj = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("cstr_to_obj_nolength", 0);

  /* "c_util.pxd":154
 *     cdef object obj
 * 
 *     if obj_type:             # <<<<<<<<<<<<<<
 *         obj = PyBytes_FromString(c_str)
 *     else:
 */
  __pyx_t_1 = (__pyx_v_obj_type != 0);
  if (__pyx_t_1) {

    /* "c_util.pxd":155
 * 
 *     if obj_type:
 *         obj = PyBytes_FromString(c_str)             # <<<<<<<<<<<<<<
 *     else:
 *         obj = PyUnicode_FromString(<const char *>c_str)
 */
    __pyx_t_2 = PyBytes_FromString(__pyx_v_c_str); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 155, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_v_obj = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "c_util.pxd":154
 *     cdef object obj
 * 
 *     if obj_type:             # <<<<<<<<<<<<<<
 *         obj = PyBytes_FromString(c_str)
 *     else:
 */
    goto __pyx_L3;
  }

  /* "c_util.pxd":157
 *         obj = PyBytes_FromString(c_str)
 *     else:
 *         obj = PyUnicode_FromString(<const char *>c_str)             # <<<<<<<<<<<<<<
 *     return obj
 * # end def
 */
  /*else*/ {
    __pyx_t_2 = PyUnicode_FromString(((char const *)__pyx_v_c_str)); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 157, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_v_obj = __pyx_t_2;
    __pyx_t_2 = 0;
  }
  __pyx_L3:;

  /* "c_util.pxd":158
 *     else:
 *         obj = PyUnicode_FromString(<const char *>c_str)
 *     return obj             # <<<<<<<<<<<<<<
 * # end def
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_obj);
  __pyx_r = __pyx_v_obj;
  goto __pyx_L0;

  /* "c_util.pxd":150
 * # end def
 * 
 * cdef inline cstr_to_obj_nolength(char* c_str,             # <<<<<<<<<<<<<<
 *                                 int obj_type):
 *     cdef object obj
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("ssw.c_util.cstr_to_obj_nolength", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_obj);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "c_util.pxd":172
 * For Python 3 it can take a bytes object or a unicode object
 * """
 * cdef inline char* obj_to_cstr(object o1):             # <<<<<<<<<<<<<<
 *     cdef char* c_str1
 *     cdef Py_ssize_t length
 */

static CYTHON_INLINE char *__pyx_f_3ssw_6c_util_obj_to_cstr(PyObject *__pyx_v_o1) {
  char *__pyx_v_c_str1;
  Py_ssize_t __pyx_v_length;
  char *__pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("obj_to_cstr", 0);

  /* "c_util.pxd":175
 *     cdef char* c_str1
 *     cdef Py_ssize_t length
 *     if PyBytes_Check(o1):             # <<<<<<<<<<<<<<
 *         if PyBytes_AsStringAndSize(o1, &(c_str1), &length) == -1:
 *             raise TypeError("obj_to_cstr:")
 */
  __pyx_t_1 = (PyBytes_Check(__pyx_v_o1) != 0);
  if (__pyx_t_1) {

    /* "c_util.pxd":176
 *     cdef Py_ssize_t length
 *     if PyBytes_Check(o1):
 *         if PyBytes_AsStringAndSize(o1, &(c_str1), &length) == -1:             # <<<<<<<<<<<<<<
 *             raise TypeError("obj_to_cstr:")
 *         return c_str1
 */
    __pyx_t_1 = ((PyBytes_AsStringAndSize(__pyx_v_o1, (&__pyx_v_c_str1), (&__pyx_v_length)) == -1L) != 0);
    if (__pyx_t_1) {

      /* "c_util.pxd":177
 *     if PyBytes_Check(o1):
 *         if PyBytes_AsStringAndSize(o1, &(c_str1), &length) == -1:
 *             raise TypeError("obj_to_cstr:")             # <<<<<<<<<<<<<<
 *         return c_str1
 *     else:
 */
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__16, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 177, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_Raise(__pyx_t_2, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __PYX_ERR(2, 177, __pyx_L1_error)

      /* "c_util.pxd":176
 *     cdef Py_ssize_t length
 *     if PyBytes_Check(o1):
 *         if PyBytes_AsStringAndSize(o1, &(c_str1), &length) == -1:             # <<<<<<<<<<<<<<
 *             raise TypeError("obj_to_cstr:")
 *         return c_str1
 */
    }

    /* "c_util.pxd":178
 *         if PyBytes_AsStringAndSize(o1, &(c_str1), &length) == -1:
 *             raise TypeError("obj_to_cstr:")
 *         return c_str1             # <<<<<<<<<<<<<<
 *     else:
 *         c_str1 = PyUnicode_AsUTF8AndSize(o1, &length)
 */
    __pyx_r = __pyx_v_c_str1;
    goto __pyx_L0;

    /* "c_util.pxd":175
 *     cdef char* c_str1
 *     cdef Py_ssize_t length
 *     if PyBytes_Check(o1):             # <<<<<<<<<<<<<<
 *         if PyBytes_AsStringAndSize(o1, &(c_str1), &length) == -1:
 *             raise TypeError("obj_to_cstr:")
 */
  }

  /* "c_util.pxd":180
 *         return c_str1
 *     else:
 *         c_str1 = PyUnicode_AsUTF8AndSize(o1, &length)             # <<<<<<<<<<<<<<
 *         if c_str1 == NULL:
 *             raise OSError("obj_to_cstr:")
 */
  /*else*/ {
    __pyx_v_c_str1 = PyUnicode_AsUTF8AndSize(__pyx_v_o1, (&__pyx_v_length));

    /* "c_util.pxd":181
 *     else:
 *         c_str1 = PyUnicode_AsUTF8AndSize(o1, &length)
 *         if c_str1 == NULL:             # <<<<<<<<<<<<<<
 *             raise OSError("obj_to_cstr:")
 *     return c_str1
 */
    __pyx_t_1 = ((__pyx_v_c_str1 == NULL) != 0);
    if (__pyx_t_1) {

      /* "c_util.pxd":182
 *         c_str1 = PyUnicode_AsUTF8AndSize(o1, &length)
 *         if c_str1 == NULL:
 *             raise OSError("obj_to_cstr:")             # <<<<<<<<<<<<<<
 *     return c_str1
 * # end def
 */
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_OSError, __pyx_tuple__17, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 182, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_Raise(__pyx_t_2, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __PYX_ERR(2, 182, __pyx_L1_error)

      /* "c_util.pxd":181
 *     else:
 *         c_str1 = PyUnicode_AsUTF8AndSize(o1, &length)
 *         if c_str1 == NULL:             # <<<<<<<<<<<<<<
 *             raise OSError("obj_to_cstr:")
 *     return c_str1
 */
    }
  }

  /* "c_util.pxd":183
 *         if c_str1 == NULL:
 *             raise OSError("obj_to_cstr:")
 *     return c_str1             # <<<<<<<<<<<<<<
 * # end def
 * 
 */
  __pyx_r = __pyx_v_c_str1;
  goto __pyx_L0;

  /* "c_util.pxd":172
 * For Python 3 it can take a bytes object or a unicode object
 * """
 * cdef inline char* obj_to_cstr(object o1):             # <<<<<<<<<<<<<<
 *     cdef char* c_str1
 *     cdef Py_ssize_t length
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_WriteUnraisable("ssw.c_util.obj_to_cstr", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "c_util.pxd":189
 * Same as above but fetches the string length too
 * """
 * cdef inline char* obj_to_cstr_len(object o1, Py_ssize_t *length):             # <<<<<<<<<<<<<<
 *     cdef char* c_str1
 *     if PyBytes_Check(o1):
 */

static CYTHON_INLINE char *__pyx_f_3ssw_6c_util_obj_to_cstr_len(PyObject *__pyx_v_o1, Py_ssize_t *__pyx_v_length) {
  char *__pyx_v_c_str1;
  char *__pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("obj_to_cstr_len", 0);

  /* "c_util.pxd":191
 * cdef inline char* obj_to_cstr_len(object o1, Py_ssize_t *length):
 *     cdef char* c_str1
 *     if PyBytes_Check(o1):             # <<<<<<<<<<<<<<
 *         if PyBytes_AsStringAndSize(o1, &(c_str1), length) == -1:
 *             raise TypeError("obj_to_cstr: PyBytes_AsStringAndSize error")
 */
  __pyx_t_1 = (PyBytes_Check(__pyx_v_o1) != 0);
  if (__pyx_t_1) {

    /* "c_util.pxd":192
 *     cdef char* c_str1
 *     if PyBytes_Check(o1):
 *         if PyBytes_AsStringAndSize(o1, &(c_str1), length) == -1:             # <<<<<<<<<<<<<<
 *             raise TypeError("obj_to_cstr: PyBytes_AsStringAndSize error")
 *         return c_str1
 */
    __pyx_t_1 = ((PyBytes_AsStringAndSize(__pyx_v_o1, (&__pyx_v_c_str1), __pyx_v_length) == -1L) != 0);
    if (__pyx_t_1) {

      /* "c_util.pxd":193
 *     if PyBytes_Check(o1):
 *         if PyBytes_AsStringAndSize(o1, &(c_str1), length) == -1:
 *             raise TypeError("obj_to_cstr: PyBytes_AsStringAndSize error")             # <<<<<<<<<<<<<<
 *         return c_str1
 *     else:
 */
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__18, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 193, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_Raise(__pyx_t_2, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __PYX_ERR(2, 193, __pyx_L1_error)

      /* "c_util.pxd":192
 *     cdef char* c_str1
 *     if PyBytes_Check(o1):
 *         if PyBytes_AsStringAndSize(o1, &(c_str1), length) == -1:             # <<<<<<<<<<<<<<
 *             raise TypeError("obj_to_cstr: PyBytes_AsStringAndSize error")
 *         return c_str1
 */
    }

    /* "c_util.pxd":194
 *         if PyBytes_AsStringAndSize(o1, &(c_str1), length) == -1:
 *             raise TypeError("obj_to_cstr: PyBytes_AsStringAndSize error")
 *         return c_str1             # <<<<<<<<<<<<<<
 *     else:
 *         c_str1 = PyUnicode_AsUTF8AndSize(o1, length)
 */
    __pyx_r = __pyx_v_c_str1;
    goto __pyx_L0;

    /* "c_util.pxd":191
 * cdef inline char* obj_to_cstr_len(object o1, Py_ssize_t *length):
 *     cdef char* c_str1
 *     if PyBytes_Check(o1):             # <<<<<<<<<<<<<<
 *         if PyBytes_AsStringAndSize(o1, &(c_str1), length) == -1:
 *             raise TypeError("obj_to_cstr: PyBytes_AsStringAndSize error")
 */
  }

  /* "c_util.pxd":196
 *         return c_str1
 *     else:
 *         c_str1 = PyUnicode_AsUTF8AndSize(o1, length)             # <<<<<<<<<<<<<<
 *         if c_str1 == NULL:
 *             raise OSError("obj_to_cstr: PyUnicode_AsUTF8AndSize error")
 */
  /*else*/ {
    __pyx_v_c_str1 = PyUnicode_AsUTF8AndSize(__pyx_v_o1, __pyx_v_length);

    /* "c_util.pxd":197
 *     else:
 *         c_str1 = PyUnicode_AsUTF8AndSize(o1, length)
 *         if c_str1 == NULL:             # <<<<<<<<<<<<<<
 *             raise OSError("obj_to_cstr: PyUnicode_AsUTF8AndSize error")
 *     return c_str1
 */
    __pyx_t_1 = ((__pyx_v_c_str1 == NULL) != 0);
    if (__pyx_t_1) {

      /* "c_util.pxd":198
 *         c_str1 = PyUnicode_AsUTF8AndSize(o1, length)
 *         if c_str1 == NULL:
 *             raise OSError("obj_to_cstr: PyUnicode_AsUTF8AndSize error")             # <<<<<<<<<<<<<<
 *     return c_str1
 * # end def
 */
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_OSError, __pyx_tuple__19, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 198, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_Raise(__pyx_t_2, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __PYX_ERR(2, 198, __pyx_L1_error)

      /* "c_util.pxd":197
 *     else:
 *         c_str1 = PyUnicode_AsUTF8AndSize(o1, length)
 *         if c_str1 == NULL:             # <<<<<<<<<<<<<<
 *             raise OSError("obj_to_cstr: PyUnicode_AsUTF8AndSize error")
 *     return c_str1
 */
    }
  }

  /* "c_util.pxd":199
 *         if c_str1 == NULL:
 *             raise OSError("obj_to_cstr: PyUnicode_AsUTF8AndSize error")
 *     return c_str1             # <<<<<<<<<<<<<<
 * # end def
 * 
 */
  __pyx_r = __pyx_v_c_str1;
  goto __pyx_L0;

  /* "c_util.pxd":189
 * Same as above but fetches the string length too
 * """
 * cdef inline char* obj_to_cstr_len(object o1, Py_ssize_t *length):             # <<<<<<<<<<<<<<
 *     cdef char* c_str1
 *     if PyBytes_Check(o1):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_WriteUnraisable("ssw.c_util.obj_to_cstr_len", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "c_util.pxd":203
 * 
 * 
 * cdef inline char* copy_string(char* in_str, int length):             # <<<<<<<<<<<<<<
 *     cdef int i
 *     cdef char * out_str = <char *> PyMem_Malloc((length+1)*sizeof(char))
 */

static CYTHON_INLINE char *__pyx_f_3ssw_6c_util_copy_string(char *__pyx_v_in_str, int __pyx_v_length) {
  char *__pyx_v_out_str;
  char *__pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("copy_string", 0);

  /* "c_util.pxd":205
 * cdef inline char* copy_string(char* in_str, int length):
 *     cdef int i
 *     cdef char * out_str = <char *> PyMem_Malloc((length+1)*sizeof(char))             # <<<<<<<<<<<<<<
 *     if out_str == NULL:
 *         raise OSError("Could not allocate memory for sequence.")
 */
  __pyx_v_out_str = ((char *)PyMem_Malloc(((__pyx_v_length + 1) * (sizeof(char)))));

  /* "c_util.pxd":206
 *     cdef int i
 *     cdef char * out_str = <char *> PyMem_Malloc((length+1)*sizeof(char))
 *     if out_str == NULL:             # <<<<<<<<<<<<<<
 *         raise OSError("Could not allocate memory for sequence.")
 *     memcpy(out_str, in_str, length+1)
 */
  __pyx_t_1 = ((__pyx_v_out_str == NULL) != 0);
  if (__pyx_t_1) {

    /* "c_util.pxd":207
 *     cdef char * out_str = <char *> PyMem_Malloc((length+1)*sizeof(char))
 *     if out_str == NULL:
 *         raise OSError("Could not allocate memory for sequence.")             # <<<<<<<<<<<<<<
 *     memcpy(out_str, in_str, length+1)
 *     return out_str
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_OSError, __pyx_tuple__20, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 207, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(2, 207, __pyx_L1_error)

    /* "c_util.pxd":206
 *     cdef int i
 *     cdef char * out_str = <char *> PyMem_Malloc((length+1)*sizeof(char))
 *     if out_str == NULL:             # <<<<<<<<<<<<<<
 *         raise OSError("Could not allocate memory for sequence.")
 *     memcpy(out_str, in_str, length+1)
 */
  }

  /* "c_util.pxd":208
 *     if out_str == NULL:
 *         raise OSError("Could not allocate memory for sequence.")
 *     memcpy(out_str, in_str, length+1)             # <<<<<<<<<<<<<<
 *     return out_str
 * 
 */
  memcpy(__pyx_v_out_str, __pyx_v_in_str, (__pyx_v_length + 1));

  /* "c_util.pxd":209
 *         raise OSError("Could not allocate memory for sequence.")
 *     memcpy(out_str, in_str, length+1)
 *     return out_str             # <<<<<<<<<<<<<<
 * 
 * cdef inline void copy_string_buffer(char* in_str, char* out_str, int length):
 */
  __pyx_r = __pyx_v_out_str;
  goto __pyx_L0;

  /* "c_util.pxd":203
 * 
 * 
 * cdef inline char* copy_string(char* in_str, int length):             # <<<<<<<<<<<<<<
 *     cdef int i
 *     cdef char * out_str = <char *> PyMem_Malloc((length+1)*sizeof(char))
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_WriteUnraisable("ssw.c_util.copy_string", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "c_util.pxd":211
 *     return out_str
 * 
 * cdef inline void copy_string_buffer(char* in_str, char* out_str, int length):             # <<<<<<<<<<<<<<
 *     memcpy(out_str, in_str, length+1)
 * 
 */

static CYTHON_INLINE void __pyx_f_3ssw_6c_util_copy_string_buffer(char *__pyx_v_in_str, char *__pyx_v_out_str, int __pyx_v_length) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("copy_string_buffer", 0);

  /* "c_util.pxd":212
 * 
 * cdef inline void copy_string_buffer(char* in_str, char* out_str, int length):
 *     memcpy(out_str, in_str, length+1)             # <<<<<<<<<<<<<<
 * 
 */
  memcpy(__pyx_v_out_str, __pyx_v_in_str, (__pyx_v_length + 1));

  /* "c_util.pxd":211
 *     return out_str
 * 
 * cdef inline void copy_string_buffer(char* in_str, char* out_str, int length):             # <<<<<<<<<<<<<<
 *     memcpy(out_str, in_str, length+1)
 * 
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}
static struct __pyx_vtabstruct_3ssw_5sswpy_SSW __pyx_vtable_3ssw_5sswpy_SSW;

static PyObject *__pyx_tp_new_3ssw_5sswpy_SSW(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_3ssw_5sswpy_SSW *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_3ssw_5sswpy_SSW *)o);
  p->__pyx_vtab = __pyx_vtabptr_3ssw_5sswpy_SSW;
  p->read = Py_None; Py_INCREF(Py_None);
  p->reference = Py_None; Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_3ssw_5sswpy_3SSW_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_3ssw_5sswpy_SSW(PyObject *o) {
  struct __pyx_obj_3ssw_5sswpy_SSW *p = (struct __pyx_obj_3ssw_5sswpy_SSW *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    ++Py_REFCNT(o);
    __pyx_pw_3ssw_5sswpy_3SSW_5__dealloc__(o);
    --Py_REFCNT(o);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->read);
  Py_CLEAR(p->reference);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_3ssw_5sswpy_SSW(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_3ssw_5sswpy_SSW *p = (struct __pyx_obj_3ssw_5sswpy_SSW *)o;
  if (p->read) {
    e = (*v)(p->read, a); if (e) return e;
  }
  if (p->reference) {
    e = (*v)(p->reference, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_3ssw_5sswpy_SSW(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_3ssw_5sswpy_SSW *p = (struct __pyx_obj_3ssw_5sswpy_SSW *)o;
  tmp = ((PyObject*)p->read);
  p->read = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->reference);
  p->reference = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_3ssw_5sswpy_SSW[] = {
  {"printResult", (PyCFunction)__pyx_pw_3ssw_5sswpy_3SSW_7printResult, METH_VARARGS|METH_KEYWORDS, __pyx_doc_3ssw_5sswpy_3SSW_6printResult},
  {"setRead", (PyCFunction)__pyx_pw_3ssw_5sswpy_3SSW_9setRead, METH_O, __pyx_doc_3ssw_5sswpy_3SSW_8setRead},
  {"setReference", (PyCFunction)__pyx_pw_3ssw_5sswpy_3SSW_11setReference, METH_O, __pyx_doc_3ssw_5sswpy_3SSW_10setReference},
  {"align", (PyCFunction)__pyx_pw_3ssw_5sswpy_3SSW_13align, METH_VARARGS|METH_KEYWORDS, __pyx_doc_3ssw_5sswpy_3SSW_12align},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_3ssw_5sswpy_3SSW_15__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_3ssw_5sswpy_3SSW_17__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_3ssw_5sswpy_SSW = {
  PyVarObject_HEAD_INIT(0, 0)
  "ssw.sswpy.SSW", /*tp_name*/
  sizeof(struct __pyx_obj_3ssw_5sswpy_SSW), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_3ssw_5sswpy_SSW, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_3ssw_5sswpy_SSW, /*tp_traverse*/
  __pyx_tp_clear_3ssw_5sswpy_SSW, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_3ssw_5sswpy_SSW, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_3ssw_5sswpy_3SSW_3__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_3ssw_5sswpy_SSW, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_sswpy(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_sswpy},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "sswpy",
    0, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_n_s_Alignment, __pyx_k_Alignment, sizeof(__pyx_k_Alignment), 0, 0, 1, 1},
  {&__pyx_n_s_CIGAR, __pyx_k_CIGAR, sizeof(__pyx_k_CIGAR), 0, 0, 1, 1},
  {&__pyx_kp_s_Could_not_allocate_memory_for_se, __pyx_k_Could_not_allocate_memory_for_se, sizeof(__pyx_k_Could_not_allocate_memory_for_se), 0, 0, 1, 0},
  {&__pyx_n_s_MemoryError, __pyx_k_MemoryError, sizeof(__pyx_k_MemoryError), 0, 0, 1, 1},
  {&__pyx_kp_s_Must_set_profile_first, __pyx_k_Must_set_profile_first, sizeof(__pyx_k_Must_set_profile_first), 0, 0, 1, 0},
  {&__pyx_n_s_NamedTuple, __pyx_k_NamedTuple, sizeof(__pyx_k_NamedTuple), 0, 0, 1, 1},
  {&__pyx_kp_s_No_solution_found, __pyx_k_No_solution_found, sizeof(__pyx_k_No_solution_found), 0, 0, 1, 0},
  {&__pyx_n_s_OSError, __pyx_k_OSError, sizeof(__pyx_k_OSError), 0, 0, 1, 1},
  {&__pyx_kp_s_Out_of_Memory, __pyx_k_Out_of_Memory, sizeof(__pyx_k_Out_of_Memory), 0, 0, 1, 0},
  {&__pyx_kp_s_Problem_Running_alignment_see_st, __pyx_k_Problem_Running_alignment_see_st, sizeof(__pyx_k_Problem_Running_alignment_see_st), 0, 0, 1, 0},
  {&__pyx_kp_s_Read_does_not_align_to_one_overh, __pyx_k_Read_does_not_align_to_one_overh, sizeof(__pyx_k_Read_does_not_align_to_one_overh), 0, 0, 1, 0},
  {&__pyx_n_s_STR_T, __pyx_k_STR_T, sizeof(__pyx_k_STR_T), 0, 0, 1, 1},
  {&__pyx_n_s_TypeError, __pyx_k_TypeError, sizeof(__pyx_k_TypeError), 0, 0, 1, 1},
  {&__pyx_n_s_Union, __pyx_k_Union, sizeof(__pyx_k_Union), 0, 0, 1, 1},
  {&__pyx_n_s_ValueError, __pyx_k_ValueError, sizeof(__pyx_k_ValueError), 0, 0, 1, 1},
  {&__pyx_kp_s__12, __pyx_k__12, sizeof(__pyx_k__12), 0, 0, 1, 0},
  {&__pyx_kp_s__7, __pyx_k__7, sizeof(__pyx_k__7), 0, 0, 1, 0},
  {&__pyx_n_s_a, __pyx_k_a, sizeof(__pyx_k_a), 0, 0, 1, 1},
  {&__pyx_n_s_align, __pyx_k_align, sizeof(__pyx_k_align), 0, 0, 1, 1},
  {&__pyx_n_s_aligner, __pyx_k_aligner, sizeof(__pyx_k_aligner), 0, 0, 1, 1},
  {&__pyx_n_s_alignment, __pyx_k_alignment, sizeof(__pyx_k_alignment), 0, 0, 1, 1},
  {&__pyx_n_s_buffer_read, __pyx_k_buffer_read, sizeof(__pyx_k_buffer_read), 0, 0, 1, 1},
  {&__pyx_n_s_buffer_ref, __pyx_k_buffer_ref, sizeof(__pyx_k_buffer_ref), 0, 0, 1, 1},
  {&__pyx_kp_s_call_setReference_first, __pyx_k_call_setReference_first, sizeof(__pyx_k_call_setReference_first), 0, 0, 1, 0},
  {&__pyx_n_s_chr, __pyx_k_chr, sizeof(__pyx_k_chr), 0, 0, 1, 1},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_kp_s_copy_obj_to_cstr, __pyx_k_copy_obj_to_cstr, sizeof(__pyx_k_copy_obj_to_cstr), 0, 0, 1, 0},
  {&__pyx_kp_s_d_s, __pyx_k_d_s, sizeof(__pyx_k_d_s), 0, 0, 1, 0},
  {&__pyx_n_s_do_print, __pyx_k_do_print, sizeof(__pyx_k_do_print), 0, 0, 1, 1},
  {&__pyx_n_s_encode, __pyx_k_encode, sizeof(__pyx_k_encode), 0, 0, 1, 1},
  {&__pyx_n_s_end, __pyx_k_end, sizeof(__pyx_k_end), 0, 0, 1, 1},
  {&__pyx_n_s_end_idx, __pyx_k_end_idx, sizeof(__pyx_k_end_idx), 0, 0, 1, 1},
  {&__pyx_n_s_file, __pyx_k_file, sizeof(__pyx_k_file), 0, 0, 1, 1},
  {&__pyx_n_s_force_align, __pyx_k_force_align, sizeof(__pyx_k_force_align), 0, 0, 1, 1},
  {&__pyx_n_s_force_overhang, __pyx_k_force_overhang, sizeof(__pyx_k_force_overhang), 0, 0, 1, 1},
  {&__pyx_n_s_format, __pyx_k_format, sizeof(__pyx_k_format), 0, 0, 1, 1},
  {&__pyx_n_s_format_force_align, __pyx_k_format_force_align, sizeof(__pyx_k_format_force_align), 0, 0, 1, 1},
  {&__pyx_n_s_gap_extension, __pyx_k_gap_extension, sizeof(__pyx_k_gap_extension), 0, 0, 1, 1},
  {&__pyx_n_s_gap_open, __pyx_k_gap_open, sizeof(__pyx_k_gap_open), 0, 0, 1, 1},
  {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_len_x, __pyx_k_len_x, sizeof(__pyx_k_len_x), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_match_score, __pyx_k_match_score, sizeof(__pyx_k_match_score), 0, 0, 1, 1},
  {&__pyx_n_s_mismatch_penalty, __pyx_k_mismatch_penalty, sizeof(__pyx_k_mismatch_penalty), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_kp_s_negative_indexing_not_supported, __pyx_k_negative_indexing_not_supported, sizeof(__pyx_k_negative_indexing_not_supported), 0, 0, 1, 0},
  {&__pyx_kp_s_no_default___reduce___due_to_non, __pyx_k_no_default___reduce___due_to_non, sizeof(__pyx_k_no_default___reduce___due_to_non), 0, 0, 1, 0},
  {&__pyx_kp_s_obj_to_cstr, __pyx_k_obj_to_cstr, sizeof(__pyx_k_obj_to_cstr), 0, 0, 1, 0},
  {&__pyx_kp_s_obj_to_cstr_PyBytes_AsStringAndS, __pyx_k_obj_to_cstr_PyBytes_AsStringAndS, sizeof(__pyx_k_obj_to_cstr_PyBytes_AsStringAndS), 0, 0, 1, 0},
  {&__pyx_kp_s_obj_to_cstr_PyUnicode_AsUTF8AndS, __pyx_k_obj_to_cstr_PyUnicode_AsUTF8AndS, sizeof(__pyx_k_obj_to_cstr_PyUnicode_AsUTF8AndS), 0, 0, 1, 0},
  {&__pyx_n_s_optimal_score, __pyx_k_optimal_score, sizeof(__pyx_k_optimal_score), 0, 0, 1, 1},
  {&__pyx_n_s_print, __pyx_k_print, sizeof(__pyx_k_print), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
  {&__pyx_n_s_read, __pyx_k_read, sizeof(__pyx_k_read), 0, 0, 1, 1},
  {&__pyx_n_s_read_end, __pyx_k_read_end, sizeof(__pyx_k_read_end), 0, 0, 1, 1},
  {&__pyx_n_s_read_out, __pyx_k_read_out, sizeof(__pyx_k_read_out), 0, 0, 1, 1},
  {&__pyx_n_s_read_start, __pyx_k_read_start, sizeof(__pyx_k_read_start), 0, 0, 1, 1},
  {&__pyx_n_s_reduce, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_cython, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_ex, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
  {&__pyx_n_s_ref_out, __pyx_k_ref_out, sizeof(__pyx_k_ref_out), 0, 0, 1, 1},
  {&__pyx_n_s_reference, __pyx_k_reference, sizeof(__pyx_k_reference), 0, 0, 1, 1},
  {&__pyx_n_s_reference_end, __pyx_k_reference_end, sizeof(__pyx_k_reference_end), 0, 0, 1, 1},
  {&__pyx_n_s_reference_start, __pyx_k_reference_start, sizeof(__pyx_k_reference_start), 0, 0, 1, 1},
  {&__pyx_n_s_res, __pyx_k_res, sizeof(__pyx_k_res), 0, 0, 1, 1},
  {&__pyx_n_s_result, __pyx_k_result, sizeof(__pyx_k_result), 0, 0, 1, 1},
  {&__pyx_n_s_setRead, __pyx_k_setRead, sizeof(__pyx_k_setRead), 0, 0, 1, 1},
  {&__pyx_n_s_setReference, __pyx_k_setReference, sizeof(__pyx_k_setReference), 0, 0, 1, 1},
  {&__pyx_n_s_setstate, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_setstate_cython, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
  {&__pyx_n_s_ssw_sswpy, __pyx_k_ssw_sswpy, sizeof(__pyx_k_ssw_sswpy), 0, 0, 1, 1},
  {&__pyx_kp_s_ssw_sswpy_pyx, __pyx_k_ssw_sswpy_pyx, sizeof(__pyx_k_ssw_sswpy_pyx), 0, 0, 1, 0},
  {&__pyx_n_s_start_idx, __pyx_k_start_idx, sizeof(__pyx_k_start_idx), 0, 0, 1, 1},
  {&__pyx_kp_s_start_idx_or_end_idx_can_t_be_gr, __pyx_k_start_idx_or_end_idx_can_t_be_gr, sizeof(__pyx_k_start_idx_or_end_idx_can_t_be_gr), 0, 0, 1, 0},
  {&__pyx_n_s_start_read, __pyx_k_start_read, sizeof(__pyx_k_start_read), 0, 0, 1, 1},
  {&__pyx_n_s_start_ref, __pyx_k_start_ref, sizeof(__pyx_k_start_ref), 0, 0, 1, 1},
  {&__pyx_n_s_sub_optimal_score, __pyx_k_sub_optimal_score, sizeof(__pyx_k_sub_optimal_score), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_typing, __pyx_k_typing, sizeof(__pyx_k_typing), 0, 0, 1, 1},
  {&__pyx_n_s_utf8, __pyx_k_utf8, sizeof(__pyx_k_utf8), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_MemoryError = __Pyx_GetBuiltinName(__pyx_n_s_MemoryError); if (!__pyx_builtin_MemoryError) __PYX_ERR(0, 154, __pyx_L1_error)
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) __PYX_ERR(0, 161, __pyx_L1_error)
  __pyx_builtin_ValueError = __Pyx_GetBuiltinName(__pyx_n_s_ValueError); if (!__pyx_builtin_ValueError) __PYX_ERR(0, 256, __pyx_L1_error)
  __pyx_builtin_chr = __Pyx_GetBuiltinName(__pyx_n_s_chr); if (!__pyx_builtin_chr) __PYX_ERR(0, 324, __pyx_L1_error)
  __pyx_builtin_TypeError = __Pyx_GetBuiltinName(__pyx_n_s_TypeError); if (!__pyx_builtin_TypeError) __PYX_ERR(1, 2, __pyx_L1_error)
  __pyx_builtin_OSError = __Pyx_GetBuiltinName(__pyx_n_s_OSError); if (!__pyx_builtin_OSError) __PYX_ERR(2, 62, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "ssw/sswpy.pyx":154
 *             res_align = <s_align*> PyMem_Malloc(sizeof(s_align))
 *             if res_align == NULL:
 *                 raise MemoryError('Out of Memory')             # <<<<<<<<<<<<<<
 *             cigar = result.CIGAR
 *             if cigar is not None:
 */
  __pyx_tuple_ = PyTuple_Pack(1, __pyx_kp_s_Out_of_Memory); if (unlikely(!__pyx_tuple_)) __PYX_ERR(0, 154, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);

  /* "ssw/sswpy.pyx":159
 *                 cigar_array = <uint32_t*> PyMem_Malloc(len(cigar)//2*sizeof(uint32_t))
 *                 if cigar_array == NULL:
 *                     raise MemoryError('Out of Memory')             # <<<<<<<<<<<<<<
 *                 j = 0
 *                 for i in range(0, len(cigar), 2):
 */
  __pyx_tuple__2 = PyTuple_Pack(1, __pyx_kp_s_Out_of_Memory); if (unlikely(!__pyx_tuple__2)) __PYX_ERR(0, 159, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);

  /* "ssw/sswpy.pyx":256
 *                                 1, 0, 0, mask_len)
 *         else:
 *             raise ValueError("Must set profile first")             # <<<<<<<<<<<<<<
 *         if result == NULL:
 *             raise ValueError("Problem Running alignment, see stdout")
 */
  __pyx_tuple__3 = PyTuple_Pack(1, __pyx_kp_s_Must_set_profile_first); if (unlikely(!__pyx_tuple__3)) __PYX_ERR(0, 256, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);

  /* "ssw/sswpy.pyx":258
 *             raise ValueError("Must set profile first")
 *         if result == NULL:
 *             raise ValueError("Problem Running alignment, see stdout")             # <<<<<<<<<<<<<<
 *         return result
 *     # end def
 */
  __pyx_tuple__4 = PyTuple_Pack(1, __pyx_kp_s_Problem_Running_alignment_see_st); if (unlikely(!__pyx_tuple__4)) __PYX_ERR(0, 258, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);

  /* "ssw/sswpy.pyx":299
 * 
 *         if start_idx < 0 or end_idx < 0:
 *             raise ValueError("negative indexing not supported")             # <<<<<<<<<<<<<<
 *         if end_idx > self.ref_length or start_idx > self.ref_length:
 *             err = "start_idx: {} or end_idx: {} can't be greater than ref_length: {}".format(
 */
  __pyx_tuple__5 = PyTuple_Pack(1, __pyx_kp_s_negative_indexing_not_supported); if (unlikely(!__pyx_tuple__5)) __PYX_ERR(0, 299, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__5);
  __Pyx_GIVEREF(__pyx_tuple__5);

  /* "ssw/sswpy.pyx":315
 * 
 *         if self.reference is None:
 *             raise ValueError("call setReference first")             # <<<<<<<<<<<<<<
 * 
 *         cdef s_align* result =  self.align_c(gap_open, gap_extension, start_idx, search_length)
 */
  __pyx_tuple__6 = PyTuple_Pack(1, __pyx_kp_s_call_setReference_first); if (unlikely(!__pyx_tuple__6)) __PYX_ERR(0, 315, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__8 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__8)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__8);
  __Pyx_GIVEREF(__pyx_tuple__8);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__9 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__9)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__9);
  __Pyx_GIVEREF(__pyx_tuple__9);

  /* "ssw/sswpy.pyx":396
 *     res: Alignment = a.align(gap_open=len_x)
 *     if res.optimal_score < 4:
 *         raise ValueError("No solution found")             # <<<<<<<<<<<<<<
 *     if force_overhang:
 *         # read must align to either the beginning or end of the reference string
 */
  __pyx_tuple__10 = PyTuple_Pack(1, __pyx_kp_s_No_solution_found); if (unlikely(!__pyx_tuple__10)) __PYX_ERR(0, 396, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__10);
  __Pyx_GIVEREF(__pyx_tuple__10);

  /* "ssw/sswpy.pyx":401
 *         if (res.reference_start != 0 or
 *             res.reference_end != len(reference) - 1):
 *             raise ValueError("Read does not align to one overhang")             # <<<<<<<<<<<<<<
 *     return res
 * # end def
 */
  __pyx_tuple__11 = PyTuple_Pack(1, __pyx_kp_s_Read_does_not_align_to_one_overh); if (unlikely(!__pyx_tuple__11)) __PYX_ERR(0, 401, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__11);
  __Pyx_GIVEREF(__pyx_tuple__11);

  /* "c_util.pxd":8
 *     if isinstance(s, str):
 *         # encode to the specific encoding used inside of the module
 *         return (<str>s).encode('utf8')             # <<<<<<<<<<<<<<
 *     else:
 *         return s
 */
  __pyx_tuple__13 = PyTuple_Pack(1, __pyx_n_s_utf8); if (unlikely(!__pyx_tuple__13)) __PYX_ERR(2, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__13);
  __Pyx_GIVEREF(__pyx_tuple__13);

  /* "c_util.pxd":53
 *     if PyBytes_Check(o1):
 *         if PyBytes_AsStringAndSize(o1, &(c_str1), length) == -1:
 *             raise TypeError("copy_obj_to_cstr:")             # <<<<<<<<<<<<<<
 *         b_length = length[0]
 *         o2 = PyBytes_FromStringAndSize(c_str1, b_length)
 */
  __pyx_tuple__14 = PyTuple_Pack(1, __pyx_kp_s_copy_obj_to_cstr); if (unlikely(!__pyx_tuple__14)) __PYX_ERR(2, 53, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__14);
  __Pyx_GIVEREF(__pyx_tuple__14);

  /* "c_util.pxd":62
 *         c_str1 = PyUnicode_AsUTF8AndSize(o1, length)
 *         if c_str1 == NULL:
 *             raise OSError("copy_obj_to_cstr:")             # <<<<<<<<<<<<<<
 *         b_length = length[0]
 *         o2 = PyUnicode_FromStringAndSize(<const char *>c_str1, b_length)
 */
  __pyx_tuple__15 = PyTuple_Pack(1, __pyx_kp_s_copy_obj_to_cstr); if (unlikely(!__pyx_tuple__15)) __PYX_ERR(2, 62, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__15);
  __Pyx_GIVEREF(__pyx_tuple__15);

  /* "c_util.pxd":177
 *     if PyBytes_Check(o1):
 *         if PyBytes_AsStringAndSize(o1, &(c_str1), &length) == -1:
 *             raise TypeError("obj_to_cstr:")             # <<<<<<<<<<<<<<
 *         return c_str1
 *     else:
 */
  __pyx_tuple__16 = PyTuple_Pack(1, __pyx_kp_s_obj_to_cstr); if (unlikely(!__pyx_tuple__16)) __PYX_ERR(2, 177, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__16);
  __Pyx_GIVEREF(__pyx_tuple__16);

  /* "c_util.pxd":182
 *         c_str1 = PyUnicode_AsUTF8AndSize(o1, &length)
 *         if c_str1 == NULL:
 *             raise OSError("obj_to_cstr:")             # <<<<<<<<<<<<<<
 *     return c_str1
 * # end def
 */
  __pyx_tuple__17 = PyTuple_Pack(1, __pyx_kp_s_obj_to_cstr); if (unlikely(!__pyx_tuple__17)) __PYX_ERR(2, 182, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__17);
  __Pyx_GIVEREF(__pyx_tuple__17);

  /* "c_util.pxd":193
 *     if PyBytes_Check(o1):
 *         if PyBytes_AsStringAndSize(o1, &(c_str1), length) == -1:
 *             raise TypeError("obj_to_cstr: PyBytes_AsStringAndSize error")             # <<<<<<<<<<<<<<
 *         return c_str1
 *     else:
 */
  __pyx_tuple__18 = PyTuple_Pack(1, __pyx_kp_s_obj_to_cstr_PyBytes_AsStringAndS); if (unlikely(!__pyx_tuple__18)) __PYX_ERR(2, 193, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__18);
  __Pyx_GIVEREF(__pyx_tuple__18);

  /* "c_util.pxd":198
 *         c_str1 = PyUnicode_AsUTF8AndSize(o1, length)
 *         if c_str1 == NULL:
 *             raise OSError("obj_to_cstr: PyUnicode_AsUTF8AndSize error")             # <<<<<<<<<<<<<<
 *     return c_str1
 * # end def
 */
  __pyx_tuple__19 = PyTuple_Pack(1, __pyx_kp_s_obj_to_cstr_PyUnicode_AsUTF8AndS); if (unlikely(!__pyx_tuple__19)) __PYX_ERR(2, 198, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__19);
  __Pyx_GIVEREF(__pyx_tuple__19);

  /* "c_util.pxd":207
 *     cdef char * out_str = <char *> PyMem_Malloc((length+1)*sizeof(char))
 *     if out_str == NULL:
 *         raise OSError("Could not allocate memory for sequence.")             # <<<<<<<<<<<<<<
 *     memcpy(out_str, in_str, length+1)
 *     return out_str
 */
  __pyx_tuple__20 = PyTuple_Pack(1, __pyx_kp_s_Could_not_allocate_memory_for_se); if (unlikely(!__pyx_tuple__20)) __PYX_ERR(2, 207, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__20);
  __Pyx_GIVEREF(__pyx_tuple__20);

  /* "ssw/sswpy.pyx":374
 * # end class
 * 
 * def force_align( read: STR_T,             # <<<<<<<<<<<<<<
 *                 reference: STR_T,
 *                 force_overhang: bool = False,
 */
  __pyx_tuple__21 = PyTuple_Pack(7, __pyx_n_s_read, __pyx_n_s_reference, __pyx_n_s_force_overhang, __pyx_n_s_aligner, __pyx_n_s_a, __pyx_n_s_len_x, __pyx_n_s_res); if (unlikely(!__pyx_tuple__21)) __PYX_ERR(0, 374, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__21);
  __Pyx_GIVEREF(__pyx_tuple__21);
  __pyx_codeobj__22 = (PyObject*)__Pyx_PyCode_New(4, 0, 7, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__21, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ssw_sswpy_pyx, __pyx_n_s_force_align, 374, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__22)) __PYX_ERR(0, 374, __pyx_L1_error)

  /* "ssw/sswpy.pyx":405
 * # end def
 * 
 * def format_force_align(  read: STR_T,             # <<<<<<<<<<<<<<
 *                         reference: STR_T,
 *                         alignment: Alignment,
 */
  __pyx_tuple__23 = PyTuple_Pack(10, __pyx_n_s_read, __pyx_n_s_reference, __pyx_n_s_alignment, __pyx_n_s_do_print, __pyx_n_s_start_ref, __pyx_n_s_start_read, __pyx_n_s_buffer_ref, __pyx_n_s_buffer_read, __pyx_n_s_ref_out, __pyx_n_s_read_out); if (unlikely(!__pyx_tuple__23)) __PYX_ERR(0, 405, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__23);
  __Pyx_GIVEREF(__pyx_tuple__23);
  __pyx_codeobj__24 = (PyObject*)__Pyx_PyCode_New(4, 0, 10, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__23, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ssw_sswpy_pyx, __pyx_n_s_format_force_align, 405, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__24)) __PYX_ERR(0, 405, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 2, __pyx_L1_error);
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(0, 2, __pyx_L1_error)
  __pyx_int_4 = PyInt_FromLong(4); if (unlikely(!__pyx_int_4)) __PYX_ERR(0, 2, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

#if PY_MAJOR_VERSION < 3
PyMODINIT_FUNC initsswpy(void); /*proto*/
PyMODINIT_FUNC initsswpy(void)
#else
PyMODINIT_FUNC PyInit_sswpy(void); /*proto*/
PyMODINIT_FUNC PyInit_sswpy(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        result = PyDict_SetItemString(moddict, to_name, value);
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__") < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static int __pyx_pymod_exec_sswpy(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_t_11;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m && __pyx_m == __pyx_pyinit_module) return 0;
  #endif
  #if CYTHON_REFNANNY
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
  if (!__Pyx_RefNanny) {
      PyErr_Clear();
      __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
      if (!__Pyx_RefNanny)
          Py_FatalError("failed to import 'refnanny' module");
  }
  #endif
  __Pyx_RefNannySetupContext("PyMODINIT_FUNC PyInit_sswpy(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 2, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 2, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 2, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("sswpy", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 2, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 2, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 2, __pyx_L1_error)
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 2, __pyx_L1_error)
  #if CYTHON_COMPILING_IN_PYPY
  Py_INCREF(__pyx_b);
  #endif
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 2, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_ssw__sswpy) {
    if (PyObject_SetAttrString(__pyx_m, "__name__", __pyx_n_s_main) < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 2, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "ssw.sswpy")) {
      if (unlikely(PyDict_SetItemString(modules, "ssw.sswpy", __pyx_m) < 0)) __PYX_ERR(0, 2, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  /*--- Global init code ---*/
  /*--- Variable export code ---*/
  /*--- Function export code ---*/
  /*--- Type init code ---*/
  __pyx_vtabptr_3ssw_5sswpy_SSW = &__pyx_vtable_3ssw_5sswpy_SSW;
  __pyx_vtable_3ssw_5sswpy_SSW.printResult_c = (int (*)(struct __pyx_obj_3ssw_5sswpy_SSW *, s_align *, Py_ssize_t))__pyx_f_3ssw_5sswpy_3SSW_printResult_c;
  __pyx_vtable_3ssw_5sswpy_SSW.align_c = (s_align *(*)(struct __pyx_obj_3ssw_5sswpy_SSW *, int, int, Py_ssize_t, int32_t))__pyx_f_3ssw_5sswpy_3SSW_align_c;
  __pyx_vtable_3ssw_5sswpy_SSW.buildDNAScoreMatrix = (int (*)(struct __pyx_obj_3ssw_5sswpy_SSW *, uint8_t const , uint8_t const , int8_t *))__pyx_f_3ssw_5sswpy_3SSW_buildDNAScoreMatrix;
  if (PyType_Ready(&__pyx_type_3ssw_5sswpy_SSW) < 0) __PYX_ERR(0, 66, __pyx_L1_error)
  __pyx_type_3ssw_5sswpy_SSW.tp_print = 0;
  #if CYTHON_COMPILING_IN_CPYTHON
  {
    PyObject *wrapper = PyObject_GetAttrString((PyObject *)&__pyx_type_3ssw_5sswpy_SSW, "__init__"); if (unlikely(!wrapper)) __PYX_ERR(0, 66, __pyx_L1_error)
    if (Py_TYPE(wrapper) == &PyWrapperDescr_Type) {
      __pyx_wrapperbase_3ssw_5sswpy_3SSW_2__init__ = *((PyWrapperDescrObject *)wrapper)->d_base;
      __pyx_wrapperbase_3ssw_5sswpy_3SSW_2__init__.doc = __pyx_doc_3ssw_5sswpy_3SSW_2__init__;
      ((PyWrapperDescrObject *)wrapper)->d_base = &__pyx_wrapperbase_3ssw_5sswpy_3SSW_2__init__;
    }
  }
  #endif
  if (__Pyx_SetVtable(__pyx_type_3ssw_5sswpy_SSW.tp_dict, __pyx_vtabptr_3ssw_5sswpy_SSW) < 0) __PYX_ERR(0, 66, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "SSW", (PyObject *)&__pyx_type_3ssw_5sswpy_SSW) < 0) __PYX_ERR(0, 66, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_3ssw_5sswpy_SSW) < 0) __PYX_ERR(0, 66, __pyx_L1_error)
  __pyx_ptype_3ssw_5sswpy_SSW = &__pyx_type_3ssw_5sswpy_SSW;
  /*--- Type import code ---*/
  __pyx_ptype_7cpython_4type_type = __Pyx_ImportType(__Pyx_BUILTIN_MODULE_NAME, "type", 
  #if CYTHON_COMPILING_IN_PYPY
  sizeof(PyTypeObject),
  #else
  sizeof(PyHeapTypeObject),
  #endif
  0); if (unlikely(!__pyx_ptype_7cpython_4type_type)) __PYX_ERR(3, 9, __pyx_L1_error)
  /*--- Variable import code ---*/
  /*--- Function import code ---*/
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  #endif

  /* "ssw/sswpy.pyx":4
 * #cython: boundscheck=False, wraparound=False
 * from typing import (
 *     NamedTuple,             # <<<<<<<<<<<<<<
 *     Union
 * )
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_NamedTuple);
  __Pyx_GIVEREF(__pyx_n_s_NamedTuple);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_NamedTuple);
  __Pyx_INCREF(__pyx_n_s_Union);
  __Pyx_GIVEREF(__pyx_n_s_Union);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_s_Union);

  /* "ssw/sswpy.pyx":3
 * 
 * #cython: boundscheck=False, wraparound=False
 * from typing import (             # <<<<<<<<<<<<<<
 *     NamedTuple,
 *     Union
 */
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_typing, __pyx_t_1, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_NamedTuple); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_NamedTuple, __pyx_t_1) < 0) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_Union); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Union, __pyx_t_1) < 0) __PYX_ERR(0, 5, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "ssw/sswpy.pyx":52
 *     uint32_t to_cigar_int(uint32_t, char)
 * 
 * Alignment = NamedTuple("Alignment", [             # <<<<<<<<<<<<<<
 *         ('CIGAR', str),
 *         ('optimal_score', int),
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_NamedTuple); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 52, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);

  /* "ssw/sswpy.pyx":53
 * 
 * Alignment = NamedTuple("Alignment", [
 *         ('CIGAR', str),             # <<<<<<<<<<<<<<
 *         ('optimal_score', int),
 *         ('sub_optimal_score', int),
 */
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 53, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_n_s_CIGAR);
  __Pyx_GIVEREF(__pyx_n_s_CIGAR);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_n_s_CIGAR);
  __Pyx_INCREF(((PyObject *)(&PyString_Type)));
  __Pyx_GIVEREF(((PyObject *)(&PyString_Type)));
  PyTuple_SET_ITEM(__pyx_t_3, 1, ((PyObject *)(&PyString_Type)));

  /* "ssw/sswpy.pyx":54
 * Alignment = NamedTuple("Alignment", [
 *         ('CIGAR', str),
 *         ('optimal_score', int),             # <<<<<<<<<<<<<<
 *         ('sub_optimal_score', int),
 *         ('reference_start', int),
 */
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 54, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_n_s_optimal_score);
  __Pyx_GIVEREF(__pyx_n_s_optimal_score);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_n_s_optimal_score);
  __Pyx_INCREF(((PyObject *)(&PyInt_Type)));
  __Pyx_GIVEREF(((PyObject *)(&PyInt_Type)));
  PyTuple_SET_ITEM(__pyx_t_4, 1, ((PyObject *)(&PyInt_Type)));

  /* "ssw/sswpy.pyx":55
 *         ('CIGAR', str),
 *         ('optimal_score', int),
 *         ('sub_optimal_score', int),             # <<<<<<<<<<<<<<
 *         ('reference_start', int),
 *         ('reference_end', int),
 */
  __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 55, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(__pyx_n_s_sub_optimal_score);
  __Pyx_GIVEREF(__pyx_n_s_sub_optimal_score);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_n_s_sub_optimal_score);
  __Pyx_INCREF(((PyObject *)(&PyInt_Type)));
  __Pyx_GIVEREF(((PyObject *)(&PyInt_Type)));
  PyTuple_SET_ITEM(__pyx_t_5, 1, ((PyObject *)(&PyInt_Type)));

  /* "ssw/sswpy.pyx":56
 *         ('optimal_score', int),
 *         ('sub_optimal_score', int),
 *         ('reference_start', int),             # <<<<<<<<<<<<<<
 *         ('reference_end', int),
 *         ('read_start', int),
 */
  __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 56, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_INCREF(__pyx_n_s_reference_start);
  __Pyx_GIVEREF(__pyx_n_s_reference_start);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_n_s_reference_start);
  __Pyx_INCREF(((PyObject *)(&PyInt_Type)));
  __Pyx_GIVEREF(((PyObject *)(&PyInt_Type)));
  PyTuple_SET_ITEM(__pyx_t_6, 1, ((PyObject *)(&PyInt_Type)));

  /* "ssw/sswpy.pyx":57
 *         ('sub_optimal_score', int),
 *         ('reference_start', int),
 *         ('reference_end', int),             # <<<<<<<<<<<<<<
 *         ('read_start', int),
 *         ('read_end', int)
 */
  __pyx_t_7 = PyTuple_New(2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 57, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_INCREF(__pyx_n_s_reference_end);
  __Pyx_GIVEREF(__pyx_n_s_reference_end);
  PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_n_s_reference_end);
  __Pyx_INCREF(((PyObject *)(&PyInt_Type)));
  __Pyx_GIVEREF(((PyObject *)(&PyInt_Type)));
  PyTuple_SET_ITEM(__pyx_t_7, 1, ((PyObject *)(&PyInt_Type)));

  /* "ssw/sswpy.pyx":58
 *         ('reference_start', int),
 *         ('reference_end', int),
 *         ('read_start', int),             # <<<<<<<<<<<<<<
 *         ('read_end', int)
 *     ]
 */
  __pyx_t_8 = PyTuple_New(2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 58, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_INCREF(__pyx_n_s_read_start);
  __Pyx_GIVEREF(__pyx_n_s_read_start);
  PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_n_s_read_start);
  __Pyx_INCREF(((PyObject *)(&PyInt_Type)));
  __Pyx_GIVEREF(((PyObject *)(&PyInt_Type)));
  PyTuple_SET_ITEM(__pyx_t_8, 1, ((PyObject *)(&PyInt_Type)));

  /* "ssw/sswpy.pyx":59
 *         ('reference_end', int),
 *         ('read_start', int),
 *         ('read_end', int)             # <<<<<<<<<<<<<<
 *     ]
 * )
 */
  __pyx_t_9 = PyTuple_New(2); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 59, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_INCREF(__pyx_n_s_read_end);
  __Pyx_GIVEREF(__pyx_n_s_read_end);
  PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_n_s_read_end);
  __Pyx_INCREF(((PyObject *)(&PyInt_Type)));
  __Pyx_GIVEREF(((PyObject *)(&PyInt_Type)));
  PyTuple_SET_ITEM(__pyx_t_9, 1, ((PyObject *)(&PyInt_Type)));

  /* "ssw/sswpy.pyx":52
 *     uint32_t to_cigar_int(uint32_t, char)
 * 
 * Alignment = NamedTuple("Alignment", [             # <<<<<<<<<<<<<<
 *         ('CIGAR', str),
 *         ('optimal_score', int),
 */
  __pyx_t_10 = PyList_New(7); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 52, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_GIVEREF(__pyx_t_3);
  PyList_SET_ITEM(__pyx_t_10, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyList_SET_ITEM(__pyx_t_10, 1, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyList_SET_ITEM(__pyx_t_10, 2, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_6);
  PyList_SET_ITEM(__pyx_t_10, 3, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_7);
  PyList_SET_ITEM(__pyx_t_10, 4, __pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_8);
  PyList_SET_ITEM(__pyx_t_10, 5, __pyx_t_8);
  __Pyx_GIVEREF(__pyx_t_9);
  PyList_SET_ITEM(__pyx_t_10, 6, __pyx_t_9);
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_t_6 = 0;
  __pyx_t_7 = 0;
  __pyx_t_8 = 0;
  __pyx_t_9 = 0;
  __pyx_t_9 = NULL;
  __pyx_t_11 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_9)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_9);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
      __pyx_t_11 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_1)) {
    PyObject *__pyx_temp[3] = {__pyx_t_9, __pyx_n_s_Alignment, __pyx_t_10};
    __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_11, 2+__pyx_t_11); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 52, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
    PyObject *__pyx_temp[3] = {__pyx_t_9, __pyx_n_s_Alignment, __pyx_t_10};
    __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_11, 2+__pyx_t_11); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 52, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  } else
  #endif
  {
    __pyx_t_8 = PyTuple_New(2+__pyx_t_11); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 52, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    if (__pyx_t_9) {
      __Pyx_GIVEREF(__pyx_t_9); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_9); __pyx_t_9 = NULL;
    }
    __Pyx_INCREF(__pyx_n_s_Alignment);
    __Pyx_GIVEREF(__pyx_n_s_Alignment);
    PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_11, __pyx_n_s_Alignment);
    __Pyx_GIVEREF(__pyx_t_10);
    PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_11, __pyx_t_10);
    __pyx_t_10 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_8, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 52, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Alignment, __pyx_t_2) < 0) __PYX_ERR(0, 52, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "ssw/sswpy.pyx":63
 * )
 * 
 * STR_T = Union[str, bytes]             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_Union); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 63, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 63, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)(&PyString_Type)));
  __Pyx_GIVEREF(((PyObject *)(&PyString_Type)));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)(&PyString_Type)));
  __Pyx_INCREF(((PyObject *)(&PyBytes_Type)));
  __Pyx_GIVEREF(((PyObject *)(&PyBytes_Type)));
  PyTuple_SET_ITEM(__pyx_t_1, 1, ((PyObject *)(&PyBytes_Type)));
  __pyx_t_8 = PyObject_GetItem(__pyx_t_2, __pyx_t_1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 63, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_STR_T, __pyx_t_8) < 0) __PYX_ERR(0, 63, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "ssw/sswpy.pyx":374
 * # end class
 * 
 * def force_align( read: STR_T,             # <<<<<<<<<<<<<<
 *                 reference: STR_T,
 *                 force_overhang: bool = False,
 */
  __pyx_t_8 = PyCFunction_NewEx(&__pyx_mdef_3ssw_5sswpy_1force_align, NULL, __pyx_n_s_ssw_sswpy); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 374, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_force_align, __pyx_t_8) < 0) __PYX_ERR(0, 374, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "ssw/sswpy.pyx":405
 * # end def
 * 
 * def format_force_align(  read: STR_T,             # <<<<<<<<<<<<<<
 *                         reference: STR_T,
 *                         alignment: Alignment,
 */
  __pyx_t_8 = PyCFunction_NewEx(&__pyx_mdef_3ssw_5sswpy_3format_force_align, NULL, __pyx_n_s_ssw_sswpy); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 405, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_format_force_align, __pyx_t_8) < 0) __PYX_ERR(0, 405, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "ssw/sswpy.pyx":2
 * 
 * #cython: boundscheck=False, wraparound=False             # <<<<<<<<<<<<<<
 * from typing import (
 *     NamedTuple,
 */
  __pyx_t_8 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_8) < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "c_util.pxd":211
 *     return out_str
 * 
 * cdef inline void copy_string_buffer(char* in_str, char* out_str, int length):             # <<<<<<<<<<<<<<
 *     memcpy(out_str, in_str, length+1)
 * 
 */

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init ssw.sswpy", 0, __pyx_lineno, __pyx_filename);
    }
    Py_DECREF(__pyx_m); __pyx_m = 0;
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init ssw.sswpy");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule((char *)modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, (char *)"RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_CheckExact(key)) || likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (PyUnicode_GET_SIZE(**name) != PyUnicode_GET_SIZE(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (PyUnicode_GET_SIZE(**argname) != PyUnicode_GET_SIZE(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* RaiseException */
#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* None */
static CYTHON_INLINE Py_ssize_t __Pyx_div_Py_ssize_t(Py_ssize_t a, Py_ssize_t b) {
    Py_ssize_t q = a / b;
    Py_ssize_t r = a - q*b;
    q -= ((r != 0) & ((r ^ b) < 0));
    return q;
}

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely((0 <= wrapped_i) & (wrapped_i < PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely((0 <= wrapped_i) & (wrapped_i < PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely((n >= 0) & (n < PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely((n >= 0) & (n < PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* UnicodeAsUCS4 */
static CYTHON_INLINE Py_UCS4 __Pyx_PyUnicode_AsPy_UCS4(PyObject* x) {
   Py_ssize_t length;
   #if CYTHON_PEP393_ENABLED
   length = PyUnicode_GET_LENGTH(x);
   if (likely(length == 1)) {
       return PyUnicode_READ_CHAR(x, 0);
   }
   #else
   length = PyUnicode_GET_SIZE(x);
   if (likely(length == 1)) {
       return PyUnicode_AS_UNICODE(x)[0];
   }
   #if Py_UNICODE_SIZE == 2
   else if (PyUnicode_GET_SIZE(x) == 2) {
       Py_UCS4 high_val = PyUnicode_AS_UNICODE(x)[0];
       if (high_val >= 0xD800 && high_val <= 0xDBFF) {
           Py_UCS4 low_val = PyUnicode_AS_UNICODE(x)[1];
           if (low_val >= 0xDC00 && low_val <= 0xDFFF) {
               return 0x10000 + (((high_val & ((1<<10)-1)) << 10) | (low_val & ((1<<10)-1)));
           }
       }
   }
   #endif
   #endif
   PyErr_Format(PyExc_ValueError,
                "only single character unicode strings can be converted to Py_UCS4, "
                "got length %" CYTHON_FORMAT_SSIZE_T "d", length);
   return (Py_UCS4)-1;
}

/* object_ord */
static long __Pyx__PyObject_Ord(PyObject* c) {
    Py_ssize_t size;
    if (PyBytes_Check(c)) {
        size = PyBytes_GET_SIZE(c);
        if (likely(size == 1)) {
            return (unsigned char) PyBytes_AS_STRING(c)[0];
        }
#if PY_MAJOR_VERSION < 3
    } else if (PyUnicode_Check(c)) {
        return (long)__Pyx_PyUnicode_AsPy_UCS4(c);
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    } else if (PyByteArray_Check(c)) {
        size = PyByteArray_GET_SIZE(c);
        if (likely(size == 1)) {
            return (unsigned char) PyByteArray_AS_STRING(c)[0];
        }
#endif
    } else {
        PyErr_Format(PyExc_TypeError,
            "ord() expected string of length 1, but %.200s found", c->ob_type->tp_name);
        return (long)(Py_UCS4)-1;
    }
    PyErr_Format(PyExc_TypeError,
        "ord() expected a character, but string of length %zd found", size);
    return (long)(Py_UCS4)-1;
}

/* GetException */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb) {
#endif
    PyObject *local_type, *local_value, *local_tb;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
    #endif
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if PY_VERSION_HEX >= 0x030700A2
    tmp_type = tstate->exc_state.exc_type;
    tmp_value = tstate->exc_state.exc_value;
    tmp_tb = tstate->exc_state.exc_traceback;
    tstate->exc_state.exc_type = local_type;
    tstate->exc_state.exc_value = local_value;
    tstate->exc_state.exc_traceback = local_tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

/* SwapException */
  #if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if PY_VERSION_HEX >= 0x030700A2
    tmp_type = tstate->exc_state.exc_type;
    tmp_value = tstate->exc_state.exc_value;
    tmp_tb = tstate->exc_state.exc_traceback;
    tstate->exc_state.exc_type = *type;
    tstate->exc_state.exc_value = *value;
    tstate->exc_state.exc_traceback = *tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = *type;
    tstate->exc_value = *value;
    tstate->exc_traceback = *tb;
    #endif
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyErr_GetExcInfo(&tmp_type, &tmp_value, &tmp_tb);
    PyErr_SetExcInfo(*type, *value, *tb);
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#endif

/* None */
  static CYTHON_INLINE void __Pyx_RaiseUnboundLocalError(const char *varname) {
    PyErr_Format(PyExc_UnboundLocalError, "local variable '%s' referenced before assignment", varname);
}

/* SaveResetException */
  #if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    #if PY_VERSION_HEX >= 0x030700A2
    *type = tstate->exc_state.exc_type;
    *value = tstate->exc_state.exc_value;
    *tb = tstate->exc_state.exc_traceback;
    #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    #endif
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if PY_VERSION_HEX >= 0x030700A2
    tmp_type = tstate->exc_state.exc_type;
    tmp_value = tstate->exc_state.exc_value;
    tmp_tb = tstate->exc_state.exc_traceback;
    tstate->exc_state.exc_type = type;
    tstate->exc_state.exc_value = value;
    tstate->exc_state.exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
#endif

/* PyFunctionFastCall */
  #if CYTHON_FAST_PYCALL
#include "frameobject.h"
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = f->f_localsplus;
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyCFunctionFastCall */
  #if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)meth)) (self, args, nargs);
    }
}
#endif

/* GetModuleGlobalName */
  static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name) {
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
    result = PyDict_GetItem(__pyx_d, name);
    if (likely(result)) {
        Py_INCREF(result);
    } else {
#else
    result = PyObject_GetItem(__pyx_d, name);
    if (!result) {
        PyErr_Clear();
#endif
        result = __Pyx_GetBuiltinName(name);
    }
    return result;
}

/* ArgTypeTest */
    static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (exact) {
        #if PY_MAJOR_VERSION == 2
        if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    }
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
    return 0;
}

/* PyObjectCallMethO */
    #if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallOneArg */
    #if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (PyCFunction_GET_FLAGS(func) & METH_FASTCALL) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* decode_c_bytes */
    static CYTHON_INLINE PyObject* __Pyx_decode_c_bytes(
         const char* cstring, Py_ssize_t length, Py_ssize_t start, Py_ssize_t stop,
         const char* encoding, const char* errors,
         PyObject* (*decode_func)(const char *s, Py_ssize_t size, const char *errors)) {
    if (unlikely((start < 0) | (stop < 0))) {
        if (start < 0) {
            start += length;
            if (start < 0)
                start = 0;
        }
        if (stop < 0)
            stop += length;
    }
    if (stop > length)
        stop = length;
    length = stop - start;
    if (unlikely(length <= 0))
        return PyUnicode_FromUnicode(NULL, 0);
    cstring += start;
    if (decode_func) {
        return decode_func(cstring, length, errors);
    } else {
        return PyUnicode_Decode(cstring, length, encoding, errors);
    }
}

/* WriteUnraisableException */
    static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#ifdef _MSC_VER
    else state = (PyGILState_STATE)-1;
#endif
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

/* SetVTable */
    static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* SetupReduce */
    static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStr(meth, __pyx_n_s_name);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    if (_PyType_Lookup((PyTypeObject*)type_obj, __pyx_n_s_getstate)) goto GOOD;
#else
    if (PyObject_HasAttr(type_obj, __pyx_n_s_getstate)) goto GOOD;
#endif
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_ex); if (unlikely(!reduce_ex)) goto BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce); if (unlikely(!reduce)) goto BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_n_s_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_cython); if (unlikely(!reduce_cython)) goto BAD;
            ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce, reduce_cython); if (unlikely(ret < 0)) goto BAD;
            ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce_cython); if (unlikely(ret < 0)) goto BAD;
            setstate = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_n_s_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate_cython); if (unlikely(!setstate_cython)) goto BAD;
                ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate, setstate_cython); if (unlikely(ret < 0)) goto BAD;
                ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate_cython); if (unlikely(ret < 0)) goto BAD;
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto GOOD;
BAD:
    if (!PyErr_Occurred())
        PyErr_Format(PyExc_RuntimeError, "Unable to initialize pickling for %s", ((PyTypeObject*)type_obj)->tp_name);
    ret = -1;
GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* Import */
    static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if (strchr(__Pyx_MODULE_NAME, '.')) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* ImportFrom */
    static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* CLineInTraceback */
    #ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(CYTHON_UNUSED PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
      use_cline = PyDict_GetItem(*cython_runtime_dict, __pyx_n_s_cline_in_traceback);
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (PyObject_Not(use_cline) != 0) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
    static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, (size_t)new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
    #include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntFromPyVerify */
    #define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* Print */
    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION < 3
static PyObject *__Pyx_GetStdout(void) {
    PyObject *f = PySys_GetObject((char *)"stdout");
    if (!f) {
        PyErr_SetString(PyExc_RuntimeError, "lost sys.stdout");
    }
    return f;
}
static int __Pyx_Print(PyObject* f, PyObject *arg_tuple, int newline) {
    int i;
    if (!f) {
        if (!(f = __Pyx_GetStdout()))
            return -1;
    }
    Py_INCREF(f);
    for (i=0; i < PyTuple_GET_SIZE(arg_tuple); i++) {
        PyObject* v;
        if (PyFile_SoftSpace(f, 1)) {
            if (PyFile_WriteString(" ", f) < 0)
                goto error;
        }
        v = PyTuple_GET_ITEM(arg_tuple, i);
        if (PyFile_WriteObject(v, f, Py_PRINT_RAW) < 0)
            goto error;
        if (PyString_Check(v)) {
            char *s = PyString_AsString(v);
            Py_ssize_t len = PyString_Size(v);
            if (len > 0) {
                switch (s[len-1]) {
                    case ' ': break;
                    case '\f': case '\r': case '\n': case '\t': case '\v':
                        PyFile_SoftSpace(f, 0);
                        break;
                    default:  break;
                }
            }
        }
    }
    if (newline) {
        if (PyFile_WriteString("\n", f) < 0)
            goto error;
        PyFile_SoftSpace(f, 0);
    }
    Py_DECREF(f);
    return 0;
error:
    Py_DECREF(f);
    return -1;
}
#else
static int __Pyx_Print(PyObject* stream, PyObject *arg_tuple, int newline) {
    PyObject* kwargs = 0;
    PyObject* result = 0;
    PyObject* end_string;
    if (unlikely(!__pyx_print)) {
        __pyx_print = PyObject_GetAttr(__pyx_b, __pyx_n_s_print);
        if (!__pyx_print)
            return -1;
    }
    if (stream) {
        kwargs = PyDict_New();
        if (unlikely(!kwargs))
            return -1;
        if (unlikely(PyDict_SetItem(kwargs, __pyx_n_s_file, stream) < 0))
            goto bad;
        if (!newline) {
            end_string = PyUnicode_FromStringAndSize(" ", 1);
            if (unlikely(!end_string))
                goto bad;
            if (PyDict_SetItem(kwargs, __pyx_n_s_end, end_string) < 0) {
                Py_DECREF(end_string);
                goto bad;
            }
            Py_DECREF(end_string);
        }
    } else if (!newline) {
        if (unlikely(!__pyx_print_kwargs)) {
            __pyx_print_kwargs = PyDict_New();
            if (unlikely(!__pyx_print_kwargs))
                return -1;
            end_string = PyUnicode_FromStringAndSize(" ", 1);
            if (unlikely(!end_string))
                return -1;
            if (PyDict_SetItem(__pyx_print_kwargs, __pyx_n_s_end, end_string) < 0) {
                Py_DECREF(end_string);
                return -1;
            }
            Py_DECREF(end_string);
        }
        kwargs = __pyx_print_kwargs;
    }
    result = PyObject_Call(__pyx_print, arg_tuple, kwargs);
    if (unlikely(kwargs) && (kwargs != __pyx_print_kwargs))
        Py_DECREF(kwargs);
    if (!result)
        return -1;
    Py_DECREF(result);
    return 0;
bad:
    if (kwargs != __pyx_print_kwargs)
        Py_XDECREF(kwargs);
    return -1;
}
#endif

/* CIntToPy */
    static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int32_t(int32_t value) {
    const int32_t neg_one = (int32_t) -1, const_zero = (int32_t) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int32_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int32_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int32_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int32_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int32_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int32_t),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
    static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
    static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint16_t(uint16_t value) {
    const uint16_t neg_one = (uint16_t) -1, const_zero = (uint16_t) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(uint16_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(uint16_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(uint16_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(uint16_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(uint16_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(uint16_t),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
    static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* PrintOne */
    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION < 3
static int __Pyx_PrintOne(PyObject* f, PyObject *o) {
    if (!f) {
        if (!(f = __Pyx_GetStdout()))
            return -1;
    }
    Py_INCREF(f);
    if (PyFile_SoftSpace(f, 0)) {
        if (PyFile_WriteString(" ", f) < 0)
            goto error;
    }
    if (PyFile_WriteObject(o, f, Py_PRINT_RAW) < 0)
        goto error;
    if (PyFile_WriteString("\n", f) < 0)
        goto error;
    Py_DECREF(f);
    return 0;
error:
    Py_DECREF(f);
    return -1;
    /* the line below is just to avoid C compiler
     * warnings about unused functions */
    return __Pyx_Print(f, NULL, 0);
}
#else
static int __Pyx_PrintOne(PyObject* stream, PyObject *o) {
    int res;
    PyObject* arg_tuple = PyTuple_Pack(1, o);
    if (unlikely(!arg_tuple))
        return -1;
    res = __Pyx_Print(stream, arg_tuple, 1);
    Py_DECREF(arg_tuple);
    return res;
}
#endif

/* CIntFromPy */
    static CYTHON_INLINE uint32_t __Pyx_PyInt_As_uint32_t(PyObject *x) {
    const uint32_t neg_one = (uint32_t) -1, const_zero = (uint32_t) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(uint32_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(uint32_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (uint32_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint32_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(uint32_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(uint32_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) >= 2 * PyLong_SHIFT) {
                            return (uint32_t) (((((uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint32_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) >= 3 * PyLong_SHIFT) {
                            return (uint32_t) (((((((uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint32_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) >= 4 * PyLong_SHIFT) {
                            return (uint32_t) (((((((((uint32_t)digits[3]) << PyLong_SHIFT) | (uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (uint32_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(uint32_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint32_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(uint32_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint32_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint32_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(uint32_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(uint32_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(uint32_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) - 1 > 2 * PyLong_SHIFT) {
                            return (uint32_t) (((uint32_t)-1)*(((((uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(uint32_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) - 1 > 2 * PyLong_SHIFT) {
                            return (uint32_t) ((((((uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(uint32_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) - 1 > 3 * PyLong_SHIFT) {
                            return (uint32_t) (((uint32_t)-1)*(((((((uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint32_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) - 1 > 3 * PyLong_SHIFT) {
                            return (uint32_t) ((((((((uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(uint32_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) - 1 > 4 * PyLong_SHIFT) {
                            return (uint32_t) (((uint32_t)-1)*(((((((((uint32_t)digits[3]) << PyLong_SHIFT) | (uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint32_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) - 1 > 4 * PyLong_SHIFT) {
                            return (uint32_t) ((((((((((uint32_t)digits[3]) << PyLong_SHIFT) | (uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(uint32_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint32_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(uint32_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint32_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            uint32_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (uint32_t) -1;
        }
    } else {
        uint32_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (uint32_t) -1;
        val = __Pyx_PyInt_As_uint32_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to uint32_t");
    return (uint32_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to uint32_t");
    return (uint32_t) -1;
}

/* CIntFromPy */
    static CYTHON_INLINE uint16_t __Pyx_PyInt_As_uint16_t(PyObject *x) {
    const uint16_t neg_one = (uint16_t) -1, const_zero = (uint16_t) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(uint16_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(uint16_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (uint16_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint16_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(uint16_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(uint16_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint16_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint16_t) >= 2 * PyLong_SHIFT) {
                            return (uint16_t) (((((uint16_t)digits[1]) << PyLong_SHIFT) | (uint16_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint16_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint16_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint16_t) >= 3 * PyLong_SHIFT) {
                            return (uint16_t) (((((((uint16_t)digits[2]) << PyLong_SHIFT) | (uint16_t)digits[1]) << PyLong_SHIFT) | (uint16_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint16_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint16_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint16_t) >= 4 * PyLong_SHIFT) {
                            return (uint16_t) (((((((((uint16_t)digits[3]) << PyLong_SHIFT) | (uint16_t)digits[2]) << PyLong_SHIFT) | (uint16_t)digits[1]) << PyLong_SHIFT) | (uint16_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (uint16_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(uint16_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint16_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(uint16_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint16_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint16_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(uint16_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(uint16_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(uint16_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint16_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint16_t) - 1 > 2 * PyLong_SHIFT) {
                            return (uint16_t) (((uint16_t)-1)*(((((uint16_t)digits[1]) << PyLong_SHIFT) | (uint16_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(uint16_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint16_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint16_t) - 1 > 2 * PyLong_SHIFT) {
                            return (uint16_t) ((((((uint16_t)digits[1]) << PyLong_SHIFT) | (uint16_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(uint16_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint16_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint16_t) - 1 > 3 * PyLong_SHIFT) {
                            return (uint16_t) (((uint16_t)-1)*(((((((uint16_t)digits[2]) << PyLong_SHIFT) | (uint16_t)digits[1]) << PyLong_SHIFT) | (uint16_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint16_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint16_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint16_t) - 1 > 3 * PyLong_SHIFT) {
                            return (uint16_t) ((((((((uint16_t)digits[2]) << PyLong_SHIFT) | (uint16_t)digits[1]) << PyLong_SHIFT) | (uint16_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(uint16_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint16_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint16_t) - 1 > 4 * PyLong_SHIFT) {
                            return (uint16_t) (((uint16_t)-1)*(((((((((uint16_t)digits[3]) << PyLong_SHIFT) | (uint16_t)digits[2]) << PyLong_SHIFT) | (uint16_t)digits[1]) << PyLong_SHIFT) | (uint16_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint16_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint16_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint16_t) - 1 > 4 * PyLong_SHIFT) {
                            return (uint16_t) ((((((((((uint16_t)digits[3]) << PyLong_SHIFT) | (uint16_t)digits[2]) << PyLong_SHIFT) | (uint16_t)digits[1]) << PyLong_SHIFT) | (uint16_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(uint16_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint16_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(uint16_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint16_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            uint16_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (uint16_t) -1;
        }
    } else {
        uint16_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (uint16_t) -1;
        val = __Pyx_PyInt_As_uint16_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to uint16_t");
    return (uint16_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to uint16_t");
    return (uint16_t) -1;
}

/* CIntFromPy */
    static CYTHON_INLINE int32_t __Pyx_PyInt_As_int32_t(PyObject *x) {
    const int32_t neg_one = (int32_t) -1, const_zero = (int32_t) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int32_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int32_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int32_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int32_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int32_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(int32_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32_t) >= 2 * PyLong_SHIFT) {
                            return (int32_t) (((((int32_t)digits[1]) << PyLong_SHIFT) | (int32_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int32_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32_t) >= 3 * PyLong_SHIFT) {
                            return (int32_t) (((((((int32_t)digits[2]) << PyLong_SHIFT) | (int32_t)digits[1]) << PyLong_SHIFT) | (int32_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int32_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32_t) >= 4 * PyLong_SHIFT) {
                            return (int32_t) (((((((((int32_t)digits[3]) << PyLong_SHIFT) | (int32_t)digits[2]) << PyLong_SHIFT) | (int32_t)digits[1]) << PyLong_SHIFT) | (int32_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int32_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int32_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int32_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int32_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int32_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int32_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int32_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int32_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int32_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32_t) - 1 > 2 * PyLong_SHIFT) {
                            return (int32_t) (((int32_t)-1)*(((((int32_t)digits[1]) << PyLong_SHIFT) | (int32_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int32_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32_t) - 1 > 2 * PyLong_SHIFT) {
                            return (int32_t) ((((((int32_t)digits[1]) << PyLong_SHIFT) | (int32_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int32_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32_t) - 1 > 3 * PyLong_SHIFT) {
                            return (int32_t) (((int32_t)-1)*(((((((int32_t)digits[2]) << PyLong_SHIFT) | (int32_t)digits[1]) << PyLong_SHIFT) | (int32_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int32_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32_t) - 1 > 3 * PyLong_SHIFT) {
                            return (int32_t) ((((((((int32_t)digits[2]) << PyLong_SHIFT) | (int32_t)digits[1]) << PyLong_SHIFT) | (int32_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int32_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32_t) - 1 > 4 * PyLong_SHIFT) {
                            return (int32_t) (((int32_t)-1)*(((((((((int32_t)digits[3]) << PyLong_SHIFT) | (int32_t)digits[2]) << PyLong_SHIFT) | (int32_t)digits[1]) << PyLong_SHIFT) | (int32_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int32_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32_t) - 1 > 4 * PyLong_SHIFT) {
                            return (int32_t) ((((((((((int32_t)digits[3]) << PyLong_SHIFT) | (int32_t)digits[2]) << PyLong_SHIFT) | (int32_t)digits[1]) << PyLong_SHIFT) | (int32_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int32_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int32_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int32_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int32_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int32_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int32_t) -1;
        }
    } else {
        int32_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int32_t) -1;
        val = __Pyx_PyInt_As_int32_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int32_t");
    return (int32_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int32_t");
    return (int32_t) -1;
}

/* CIntToPy */
    static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
    static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* FastTypeChecks */
    #if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CheckBinaryVersion */
    static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* ModuleImport */
    #ifndef __PYX_HAVE_RT_ImportModule
#define __PYX_HAVE_RT_ImportModule
static PyObject *__Pyx_ImportModule(const char *name) {
    PyObject *py_name = 0;
    PyObject *py_module = 0;
    py_name = __Pyx_PyIdentifier_FromString(name);
    if (!py_name)
        goto bad;
    py_module = PyImport_Import(py_name);
    Py_DECREF(py_name);
    return py_module;
bad:
    Py_XDECREF(py_name);
    return 0;
}
#endif

/* TypeImport */
    #ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name,
    size_t size, int strict)
{
    PyObject *py_module = 0;
    PyObject *result = 0;
    PyObject *py_name = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    py_module = __Pyx_ImportModule(module_name);
    if (!py_module)
        goto bad;
    py_name = __Pyx_PyIdentifier_FromString(class_name);
    if (!py_name)
        goto bad;
    result = PyObject_GetAttr(py_module, py_name);
    Py_DECREF(py_name);
    py_name = 0;
    Py_DECREF(py_module);
    py_module = 0;
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (!strict && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. Expected %zd, got %zd",
            module_name, class_name, basicsize, size);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    else if ((size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s has the wrong size, try recompiling. Expected %zd, got %zd",
            module_name, class_name, basicsize, size);
        goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(py_module);
    Py_XDECREF(result);
    return NULL;
}
#endif

/* InitStrings */
    static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            PyErr_Clear();
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(x);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
